@@ -74,7 +74,7 @@ ROLE(CARET,
 ROLE(ALERT,
      "alert",
      ATK_ROLE_ALERT,
-     NSAccessibilityWindowRole,
+     NSAccessibilityGroupRole,
      ROLE_SYSTEM_ALERT,
      ROLE_SYSTEM_ALERT,
      eNoNameRule)
@@ -187,7 +187,7 @@ ROLE(GROUPING,
 ROLE(SEPARATOR,
      "separator",
      ATK_ROLE_SEPARATOR,
-     NSAccessibilityUnknownRole,
+     NSAccessibilitySplitterRole,
      ROLE_SYSTEM_SEPARATOR,
      ROLE_SYSTEM_SEPARATOR,
      eNoNameRule)
@@ -211,7 +211,7 @@ ROLE(STATUSBAR,
 ROLE(TABLE,
      "table",
      ATK_ROLE_TABLE,
-     NSAccessibilityGroupRole,
+     NSAccessibilityTableRole,
      ROLE_SYSTEM_TABLE,
      ROLE_SYSTEM_TABLE,
      eNoNameRule)
@@ -219,7 +219,7 @@ ROLE(TABLE,
 ROLE(COLUMNHEADER,
      "columnheader",
      ATK_ROLE_COLUMN_HEADER,
-     NSAccessibilityGroupRole,
+     NSAccessibilityCellRole,
      ROLE_SYSTEM_COLUMNHEADER,
      ROLE_SYSTEM_COLUMNHEADER,
      eNameFromSubtreeRule)
@@ -227,7 +227,7 @@ ROLE(COLUMNHEADER,
 ROLE(ROWHEADER,
      "rowheader",
      ATK_ROLE_ROW_HEADER,
-     NSAccessibilityGroupRole,
+     NSAccessibilityCellRole,
      ROLE_SYSTEM_ROWHEADER,
      ROLE_SYSTEM_ROWHEADER,
      eNameFromSubtreeRule)
@@ -251,7 +251,7 @@ ROLE(ROW,
 ROLE(CELL,
      "cell",
      ATK_ROLE_TABLE_CELL,
-     NSAccessibilityGroupRole,
+     NSAccessibilityCellRole,
      ROLE_SYSTEM_CELL,
      ROLE_SYSTEM_CELL,
      eNameFromSubtreeIfReqRule)
@@ -754,7 +754,7 @@ ROLE(TERMINAL,
 
 ROLE(TEXT_CONTAINER,
      "text container",
-     ATK_ROLE_TEXT,
+     ATK_ROLE_SECTION,
      NSAccessibilityGroupRole,
      USE_ROLE_STRING,
      IA2_ROLE_TEXT_FRAME,
@@ -955,7 +955,7 @@ ROLE(IMAGE_MAP,
 ROLE(OPTION,
      "listbox option",
      ATK_ROLE_LIST_ITEM,
-     NSAccessibilityRowRole,
+     NSAccessibilityStaticTextRole,
      ROLE_SYSTEM_LISTITEM,
      ROLE_SYSTEM_LISTITEM,
      eNameFromSubtreeRule)
@@ -1055,3 +1055,300 @@ ROLE(KEY,
      ROLE_SYSTEM_PUSHBUTTON,
      ROLE_SYSTEM_PUSHBUTTON,
      eNameFromSubtreeRule)
+
+ROLE(SWITCH,
+     "switch",
+     ATK_ROLE_TOGGLE_BUTTON,
+     NSAccessibilityCheckBoxRole,
+     ROLE_SYSTEM_CHECKBUTTON,
+     IA2_ROLE_TOGGLE_BUTTON,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_MATH,
+     "math",
+     ATK_ROLE_MATH,
+     NSAccessibilityGroupRole,
+     ROLE_SYSTEM_EQUATION,
+     ROLE_SYSTEM_EQUATION,
+     eNoNameRule)
+
+ROLE(MATHML_IDENTIFIER,
+     "mathml identifier",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_NUMBER,
+     "mathml number",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_OPERATOR,
+     "mathml operator",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_TEXT,
+     "mathml text",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_STRING_LITERAL,
+     "mathml string literal",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_GLYPH,
+     "mathml glyph",
+     ATK_ROLE_IMAGE,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNameFromSubtreeRule)
+
+ROLE(MATHML_ROW,
+     "mathml row",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_FRACTION,
+     "mathml fraction",
+     ATK_ROLE_MATH_FRACTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_SQUARE_ROOT,
+     "mathml square root",
+     ATK_ROLE_MATH_ROOT,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_ROOT,
+     "mathml root",
+     ATK_ROLE_MATH_ROOT,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_FENCED,
+     "mathml fenced",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_ENCLOSED,
+     "mathml enclosed",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STYLE,
+     "mathml style",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_SUB,
+     "mathml sub",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_SUP,
+     "mathml sup",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_SUB_SUP,
+     "mathml sub sup",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_UNDER,
+     "mathml under",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_OVER,
+     "mathml over",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_UNDER_OVER,
+     "mathml under over",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_MULTISCRIPTS,
+     "mathml multiscripts",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_TABLE,
+     "mathml table",
+     ATK_ROLE_TABLE,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_LABELED_ROW,
+     "mathml labeled row",
+     ATK_ROLE_TABLE_ROW,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_TABLE_ROW,
+     "mathml table row",
+     ATK_ROLE_TABLE_ROW,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_CELL,
+     "mathml cell",
+     ATK_ROLE_TABLE_CELL,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_ACTION,
+     "mathml action",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_ERROR,
+     "mathml error",
+     ATK_ROLE_SECTION,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK,
+     "mathml stack",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_LONG_DIVISION,
+     "mathml long division",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK_GROUP,
+     "mathml stack group",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK_ROW,
+     "mathml stack row",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK_CARRIES,
+     "mathml stack carries",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK_CARRY,
+     "mathml stack carry",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(MATHML_STACK_LINE,
+     "mathml stack line",
+     ATK_ROLE_UNKNOWN,
+     NSAccessibilityGroupRole,
+     0,
+     IA2_ROLE_UNKNOWN,
+     eNoNameRule)
+
+ROLE(RADIO_GROUP,
+     "grouping",
+     ATK_ROLE_PANEL,
+     NSAccessibilityRadioGroupRole,
+     ROLE_SYSTEM_GROUPING,
+     ROLE_SYSTEM_GROUPING,
+     eNoNameRule)
+
+ROLE(TEXT,
+     "text",
+     ATK_ROLE_STATIC,
+     NSAccessibilityGroupRole,
+     USE_ROLE_STRING,
+     IA2_ROLE_TEXT_FRAME,
+     eNameFromSubtreeIfReqRule)
+
