@@ -11,13 +11,16 @@ module.metadata = {
 // NOTE: This file should only deal with xul/native tabs
 
 
-const { Ci } = require('chrome');
+const { Ci, Cu } = require('chrome');
 const { defer } = require("../lang/functional");
 const { windows, isBrowser } = require('../window/utils');
 const { isPrivateBrowsingSupported } = require('../self');
+const { ShimWaiver } = Cu.import("resource://gre/modules/ShimWaiver.jsm");
 
 // Bug 834961: ignore private windows when they are not supported
-function getWindows() windows(null, { includePrivate: isPrivateBrowsingSupported });
+function getWindows() {
+  return windows(null, { includePrivate: isPrivateBrowsingSupported });
+}
 
 const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 
@@ -91,7 +94,7 @@ function getTabs(window) {
     return window.BrowserApp.tabs;
 
   // icecat - default
-  return Array.filter(getTabContainer(window).children, function(t) !t.closing);
+  return Array.filter(getTabContainer(window).children, t => !t.closing);
 }
 exports.getTabs = getTabs;
 
@@ -213,14 +216,25 @@ function getTabForId(id) {
 exports.getTabForId = getTabForId;
 
 function getTabTitle(tab) {
-  return getBrowserForTab(tab).contentDocument.title || tab.label || "";
+  return getBrowserForTab(tab).contentTitle || tab.label || "";
 }
 exports.getTabTitle = getTabTitle;
 
 function setTabTitle(tab, title) {
   title = String(title);
-  if (tab.browser)
+  if (tab.browser) {
+    // IceCatMobile
     tab.browser.contentDocument.title = title;
+  }
+  else {
+    let browser = getBrowserForTab(tab);
+    // Note that we aren't actually setting the document title in e10s, just
+    // the title the browser thinks the content has
+    if (browser.isRemoteBrowser)
+      browser._contentTitle = title;
+    else
+      browser.contentDocument.title = title;
+  }
   tab.label = String(title);
 }
 exports.setTabTitle = setTabTitle;
@@ -249,18 +263,24 @@ function getTabForContentWindow(window) {
 }
 exports.getTabForContentWindow = getTabForContentWindow;
 
+// only sdk/selection.js is relying on shims
+function getTabForContentWindowNoShim(window) {
+  function getTabContentWindowNoShim(tab) {
+    let browser = getBrowserForTab(tab);
+    return ShimWaiver.getProperty(browser, "contentWindow");
+  }
+  return getTabs().find(tab => getTabContentWindowNoShim(tab) === window.top) || null;
+}
+exports.getTabForContentWindowNoShim = getTabForContentWindowNoShim;
+
 function getTabURL(tab) {
-  if (tab.browser) // icecatmobile
-    return String(tab.browser.currentURI.spec);
   return String(getBrowserForTab(tab).currentURI.spec);
 }
 exports.getTabURL = getTabURL;
 
 function setTabURL(tab, url) {
-  url = String(url);
-  if (tab.browser)
-    return tab.browser.loadURI(url);
-  return getBrowserForTab(tab).loadURI(url);
+  let browser = getBrowserForTab(tab);
+  browser.loadURI(String(url));
 }
 // "TabOpen" event is fired when it's still "about:blank" is loaded in the
 // changing `location` property of the `contentDocument` has no effect since
@@ -294,7 +314,9 @@ function getTabForBrowser(browser) {
         return tab;
     }
   }
-  return null;
+
+  let tabbrowser = browser.getTabBrowser && browser.getTabBrowser()
+  return !!tabbrowser && tabbrowser.getTabForBrowser(browser);
 }
 exports.getTabForBrowser = getTabForBrowser;
 
@@ -312,15 +334,13 @@ function unpin(tab) {
 }
 exports.unpin = unpin;
 
-function isPinned(tab) !!tab.pinned
+function isPinned(tab) {
+  return !!tab.pinned;
+}
 exports.isPinned = isPinned;
 
 function reload(tab) {
-  let gBrowser = getTabBrowserForTab(tab);
-  // IceCat
-  if (gBrowser) gBrowser.unpinTab(tab);
-  // IceCatMobile
-  else if (tab.browser) tab.browser.reload();
+  getBrowserForTab(tab).reload();
 }
 exports.reload = reload
 
@@ -328,8 +348,7 @@ function getIndex(tab) {
   let gBrowser = getTabBrowserForTab(tab);
   // IceCat
   if (gBrowser) {
-    let document = getBrowserForTab(tab).contentDocument;
-    return gBrowser.getBrowserIndexForDocument(document);
+    return tab._tPos;
   }
   // IceCatMobile
   else {
