@@ -29,12 +29,18 @@ const BrowserWindow = Class({
     throw new Error(ERR_ICECATMOBILE_MSG);
     return null;
   },
-  get title() getWindowTitle(windowNS(this).window),
+  get title() {
+    return getWindowTitle(windowNS(this).window);
+  },
   // NOTE: IceCatMobile only has one window, which is assumed below
   // TODO: remove assumption below
   // NOTE: tabs requires windows
-  get tabs() require('../tabs'),
-  get activeTab() require('../tabs').activeTab,
+  get tabs() {
+    return require('../tabs');
+  },
+  get activeTab() {
+    return require('../tabs').activeTab;
+  },
   on: method(on),
   removeListener: method(off),
   once: method(once)
