@@ -92,7 +92,7 @@ function createWorker(assert, xrayWindow, contentScript, done) {
 
 /* Examples for the `createProxyTest` uses */
 
-let html = "<script>var documentGlobal = true</script>";
+var html = "<script>var documentGlobal = true</script>";
 
 exports["test Create Proxy Test"] = createProxyTest(html, function (helper, assert) {
   // You can get access to regular `test` object in second argument of
@@ -169,7 +169,7 @@ exports["test Shared To String Proxies"] = createProxyTest("", function(helper)
 */
 
 // Ensure that postMessage is working correctly across documents with an iframe
-let html = '<iframe id="iframe" name="test" src="data:text/html;charset=utf-8," />';
+var html = '<iframe id="iframe" name="test" src="data:text/html;charset=utf-8," />';
 exports["test postMessage"] = createProxyTest(html, function (helper, assert) {
   let ifWindow = helper.xrayWindow.document.getElementById("iframe").contentWindow;
   // Listen without proxies, to check that it will work in regular case
@@ -199,7 +199,7 @@ exports["test postMessage"] = createProxyTest(html, function (helper, assert) {
   );
 });
 
-let html = '<input id="input2" type="checkbox" />';
+var html = '<input id="input2" type="checkbox" />';
 exports["test Object Listener"] = createProxyTest(html, function (helper) {
 
   helper.createWorker(
@@ -255,7 +255,7 @@ exports["test Object Listener 2"] = createProxyTest("", function (helper) {
 
 });
 
-let html = '<input id="input" type="text" /><input id="input3" type="checkbox" />' +
+var html = '<input id="input" type="text" /><input id="input3" type="checkbox" />' +
              '<input id="input2" type="checkbox" />';
 
 exports.testStringOverload = createProxyTest(html, function (helper, assert) {
@@ -294,21 +294,21 @@ exports.testStringOverload = createProxyTest(html, function (helper, assert) {
   );
 });
 
-exports["test MozMatchedSelector"] = createProxyTest("", function (helper) {
+exports["test Element.matches()"] = createProxyTest("", function (helper) {
   helper.createWorker(
     'new ' + function ContentScriptScope() {
-      // Check mozMatchesSelector XrayWrappers bug:
-      // mozMatchesSelector returns bad results when we are not calling it from the node itself
-      // SEE BUG 658909: mozMatchesSelector returns incorrect results with XrayWrappers
-      assert(document.createElement( "div" ).mozMatchesSelector("div"),
-             "mozMatchesSelector works while being called from the node");
-      assert(document.documentElement.mozMatchesSelector.call(
+      // Check matches XrayWrappers bug (Bug 658909):
+      // Test that Element.matches() does not return bad results when we are
+      // not calling it from the node itself.
+      assert(document.createElement( "div" ).matches("div"),
+             "matches works while being called from the node");
+      assert(document.documentElement.matches.call(
                document.createElement( "div" ),
                "div"
              ),
-             "mozMatchesSelector works while being called from a " +
+             "matches works while being called from a " +
              "function reference to " +
-             "document.documentElement.mozMatchesSelector.call");
+             "document.documentElement.matches.call");
       done();
     }
   );
@@ -504,7 +504,7 @@ exports["test Document TagName"] = createProxyTest("", function (helper) {
 
 });
 
-let html = '<iframe id="iframe" name="test" src="data:text/html;charset=utf-8," />';
+var html = '<iframe id="iframe" name="test" src="data:text/html;charset=utf-8," />';
 exports["test Window Frames"] = createProxyTest(html, function (helper) {
 
   helper.createWorker(
@@ -539,7 +539,7 @@ exports["test Collections"] = createProxyTest("", function (helper) {
 
 });
 
-let html = '<input id="input" type="text" /><input id="input3" type="checkbox" />' +
+var html = '<input id="input" type="text" /><input id="input3" type="checkbox" />' +
              '<input id="input2" type="checkbox" />';
 exports["test Collections 2"] = createProxyTest(html, function (helper) {
 
@@ -634,8 +634,12 @@ exports["test Prototype Inheritance"] = createProxyTest("", function (helper) {
 
 exports["test Functions"] = createProxyTest("", function (helper) {
 
-  helper.rawWindow.callFunction = function callFunction(f) f();
-  helper.rawWindow.isEqual = function isEqual(a, b) a == b;
+  helper.rawWindow.callFunction = function callFunction(f) {
+    return f();
+  };
+  helper.rawWindow.isEqual = function isEqual(a, b) {
+    return a == b;
+  };
   // bug 784116: workaround in order to allow proxy code to cache proxies on
   // these functions:
   helper.rawWindow.callFunction.__exposedProps__ = {__proxy: 'rw'};
@@ -656,7 +660,7 @@ exports["test Functions"] = createProxyTest("", function (helper) {
 
 });
 
-let html = '<input id="input2" type="checkbox" />';
+var html = '<input id="input2" type="checkbox" />';
 exports["test Listeners"] = createProxyTest(html, function (helper) {
 
   helper.createWorker(
@@ -710,12 +714,13 @@ exports["test Listeners"] = createProxyTest(html, function (helper) {
 
 });
 
-exports["test MozRequestAnimationFrame"] = createProxyTest("", function (helper) {
+exports["test requestAnimationFrame"] = createProxyTest("", function (helper) {
 
   helper.createWorker(
     'new ' + function ContentScriptScope() {
-      window.mozRequestAnimationFrame(function callback() {
-        assert(callback == this, "callback is equal to `this`");
+      var self = (function() { return this; })();
+      window.requestAnimationFrame(function callback() {
+        assert(self == this, "self is equal to `this`");
         done();
       });
     }
@@ -726,7 +731,7 @@ exports["test MozRequestAnimationFrame"] = createProxyTest("", function (helper)
 exports["testGlobalScope"] = createProxyTest("", function (helper) {
 
   helper.createWorker(
-    'let toplevelScope = true;' +
+    'var toplevelScope = true;' +
     'assert(window.toplevelScope, "variables in toplevel scope are set to `window` object");' +
     'assert(this.toplevelScope, "variables in toplevel scope are set to `this` object");' +
     'done();'
@@ -789,7 +794,7 @@ exports["test Cross Domain Iframe"] = createProxyTest("", function (helper) {
 });
 
 // Bug 769006: Ensure that MutationObserver works fine with proxies
-let html = '<a href="foo">link</a>';
+var html = '<a href="foo">link</a>';
 exports["test MutationObvserver"] = createProxyTest(html, function (helper) {
 
   helper.createWorker(
@@ -827,7 +832,7 @@ exports["test MutationObvserver"] = createProxyTest(html, function (helper) {
 
 });
 
-let html = '<script>' +
+var html = '<script>' +
   'var accessCheck = function() {' +
   '  assert(true, "exporting function works");' +
   '  try{' +
