@@ -13,7 +13,7 @@ module.metadata = {
 const { Cc, Ci } = require("chrome");
 const { on } = require("sdk/event/core");
 const { setTimeout } = require("sdk/timers");
-const { LoaderWithHookedConsole } = require("sdk/test/loader");
+const { LoaderWithHookedConsole, Loader } = require("sdk/test/loader");
 const { Worker } = require("sdk/content/worker");
 const { close } = require("sdk/window/helpers");
 const { set: setPref } = require("sdk/preferences/service");
@@ -118,14 +118,15 @@ exports["test:sample"] = WorkerTest(
         assert.equal(worker.url, window.location.href,
                          "worker.url still works");
         done();
+      },
+      onAttach: function() {
+        assert.equal(worker.url, window.location.href,
+                         "worker.url works");
+        assert.equal(worker.contentURL, window.location.href,
+                         "worker.contentURL works");
+        worker.postMessage("hi!");
       }
     });
-
-    assert.equal(worker.url, window.location.href,
-                     "worker.url works");
-    assert.equal(worker.contentURL, window.location.href,
-                     "worker.contentURL works");
-    worker.postMessage("hi!");
   }
 );
 
@@ -485,7 +486,7 @@ exports["test:setInterval async Errors passed to .onError"] = WorkerTest(
       onError: function(err) {
         count++;
         assert.equal(err.message, "ubik",
-            "error (corectly) propagated  " + count + " time(s)");
+            "error (correctly) propagated  " + count + " time(s)");
         if (count >= 3) done();
       }
     });
@@ -742,9 +743,17 @@ exports["test:check worker API with page history"] = WorkerTest(
           // that will be disable until the page gets visible again
           self.on("pagehide", function () {
             setTimeout(function () {
-              self.postMessage("timeout restored");
+              self.port.emit("timeout");
             }, 0);
           });
+
+          self.on("message", function() {
+            self.postMessage("saw message");
+          });
+
+          self.on("event", function() {
+            self.port.emit("event", "saw event");
+          });
         },
         contentScriptWhen: "start"
       });
@@ -761,21 +770,10 @@ exports["test:check worker API with page history"] = WorkerTest(
       // Wait for the document to be hidden
       browser.addEventListener("pagehide", function onpagehide() {
         browser.removeEventListener("pagehide", onpagehide, false);
-        // Now any event sent to this worker should throw
+        // Now any event sent to this worker should be cached
 
-        setTimeout(_ => {
-          assert.throws(
-              function () { worker.postMessage("data"); },
-              /The page is currently hidden and can no longer be used/,
-              "postMessage should throw when the page is hidden in history"
-              );
-
-          assert.throws(
-              function () { worker.port.emit("event"); },
-              /The page is currently hidden and can no longer be used/,
-              "port.emit should throw when the page is hidden in history"
-              );
-        })
+        worker.postMessage("message");
+        worker.port.emit("event");
 
         // Display the page with attached content script back in order to resume
         // its timeout and receive the expected message.
@@ -784,10 +782,30 @@ exports["test:check worker API with page history"] = WorkerTest(
         // do not receive the message immediatly, so that the timeout is
         // actually disabled
         setTimeout(function () {
-          worker.on("message", function (data) {
-            assert.ok(data, "timeout restored");
-            done();
+          worker.on("pageshow", function() {
+            let promise = Promise.all([
+              new Promise(resolve => {
+                worker.port.on("event", () => {
+                  assert.pass("Saw event");
+                  resolve();
+                });
+              }),
+              new Promise(resolve => {
+                worker.on("message", () => {
+                  assert.pass("Saw message");
+                  resolve();
+                });
+              }),
+              new Promise(resolve => {
+                worker.port.on("timeout", () => {
+                  assert.pass("Timer fired");
+                  resolve();
+                });
+              })
+            ]);
+            promise.then(done);
           });
+
           browser.goForward();
         }, 500);
 
@@ -824,10 +842,9 @@ exports["test:worker events"] = WorkerTest(
       contentScript: 'new ' + function WorkerScope() {
         self.postMessage('start');
       },
-      onAttach: win => {
+      onAttach: () => {
         events.push('attach');
         assert.pass('attach event called when attached');
-        assert.equal(window, win, 'attach event passes in attached window');
       },
       onError: err => {
         assert.equal(err.message, 'Custom',
@@ -859,13 +876,16 @@ exports["test:onDetach in contentScript on destroy"] = WorkerTest(
           window.location.hash += '!' + reason;
         })
       },
+
+      onAttach: function() {
+        browser.contentWindow.addEventListener('hashchange', _ => {
+          assert.equal(browser.contentWindow.location.hash, '#detach!',
+                       "location.href is as expected");
+          done();
+        })
+        worker.destroy();
+      }
     });
-    browser.contentWindow.addEventListener('hashchange', _ => {
-      assert.equal(browser.contentWindow.location.hash, '#detach!',
-                   "location.href is as expected");
-      done();
-    })
-    worker.destroy();
   }
 );
 
@@ -977,5 +997,133 @@ exports["test:destroy unbinds listeners from port"] = WorkerTest(
   }
 );
 
+exports["test:destroy kills child worker"] = WorkerTest(
+  "data:text/html;charset=utf-8,<html><body><p id='detail'></p></body></html>",
+  function(assert, browser, done) {
+    let worker1 = Worker({
+      window: browser.contentWindow,
+      contentScript: "new " + function WorkerScope() {
+        self.port.on("ping", detail => {
+          let event = document.createEvent("CustomEvent");
+          event.initCustomEvent("Test:Ping", true, true, detail);
+          document.dispatchEvent(event);
+          self.port.emit("pingsent");
+        });
+
+        let listener = function(event) {
+          self.port.emit("pong", event.detail);
+        };
+
+        self.port.on("detach", () => {
+          window.removeEventListener("Test:Pong", listener);
+        });
+        window.addEventListener("Test:Pong", listener);
+      },
+      onAttach: function() {
+        let worker2 = Worker({
+          window: browser.contentWindow,
+          contentScript: "new " + function WorkerScope() {
+            let listener = function(event) {
+              let newEvent = document.createEvent("CustomEvent");
+              newEvent.initCustomEvent("Test:Pong", true, true, event.detail);
+              document.dispatchEvent(newEvent);
+            };
+            self.port.on("detach", () => {
+              window.removeEventListener("Test:Ping", listener);
+            })
+            window.addEventListener("Test:Ping", listener);
+            self.postMessage();
+          },
+          onMessage: function() {
+            worker1.port.emit("ping", "test1");
+            worker1.port.once("pong", detail => {
+              assert.equal(detail, "test1", "Saw the right message");
+              worker1.port.once("pingsent", () => {
+                assert.pass("The message was sent");
+
+                worker2.destroy();
+
+                worker1.port.emit("ping", "test2");
+                worker1.port.once("pong", detail => {
+                  assert.fail("worker2 shouldn't have responded");
+                })
+                worker1.port.once("pingsent", () => {
+                  assert.pass("The message was sent");
+                  worker1.destroy();
+                  done();
+                });
+              });
+            })
+          }
+        });
+      }
+    });
+  }
+);
+
+exports["test:unload kills child worker"] = WorkerTest(
+  "data:text/html;charset=utf-8,<html><body><p id='detail'></p></body></html>",
+  function(assert, browser, done) {
+    let loader = Loader(module);
+    let worker1 = Worker({
+      window: browser.contentWindow,
+      contentScript: "new " + function WorkerScope() {
+        self.port.on("ping", detail => {
+          let event = document.createEvent("CustomEvent");
+          event.initCustomEvent("Test:Ping", true, true, detail);
+          document.dispatchEvent(event);
+          self.port.emit("pingsent");
+        });
+
+        let listener = function(event) {
+          self.port.emit("pong", event.detail);
+        };
+
+        self.port.on("detach", () => {
+          window.removeEventListener("Test:Pong", listener);
+        });
+        window.addEventListener("Test:Pong", listener);
+      },
+      onAttach: function() {
+        let worker2 = loader.require("sdk/content/worker").Worker({
+          window: browser.contentWindow,
+          contentScript: "new " + function WorkerScope() {
+            let listener = function(event) {
+              let newEvent = document.createEvent("CustomEvent");
+              newEvent.initCustomEvent("Test:Pong", true, true, event.detail);
+              document.dispatchEvent(newEvent);
+            };
+            self.port.on("detach", () => {
+              window.removeEventListener("Test:Ping", listener);
+            })
+            window.addEventListener("Test:Ping", listener);
+            self.postMessage();
+          },
+          onMessage: function() {
+            worker1.port.emit("ping", "test1");
+            worker1.port.once("pong", detail => {
+              assert.equal(detail, "test1", "Saw the right message");
+              worker1.port.once("pingsent", () => {
+                assert.pass("The message was sent");
+
+                loader.unload();
+
+                worker1.port.emit("ping", "test2");
+                worker1.port.once("pong", detail => {
+                  assert.fail("worker2 shouldn't have responded");
+                })
+                worker1.port.once("pingsent", () => {
+                  assert.pass("The message was sent");
+                  worker1.destroy();
+                  done();
+                });
+              });
+            })
+          }
+        });
+      }
+    });
+  }
+);
 
 require("sdk/test").run(exports);
