@@ -1,21 +1,20 @@
-/* -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 40 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "BluetoothDaemonAvrcpInterface.h"
-#include "BluetoothDaemonSetupInterface.h"
 #include "mozilla/unused.h"
 
 BEGIN_BLUETOOTH_NAMESPACE
 
+using namespace mozilla::ipc;
+
 //
 // AVRCP module
 //
 
-const int BluetoothDaemonAvrcpModule::MAX_NUM_CLIENTS = 1;
-
 BluetoothAvrcpNotificationHandler*
   BluetoothDaemonAvrcpModule::sNotificationHandler;
 
@@ -26,31 +25,23 @@ BluetoothDaemonAvrcpModule::SetNotificationHandler(
   sNotificationHandler = aNotificationHandler;
 }
 
-nsresult
-BluetoothDaemonAvrcpModule::Send(BluetoothDaemonPDU* aPDU,
-                                 BluetoothAvrcpResultHandler* aRes)
-{
-  if (aRes) {
-    aRes->AddRef(); // Keep reference for response
-  }
-  return Send(aPDU, static_cast<void*>(aRes));
-}
-
 void
-BluetoothDaemonAvrcpModule::HandleSvc(const BluetoothDaemonPDUHeader& aHeader,
-                                      BluetoothDaemonPDU& aPDU, void* aUserData)
+BluetoothDaemonAvrcpModule::HandleSvc(const DaemonSocketPDUHeader& aHeader,
+                                      DaemonSocketPDU& aPDU,
+                                      DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleOp[])(
-    const BluetoothDaemonPDUHeader&, BluetoothDaemonPDU&, void*) = {
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::HandleRsp),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::HandleNtf),
+    const DaemonSocketPDUHeader&, DaemonSocketPDU&,
+    DaemonSocketResultHandler*) = {
+    [0] = &BluetoothDaemonAvrcpModule::HandleRsp,
+    [1] = &BluetoothDaemonAvrcpModule::HandleNtf
   };
 
   MOZ_ASSERT(!NS_IsMainThread());
 
   unsigned int isNtf = !!(aHeader.mOpcode & 0x80);
 
-  (this->*(HandleOp[isNtf]))(aHeader, aPDU, aUserData);
+  (this->*(HandleOp[isNtf]))(aHeader, aPDU, aRes);
 }
 
 // Commands
@@ -63,8 +54,8 @@ BluetoothDaemonAvrcpModule::GetPlayStatusRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_GET_PLAY_STATUS_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAY_STATUS_RSP,
                            1 + // Play status
                            4 + // Duration
                            4)); // Position
@@ -77,7 +68,7 @@ BluetoothDaemonAvrcpModule::GetPlayStatusRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -88,8 +79,8 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_ATTR_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_ATTR_RSP,
                            1 + // # Attributes
                            aNumAttr)); // Player attributes
 
@@ -103,7 +94,7 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -113,8 +104,8 @@ BluetoothDaemonAvrcpModule::ListPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_VALUE_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_VALUE_RSP,
                            1 + // # Values
                            aNumVal)); // Player values
 
@@ -127,7 +118,7 @@ BluetoothDaemonAvrcpModule::ListPlayerAppValueRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -138,8 +129,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_RSP,
                            1 + // # Pairs
                            2 * aNumAttrs)); // Attribute-value pairs
   nsresult rv = PackPDU(
@@ -152,7 +143,7 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -163,8 +154,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
                            0)); // Dynamically allocated
   nsresult rv = PackPDU(
     PackConversion<int, uint8_t>(aNumAttr),
@@ -176,7 +167,7 @@ BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -187,8 +178,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
                            0)); // Dynamically allocated
   nsresult rv = PackPDU(
     PackConversion<int, uint8_t>(aNumVal),
@@ -200,7 +191,7 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -211,8 +202,8 @@ BluetoothDaemonAvrcpModule::GetElementAttrRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_GET_ELEMENT_ATTR_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_ELEMENT_ATTR_RSP,
                            0)); // Dynamically allocated
   nsresult rv = PackPDU(
     aNumAttr,
@@ -224,7 +215,7 @@ BluetoothDaemonAvrcpModule::GetElementAttrRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -234,8 +225,8 @@ BluetoothDaemonAvrcpModule::SetPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_SET_PLAYER_APP_VALUE_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_SET_PLAYER_APP_VALUE_RSP,
                            1)); // Status code
 
   nsresult rv = PackPDU(aRspStatus, *pdu);
@@ -246,7 +237,7 @@ BluetoothDaemonAvrcpModule::SetPlayerAppValueRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -258,15 +249,16 @@ BluetoothDaemonAvrcpModule::RegisterNotificationRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_REGISTER_NOTIFICATION_RSP,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_REGISTER_NOTIFICATION_RSP,
                            1 + // Event
                            1 + // Type
                            1 + // Data length
                            256)); // Maximum data length
 
-  nsresult rv = PackPDU(aEvent, aType,
-                        BluetoothAvrcpEventParamPair(aEvent, aParam), *pdu);
+  BluetoothAvrcpEventParamPair data(aEvent, aParam);
+  nsresult rv = PackPDU(aEvent, aType, static_cast<uint8_t>(data.GetLength()),
+                        data, *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -274,7 +266,7 @@ BluetoothDaemonAvrcpModule::RegisterNotificationRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -284,8 +276,8 @@ BluetoothDaemonAvrcpModule::SetVolumeCmd(uint8_t aVolume,
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<BluetoothDaemonPDU> pdu(
-    new BluetoothDaemonPDU(SERVICE_ID, OPCODE_SET_VOLUME,
+  nsAutoPtr<DaemonSocketPDU> pdu(
+    new DaemonSocketPDU(SERVICE_ID, OPCODE_SET_VOLUME,
                            1)); // Volume
 
   nsresult rv = PackPDU(aVolume, *pdu);
@@ -296,7 +288,7 @@ BluetoothDaemonAvrcpModule::SetVolumeCmd(uint8_t aVolume,
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.forget();
   return NS_OK;
 }
 
@@ -305,8 +297,8 @@ BluetoothDaemonAvrcpModule::SetVolumeCmd(uint8_t aVolume,
 
 void
 BluetoothDaemonAvrcpModule::ErrorRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ErrorRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::OnError, UnpackPDUInitOp(aPDU));
@@ -314,8 +306,8 @@ BluetoothDaemonAvrcpModule::ErrorRsp(
 
 void
 BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::GetPlayStatusRsp,
@@ -324,8 +316,8 @@ BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::ListPlayerAppAttrRsp,
@@ -334,8 +326,8 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::ListPlayerAppValueRsp,
@@ -344,8 +336,8 @@ BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::GetPlayerAppValueRsp,
@@ -354,8 +346,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::GetPlayerAppAttrTextRsp,
@@ -364,8 +356,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::GetPlayerAppValueTextRsp,
@@ -374,8 +366,8 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::GetElementAttrRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::GetElementAttrRsp,
@@ -384,8 +376,8 @@ BluetoothDaemonAvrcpModule::GetElementAttrRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::SetPlayerAppValueRsp,
@@ -394,8 +386,8 @@ BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::RegisterNotificationRsp,
@@ -404,8 +396,8 @@ BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp(
 
 void
 BluetoothDaemonAvrcpModule::SetVolumeRsp(
-  const BluetoothDaemonPDUHeader& aHeader,
-  BluetoothDaemonPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
+  const DaemonSocketPDUHeader& aHeader,
+  DaemonSocketPDU& aPDU, BluetoothAvrcpResultHandler* aRes)
 {
   ResultRunnable::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::SetVolume,
@@ -414,35 +406,35 @@ BluetoothDaemonAvrcpModule::SetVolumeRsp(
 
 void
 BluetoothDaemonAvrcpModule::HandleRsp(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU,
-  void* aUserData)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU,
+  DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleRsp[])(
-    const BluetoothDaemonPDUHeader&,
-    BluetoothDaemonPDU&,
+    const DaemonSocketPDUHeader&,
+    DaemonSocketPDU&,
     BluetoothAvrcpResultHandler*) = {
-    INIT_ARRAY_AT(OPCODE_ERROR,
-      &BluetoothDaemonAvrcpModule::ErrorRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAY_STATUS_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp),
-    INIT_ARRAY_AT(OPCODE_LIST_PLAYER_APP_ATTR_RSP,
-      &BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp),
-    INIT_ARRAY_AT(OPCODE_LIST_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_ELEMENT_ATTR_RSP,
-      &BluetoothDaemonAvrcpModule::GetElementAttrRspRsp),
-    INIT_ARRAY_AT(OPCODE_SET_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_REGISTER_NOTIFICATION_RSP,
-      &BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp),
-    INIT_ARRAY_AT(OPCODE_SET_VOLUME,
-      &BluetoothDaemonAvrcpModule::SetVolumeRsp)
+    [OPCODE_ERROR] =
+      &BluetoothDaemonAvrcpModule::ErrorRsp,
+    [OPCODE_GET_PLAY_STATUS_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp,
+    [OPCODE_LIST_PLAYER_APP_ATTR_RSP] =
+      &BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp,
+    [OPCODE_LIST_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp,
+    [OPCODE_GET_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp,
+    [OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp,
+    [OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp,
+    [OPCODE_GET_ELEMENT_ATTR_RSP]=
+      &BluetoothDaemonAvrcpModule::GetElementAttrRspRsp,
+    [OPCODE_SET_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp,
+    [OPCODE_REGISTER_NOTIFICATION_RSP] =
+      &BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp,
+    [OPCODE_SET_VOLUME] =
+      &BluetoothDaemonAvrcpModule::SetVolumeRsp
   };
 
   MOZ_ASSERT(!NS_IsMainThread()); // I/O thread
@@ -452,9 +444,8 @@ BluetoothDaemonAvrcpModule::HandleRsp(
     return;
   }
 
-  nsRefPtr<BluetoothAvrcpResultHandler> res =
-    already_AddRefed<BluetoothAvrcpResultHandler>(
-      static_cast<BluetoothAvrcpResultHandler*>(aUserData));
+  RefPtr<BluetoothAvrcpResultHandler> res =
+    static_cast<BluetoothAvrcpResultHandler*>(aRes);
 
   if (!res) {
     return; // Return early if no result handler has been set for response
@@ -485,18 +476,17 @@ class BluetoothDaemonAvrcpModule::RemoteFeatureInitOp final
   : private PDUInitOp
 {
 public:
-  RemoteFeatureInitOp(BluetoothDaemonPDU& aPDU)
+  RemoteFeatureInitOp(DaemonSocketPDU& aPDU)
     : PDUInitOp(aPDU)
   { }
 
   nsresult
-  operator () (nsString& aArg1, unsigned long& aArg2) const
+  operator () (BluetoothAddress& aArg1, unsigned long& aArg2) const
   {
-    BluetoothDaemonPDU& pdu = GetPDU();
+    DaemonSocketPDU& pdu = GetPDU();
 
     /* Read address */
-    nsresult rv = UnpackPDU(
-      pdu, UnpackConversion<BluetoothAddress, nsAString>(aArg1));
+    nsresult rv = UnpackPDU(pdu, aArg1);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -504,7 +494,7 @@ public:
     /* Read feature */
     rv = UnpackPDU(
       pdu,
-      UnpackConversion<BluetoothAvrcpRemoteFeature, unsigned long>(aArg2));
+      UnpackConversion<BluetoothAvrcpRemoteFeatureBits, unsigned long>(aArg2));
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -515,7 +505,7 @@ public:
 
 void
 BluetoothDaemonAvrcpModule::RemoteFeatureNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   RemoteFeatureNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::RemoteFeatureNotification,
@@ -524,7 +514,7 @@ BluetoothDaemonAvrcpModule::RemoteFeatureNtf(
 
 void
 BluetoothDaemonAvrcpModule::GetPlayStatusNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   GetPlayStatusNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::GetPlayStatusNotification,
@@ -533,7 +523,7 @@ BluetoothDaemonAvrcpModule::GetPlayStatusNtf(
 
 void
 BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   ListPlayerAppAttrNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::ListPlayerAppAttrNotification,
@@ -542,7 +532,7 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf(
 
 void
 BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   ListPlayerAppValuesNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::ListPlayerAppValuesNotification,
@@ -554,7 +544,7 @@ class BluetoothDaemonAvrcpModule::GetPlayerAppValueInitOp final
   : private PDUInitOp
 {
 public:
-  GetPlayerAppValueInitOp(BluetoothDaemonPDU& aPDU)
+  GetPlayerAppValueInitOp(DaemonSocketPDU& aPDU)
     : PDUInitOp(aPDU)
   { }
 
@@ -562,7 +552,7 @@ public:
   operator () (uint8_t& aArg1,
                nsAutoArrayPtr<BluetoothAvrcpPlayerAttribute>& aArg2) const
   {
-    BluetoothDaemonPDU& pdu = GetPDU();
+    DaemonSocketPDU& pdu = GetPDU();
 
     /* Read number of attributes */
     nsresult rv = UnpackPDU(pdu, aArg1);
@@ -583,7 +573,7 @@ public:
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   GetPlayerAppValueNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::GetPlayerAppValueNotification,
@@ -595,7 +585,7 @@ class BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextInitOp final
   : private PDUInitOp
 {
 public:
-  GetPlayerAppAttrsTextInitOp(BluetoothDaemonPDU& aPDU)
+  GetPlayerAppAttrsTextInitOp(DaemonSocketPDU& aPDU)
     : PDUInitOp(aPDU)
   { }
 
@@ -603,7 +593,7 @@ public:
   operator () (uint8_t& aArg1,
                nsAutoArrayPtr<BluetoothAvrcpPlayerAttribute>& aArg2) const
   {
-    BluetoothDaemonPDU& pdu = GetPDU();
+    DaemonSocketPDU& pdu = GetPDU();
 
     /* Read number of attributes */
     nsresult rv = UnpackPDU(pdu, aArg1);
@@ -624,7 +614,7 @@ public:
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   GetPlayerAppAttrsTextNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::GetPlayerAppAttrsTextNotification,
@@ -636,7 +626,7 @@ class BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextInitOp final
   : private PDUInitOp
 {
 public:
-  GetPlayerAppValuesTextInitOp(BluetoothDaemonPDU& aPDU)
+  GetPlayerAppValuesTextInitOp(DaemonSocketPDU& aPDU)
     : PDUInitOp(aPDU)
   { }
 
@@ -644,7 +634,7 @@ public:
   operator () (uint8_t& aArg1, uint8_t& aArg2,
                nsAutoArrayPtr<uint8_t>& aArg3) const
   {
-    BluetoothDaemonPDU& pdu = GetPDU();
+    DaemonSocketPDU& pdu = GetPDU();
 
     /* Read attribute */
     nsresult rv = UnpackPDU(pdu, aArg1);
@@ -670,7 +660,7 @@ public:
 
 void
 BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   GetPlayerAppValuesTextNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::GetPlayerAppValuesTextNotification,
@@ -679,7 +669,7 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf(
 
 void
 BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   SetPlayerAppValueNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::SetPlayerAppValueNotification,
@@ -691,7 +681,7 @@ class BluetoothDaemonAvrcpModule::GetElementAttrInitOp final
   : private PDUInitOp
 {
 public:
-  GetElementAttrInitOp(BluetoothDaemonPDU& aPDU)
+  GetElementAttrInitOp(DaemonSocketPDU& aPDU)
     : PDUInitOp(aPDU)
   { }
 
@@ -699,7 +689,7 @@ public:
   operator () (uint8_t& aArg1,
                nsAutoArrayPtr<BluetoothAvrcpMediaAttribute>& aArg2) const
   {
-    BluetoothDaemonPDU& pdu = GetPDU();
+    DaemonSocketPDU& pdu = GetPDU();
 
     /* Read number of attributes */
     nsresult rv = UnpackPDU(pdu, aArg1);
@@ -720,7 +710,7 @@ public:
 
 void
 BluetoothDaemonAvrcpModule::GetElementAttrNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   GetElementAttrNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::GetElementAttrNotification,
@@ -729,7 +719,7 @@ BluetoothDaemonAvrcpModule::GetElementAttrNtf(
 
 void
 BluetoothDaemonAvrcpModule::RegisterNotificationNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   RegisterNotificationNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::RegisterNotificationNotification,
@@ -739,80 +729,53 @@ BluetoothDaemonAvrcpModule::RegisterNotificationNtf(
 #if ANDROID_VERSION >= 19
 void
 BluetoothDaemonAvrcpModule::VolumeChangeNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   VolumeChangeNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::VolumeChangeNotification,
     UnpackPDUInitOp(aPDU));
 }
 
-// Init operator class for PassthroughCmdNotification
-class BluetoothDaemonAvrcpModule::PassthroughCmdInitOp final
-  : private PDUInitOp
-{
-public:
-  PassthroughCmdInitOp(BluetoothDaemonPDU& aPDU)
-    : PDUInitOp(aPDU)
-  { }
-
-  nsresult
-  operator () (int& aArg1, int& aArg2) const
-  {
-    BluetoothDaemonPDU& pdu = GetPDU();
-
-    nsresult rv = UnpackPDU(pdu, UnpackConversion<uint8_t, int>(aArg1));
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-    rv = UnpackPDU(pdu, UnpackConversion<uint8_t, int>(aArg2));
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-    WarnAboutTrailingData();
-    return NS_OK;
-  }
-};
-
 void
 BluetoothDaemonAvrcpModule::PassthroughCmdNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   PassthroughCmdNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::PassthroughCmdNotification,
-    PassthroughCmdInitOp(aPDU));
+    UnpackPDUInitOp(aPDU));
 }
 #endif
 
 void
 BluetoothDaemonAvrcpModule::HandleNtf(
-  const BluetoothDaemonPDUHeader& aHeader, BluetoothDaemonPDU& aPDU,
-  void* aUserData)
+  const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU,
+  DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleNtf[])(
-    const BluetoothDaemonPDUHeader&, BluetoothDaemonPDU&) = {
+    const DaemonSocketPDUHeader&, DaemonSocketPDU&) = {
 #if ANDROID_VERSION >= 19
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::RemoteFeatureNtf),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::GetPlayStatusNtf),
-    INIT_ARRAY_AT(2, &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf),
-    INIT_ARRAY_AT(3, &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf),
-    INIT_ARRAY_AT(4, &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf),
-    INIT_ARRAY_AT(5, &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf),
-    INIT_ARRAY_AT(6, &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf),
-    INIT_ARRAY_AT(7, &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf),
-    INIT_ARRAY_AT(8, &BluetoothDaemonAvrcpModule::GetElementAttrNtf),
-    INIT_ARRAY_AT(9, &BluetoothDaemonAvrcpModule::RegisterNotificationNtf),
-    INIT_ARRAY_AT(10, &BluetoothDaemonAvrcpModule::VolumeChangeNtf),
-    INIT_ARRAY_AT(11, &BluetoothDaemonAvrcpModule::PassthroughCmdNtf)
+    [0] = &BluetoothDaemonAvrcpModule::RemoteFeatureNtf,
+    [1] = &BluetoothDaemonAvrcpModule::GetPlayStatusNtf,
+    [2] = &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf,
+    [3] = &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf,
+    [4] = &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf,
+    [5] = &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf,
+    [6] = &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf,
+    [7] = &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf,
+    [8] = &BluetoothDaemonAvrcpModule::GetElementAttrNtf,
+    [9] = &BluetoothDaemonAvrcpModule::RegisterNotificationNtf,
+    [10] = &BluetoothDaemonAvrcpModule::VolumeChangeNtf,
+    [11] = &BluetoothDaemonAvrcpModule::PassthroughCmdNtf
 #else
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::GetPlayStatusNtf),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf),
-    INIT_ARRAY_AT(2, &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf),
-    INIT_ARRAY_AT(3, &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf),
-    INIT_ARRAY_AT(4, &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf),
-    INIT_ARRAY_AT(5, &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf),
-    INIT_ARRAY_AT(6, &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf),
-    INIT_ARRAY_AT(7, &BluetoothDaemonAvrcpModule::GetElementAttrNtf),
-    INIT_ARRAY_AT(8, &BluetoothDaemonAvrcpModule::RegisterNotificationNtf)
+    [0] = &BluetoothDaemonAvrcpModule::GetPlayStatusNtf,
+    [1] = &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf,
+    [2] = &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf,
+    [3] = &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf,
+    [4] = &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf,
+    [5] = &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf,
+    [6] = &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf,
+    [7] = &BluetoothDaemonAvrcpModule::GetElementAttrNtf,
+    [8] = &BluetoothDaemonAvrcpModule::RegisterNotificationNtf
 #endif
   };
 
@@ -840,111 +803,13 @@ BluetoothDaemonAvrcpInterface::BluetoothDaemonAvrcpInterface(
 BluetoothDaemonAvrcpInterface::~BluetoothDaemonAvrcpInterface()
 { }
 
-class BluetoothDaemonAvrcpInterface::InitResultHandler final
-  : public BluetoothSetupResultHandler
-{
-public:
-  InitResultHandler(BluetoothAvrcpResultHandler* aRes)
-    : mRes(aRes)
-  {
-    MOZ_ASSERT(mRes);
-  }
-
-  void OnError(BluetoothStatus aStatus) override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    mRes->OnError(aStatus);
-  }
-
-  void RegisterModule() override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    mRes->Init();
-  }
-
-private:
-  nsRefPtr<BluetoothAvrcpResultHandler> mRes;
-};
-
 void
-BluetoothDaemonAvrcpInterface::Init(
-  BluetoothAvrcpNotificationHandler* aNotificationHandler,
-  BluetoothAvrcpResultHandler* aRes)
+BluetoothDaemonAvrcpInterface::SetNotificationHandler(
+  BluetoothAvrcpNotificationHandler* aNotificationHandler)
 {
   MOZ_ASSERT(mModule);
 
-  // Set notification handler _before_ registering the module. It could
-  // happen that we receive notifications, before the result handler runs.
   mModule->SetNotificationHandler(aNotificationHandler);
-
-  InitResultHandler* res;
-
-  if (aRes) {
-    res = new InitResultHandler(aRes);
-  } else {
-    // We don't need a result handler if the caller is not interested.
-    res = nullptr;
-  }
-
-  nsresult rv = mModule->RegisterModule(
-    BluetoothDaemonAvrcpModule::SERVICE_ID,
-    BluetoothDaemonAvrcpModule::MAX_NUM_CLIENTS, 0x00, res);
-
-  if (NS_FAILED(rv) && aRes) {
-    DispatchError(aRes, STATUS_FAIL);
-  }
-}
-
-class BluetoothDaemonAvrcpInterface::CleanupResultHandler final
-  : public BluetoothSetupResultHandler
-{
-public:
-  CleanupResultHandler(BluetoothDaemonAvrcpModule* aModule,
-                       BluetoothAvrcpResultHandler* aRes)
-    : mModule(aModule)
-    , mRes(aRes)
-  {
-    MOZ_ASSERT(mModule);
-  }
-
-  void OnError(BluetoothStatus aStatus) override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    if (mRes) {
-      mRes->OnError(aStatus);
-    }
-  }
-
-  void UnregisterModule() override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    // Clear notification handler _after_ module has been
-    // unregistered. While unregistering the module, we might
-    // still receive notifications.
-    mModule->SetNotificationHandler(nullptr);
-
-    if (mRes) {
-      mRes->Cleanup();
-    }
-  }
-
-private:
-  BluetoothDaemonAvrcpModule* mModule;
-  nsRefPtr<BluetoothAvrcpResultHandler> mRes;
-};
-
-void
-BluetoothDaemonAvrcpInterface::Cleanup(
-  BluetoothAvrcpResultHandler* aRes)
-{
-  MOZ_ASSERT(mModule);
-
-  mModule->UnregisterModule(BluetoothDaemonAvrcpModule::SERVICE_ID,
-                            new CleanupResultHandler(mModule, aRes));
 }
 
 void
@@ -954,7 +819,11 @@ BluetoothDaemonAvrcpInterface::GetPlayStatusRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->GetPlayStatusRspCmd(aPlayStatus, aSongLen, aSongPos, aRes);
+  nsresult rv = mModule->GetPlayStatusRspCmd(aPlayStatus, aSongLen,
+                                             aSongPos, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -964,7 +833,10 @@ BluetoothDaemonAvrcpInterface::ListPlayerAppAttrRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->ListPlayerAppAttrRspCmd(aNumAttr, aPAttrs, aRes);
+  nsresult rv = mModule->ListPlayerAppAttrRspCmd(aNumAttr, aPAttrs, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -973,7 +845,10 @@ BluetoothDaemonAvrcpInterface::ListPlayerAppValueRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->ListPlayerAppValueRspCmd(aNumVal, aPVals, aRes);
+  nsresult rv = mModule->ListPlayerAppValueRspCmd(aNumVal, aPVals, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -983,7 +858,11 @@ BluetoothDaemonAvrcpInterface::GetPlayerAppValueRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->GetPlayerAppValueRspCmd(aNumAttrs, aIds, aValues, aRes);
+  nsresult rv = mModule->GetPlayerAppValueRspCmd(aNumAttrs, aIds,
+                                                 aValues, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -993,7 +872,11 @@ BluetoothDaemonAvrcpInterface::GetPlayerAppAttrTextRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->GetPlayerAppAttrTextRspCmd(aNumAttr, aIds, aTexts, aRes);
+  nsresult rv = mModule->GetPlayerAppAttrTextRspCmd(aNumAttr, aIds,
+                                                    aTexts, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -1003,7 +886,11 @@ BluetoothDaemonAvrcpInterface::GetPlayerAppValueTextRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->GetPlayerAppValueTextRspCmd(aNumVal, aIds, aTexts, aRes);
+  nsresult rv = mModule->GetPlayerAppValueTextRspCmd(aNumVal, aIds,
+                                                     aTexts, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -1013,7 +900,10 @@ BluetoothDaemonAvrcpInterface::GetElementAttrRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->GetElementAttrRspCmd(aNumAttr, aAttr, aRes);
+  nsresult rv = mModule->GetElementAttrRspCmd(aNumAttr, aAttr, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -1022,7 +912,10 @@ BluetoothDaemonAvrcpInterface::SetPlayerAppValueRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->SetPlayerAppValueRspCmd(aRspStatus, aRes);
+  nsresult rv = mModule->SetPlayerAppValueRspCmd(aRspStatus, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -1034,7 +927,11 @@ BluetoothDaemonAvrcpInterface::RegisterNotificationRsp(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->RegisterNotificationRspCmd(aEvent, aType, aParam, aRes);
+  nsresult rv = mModule->RegisterNotificationRspCmd(aEvent, aType,
+                                                    aParam, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
@@ -1043,17 +940,32 @@ BluetoothDaemonAvrcpInterface::SetVolume(
 {
   MOZ_ASSERT(mModule);
 
-  mModule->SetVolumeCmd(aVolume, aRes);
+  nsresult rv = mModule->SetVolumeCmd(aVolume, aRes);
+  if (NS_FAILED(rv)) {
+    DispatchError(aRes, rv);
+  }
 }
 
 void
 BluetoothDaemonAvrcpInterface::DispatchError(
   BluetoothAvrcpResultHandler* aRes, BluetoothStatus aStatus)
 {
-  BluetoothResultRunnable1<BluetoothAvrcpResultHandler, void,
-                           BluetoothStatus, BluetoothStatus>::Dispatch(
+  DaemonResultRunnable1<BluetoothAvrcpResultHandler, void,
+                        BluetoothStatus, BluetoothStatus>::Dispatch(
     aRes, &BluetoothAvrcpResultHandler::OnError,
     ConstantInitOp1<BluetoothStatus>(aStatus));
 }
 
+void
+BluetoothDaemonAvrcpInterface::DispatchError(
+  BluetoothAvrcpResultHandler* aRes, nsresult aRv)
+{
+  BluetoothStatus status;
+
+  if (NS_WARN_IF(NS_FAILED(Convert(aRv, status)))) {
+    status = STATUS_FAIL;
+  }
+  DispatchError(aRes, status);
+}
+
 END_BLUETOOTH_NAMESPACE
