@@ -1,5 +1,5 @@
-/* -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 40 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,8 +9,6 @@
 #include "BluetoothChild.h"
 
 #include "mozilla/Assertions.h"
-#include "mozilla/ClearOnShutdown.h"
-#include "mozilla/StaticPtr.h"
 #include "nsDebug.h"
 #include "nsISupportsImpl.h"
 #include "nsThreadUtils.h"
@@ -19,14 +17,13 @@
 #include "BluetoothService.h"
 #include "BluetoothServiceChildProcess.h"
 
-using namespace mozilla;
 USING_BLUETOOTH_NAMESPACE
 
 namespace {
 
-StaticRefPtr<BluetoothServiceChildProcess> sBluetoothService;
+BluetoothServiceChildProcess* sBluetoothService;
 
-} // anonymous namespace
+} // namespace
 
 /*******************************************************************************
  * BluetoothChild
@@ -40,7 +37,6 @@ BluetoothChild::BluetoothChild(BluetoothServiceChildProcess* aBluetoothService)
   MOZ_ASSERT(aBluetoothService);
 
   sBluetoothService = aBluetoothService;
-  ClearOnShutdown(&sBluetoothService);
 }
 
 BluetoothChild::~BluetoothChild()
@@ -164,7 +160,7 @@ BluetoothRequestChild::Recv__delete__(const BluetoothReply& aReply)
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(mReplyRunnable);
 
-  nsRefPtr<BluetoothReplyRunnable> replyRunnable;
+  RefPtr<BluetoothReplyRunnable> replyRunnable;
   mReplyRunnable.swap(replyRunnable);
 
   if (replyRunnable) {
