@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,6 +13,7 @@
 #include "mozilla/ErrorResult.h"
 #include "mozilla/Preferences.h"
 #include "AccessCheck.h"
+#include "nsContentUtils.h"
 
 namespace mozilla {
 namespace dom {
@@ -42,14 +45,14 @@ DataStore::Constructor(GlobalObject& aGlobal, ErrorResult& aRv)
     return nullptr;
   }
 
-  nsRefPtr<DataStore> store = new DataStore(window);
+  RefPtr<DataStore> store = new DataStore(window);
   return store.forget();
 }
 
 JSObject*
-DataStore::WrapObject(JSContext* aCx)
+DataStore::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
 {
-  return DataStoreBinding::Wrap(aCx, this);
+  return DataStoreBinding::Wrap(aCx, this, aGivenProto);
 }
 
 /*static*/ bool
