@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,13 +10,12 @@
 #include "mozilla/dom/devicestorage/PDeviceStorageRequestChild.h"
 
 class DeviceStorageFile;
+class DeviceStorageRequest;
 struct DeviceStorageFileDescriptor;
 
 namespace mozilla {
 namespace dom {
 
-class DOMRequest;
-
 namespace devicestorage {
 
 class DeviceStorageRequestChildCallback
@@ -28,21 +28,13 @@ class DeviceStorageRequestChild : public PDeviceStorageRequestChild
 {
 public:
   DeviceStorageRequestChild();
-  DeviceStorageRequestChild(DOMRequest* aRequest, DeviceStorageFile* aFile);
-  DeviceStorageRequestChild(DOMRequest* aRequest, DeviceStorageFile* aFile,
-                            DeviceStorageFileDescriptor* aFileDescrptor);
+  explicit DeviceStorageRequestChild(DeviceStorageRequest* aRequest);
   ~DeviceStorageRequestChild();
 
-  void SetCallback(class DeviceStorageRequestChildCallback *aCallback);
-
   virtual bool Recv__delete__(const DeviceStorageResponseValue& value);
 
 private:
-  nsRefPtr<DOMRequest> mRequest;
-  nsRefPtr<DeviceStorageFile> mDSFile;
-  nsRefPtr<DeviceStorageFileDescriptor> mDSFileDescriptor;
-
-  DeviceStorageRequestChildCallback* mCallback;
+  RefPtr<DeviceStorageRequest> mRequest;
 };
 
 } // namespace devicestorage
