@@ -8,14 +8,12 @@
 #include "nsMimeTypes.h"
 #include "MediaDecoderStateMachine.h"
 #include "AbstractMediaDecoder.h"
-#include "MediaResource.h"
 #include "GStreamerReader.h"
 #if GST_VERSION_MAJOR >= 1
 #include "GStreamerAllocator.h"
 #endif
 #include "GStreamerFormatHelper.h"
 #include "VideoUtils.h"
-#include "mozilla/dom/TimeRanges.h"
 #include "mozilla/Endian.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/unused.h"
@@ -26,17 +24,14 @@ namespace mozilla {
 
 using namespace gfx;
 using namespace layers;
+using namespace media;
 
 // Un-comment to enable logging of seek bisections.
 //#define SEEK_LOGGING
 
-#ifdef PR_LOGGING
-extern PRLogModuleInfo* gMediaDecoderLog;
+extern LazyLogModule gMediaDecoderLog;
 #define LOG(type, msg, ...) \
-  PR_LOG(gMediaDecoderLog, type, ("GStreamerReader(%p) " msg, this, ##__VA_ARGS__))
-#else
-#define LOG(type, msg, ...)
-#endif
+  MOZ_LOG(gMediaDecoderLog, type, ("GStreamerReader(%p) " msg, this, ##__VA_ARGS__))
 
 #if DEBUG
 static const unsigned int MAX_CHANNELS = 4;
@@ -92,7 +87,8 @@ GStreamerReader::GStreamerReader(AbstractMediaDecoder* aDecoder)
   mConfigureAlignment(true),
 #endif
   fpsNum(0),
-  fpsDen(0)
+  fpsDen(0),
+  mResource(aDecoder->GetResource())
 {
   MOZ_COUNT_CTOR(GStreamerReader);
 
@@ -119,7 +115,7 @@ GStreamerReader::~GStreamerReader()
   NS_ASSERTION(!mPlayBin, "No Shutdown() after Init()");
 }
 
-nsresult GStreamerReader::Init(MediaDecoderReader* aCloneDonor)
+nsresult GStreamerReader::InitInternal()
 {
   GStreamerFormatHelper::Instance();
 
@@ -136,7 +132,7 @@ nsresult GStreamerReader::Init(MediaDecoderReader* aCloneDonor)
   mPlayBin = gst_element_factory_make("playbin2", nullptr);
 #endif
   if (!mPlayBin) {
-    LOG(PR_LOG_ERROR, "couldn't create playbin");
+    LOG(LogLevel::Error, "couldn't create playbin");
     return NS_ERROR_FAILURE;
   }
   g_object_set(mPlayBin, "buffer-size", 0, nullptr);
@@ -182,7 +178,7 @@ nsresult GStreamerReader::Init(MediaDecoderReader* aCloneDonor)
   return NS_OK;
 }
 
-nsRefPtr<ShutdownPromise>
+RefPtr<ShutdownPromise>
 GStreamerReader::Shutdown()
 {
   ResetDecode();
@@ -262,7 +258,7 @@ GValueArray *GStreamerReader::ElementFilter(GstURIDecodeBin *aBin,
     GValue *value = &aFactories->values[i];
     GstPluginFeature *factory = GST_PLUGIN_FEATURE(g_value_peek_pointer(value));
 
-    if (!GStreamerFormatHelper::IsPluginFeatureBlacklisted(factory)) {
+    if (!GStreamerFormatHelper::IsPluginFeatureBlocked(factory)) {
       g_value_array_append(filtered, value);
     }
   }
@@ -285,30 +281,29 @@ void GStreamerReader::PlayBinSourceSetup(GstAppSrc* aSource)
 {
   mSource = GST_APP_SRC(aSource);
   gst_app_src_set_callbacks(mSource, &mSrcCallbacks, (gpointer) this, nullptr);
-  MediaResource* resource = mDecoder->GetResource();
 
   /* do a short read to trigger a network request so that GetLength() below
    * returns something meaningful and not -1
    */
   char buf[512];
   unsigned int size = 0;
-  resource->Read(buf, sizeof(buf), &size);
-  resource->Seek(SEEK_SET, 0);
+  mResource.Read(buf, sizeof(buf), &size);
+  mResource.Seek(SEEK_SET, 0);
 
   /* now we should have a length */
   int64_t resourceLength = GetDataLength();
   gst_app_src_set_size(mSource, resourceLength);
-  if (resource->IsDataCachedToEndOfResource(0) ||
+  if (mResource.GetResource()->IsDataCachedToEndOfResource(0) ||
       (resourceLength != -1 && resourceLength <= SHORT_FILE_SIZE)) {
     /* let the demuxer work in pull mode for local files (or very short files)
      * so that we get optimal seeking accuracy/performance
      */
-    LOG(PR_LOG_DEBUG, "configuring random access, len %lld", resourceLength);
+    LOG(LogLevel::Debug, "configuring random access, len %lld", resourceLength);
     gst_app_src_set_stream_type(mSource, GST_APP_STREAM_TYPE_RANDOM_ACCESS);
   } else {
     /* make the demuxer work in push mode so that seeking is kept to a minimum
      */
-    LOG(PR_LOG_DEBUG, "configuring push mode, len %lld", resourceLength);
+    LOG(LogLevel::Debug, "configuring push mode, len %lld", resourceLength);
     gst_app_src_set_stream_type(mSource, GST_APP_STREAM_TYPE_SEEKABLE);
   }
 
@@ -327,19 +322,17 @@ void GStreamerReader::PlayBinSourceSetup(GstAppSrc* aSource)
  */
 nsresult GStreamerReader::ParseMP3Headers()
 {
-  MediaResource *resource = mDecoder->GetResource();
-
   const uint32_t MAX_READ_BYTES = 4096;
 
   uint64_t offset = 0;
   char bytes[MAX_READ_BYTES];
   uint32_t bytesRead;
   do {
-    nsresult rv = resource->ReadAt(offset, bytes, MAX_READ_BYTES, &bytesRead);
+    nsresult rv = mResource.ReadAt(offset, bytes, MAX_READ_BYTES, &bytesRead);
     NS_ENSURE_SUCCESS(rv, rv);
     NS_ENSURE_TRUE(bytesRead, NS_ERROR_FAILURE);
 
-    mMP3FrameParser.Parse(bytes, bytesRead, offset);
+    mMP3FrameParser.Parse(reinterpret_cast<uint8_t*>(bytes), bytesRead, offset);
     offset += bytesRead;
   } while (!mMP3FrameParser.ParsedHeaders());
 
@@ -358,7 +351,7 @@ nsresult GStreamerReader::ParseMP3Headers()
 int64_t
 GStreamerReader::GetDataLength()
 {
-  int64_t streamLen = mDecoder->GetResource()->GetLength();
+  int64_t streamLen = mResource.GetLength();
 
   if (streamLen < 0) {
     return streamLen;
@@ -370,7 +363,7 @@ GStreamerReader::GetDataLength()
 nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
                                        MetadataTags** aTags)
 {
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
+  MOZ_ASSERT(OnTaskQueue());
   nsresult ret = NS_OK;
 
   /*
@@ -378,6 +371,9 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
    * might be concurrent stream operations happening on both decoding and gstreamer
    * threads which will screw the GStreamer state machine.
    */
+  LOG(LogLevel::Debug, "content-type: %s %s",
+      mDecoder->GetResource()->GetContentType().get(),
+      mDecoder->GetResource()->GetContentURL().get());
   bool isMP3 = mDecoder->GetResource()->GetContentType().EqualsASCII(AUDIO_MP3);
   if (isMP3) {
     ParseMP3Headers();
@@ -425,9 +421,9 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
       gst_object_unref(filter);
     }
 
-    LOG(PR_LOG_DEBUG, "starting metadata pipeline");
+    LOG(LogLevel::Debug, "starting metadata pipeline");
     if (gst_element_set_state(mPlayBin, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
-      LOG(PR_LOG_DEBUG, "metadata pipeline state change failed");
+      LOG(LogLevel::Debug, "metadata pipeline state change failed");
       ret = NS_ERROR_FAILURE;
       continue;
     }
@@ -438,19 +434,19 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
     message = gst_bus_timed_pop_filtered(mBus, GST_CLOCK_TIME_NONE,
                  (GstMessageType)(GST_MESSAGE_ASYNC_DONE | GST_MESSAGE_ERROR | GST_MESSAGE_EOS));
     if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_ASYNC_DONE) {
-      LOG(PR_LOG_DEBUG, "read metadata pipeline prerolled");
+      LOG(LogLevel::Debug, "read metadata pipeline prerolled");
       gst_message_unref(message);
       ret = NS_OK;
       break;
     } else {
-      LOG(PR_LOG_DEBUG, "read metadata pipeline failed to preroll: %s",
+      LOG(LogLevel::Debug, "read metadata pipeline failed to preroll: %s",
             gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
 
       if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_ERROR) {
         GError* error;
         gchar* debug;
         gst_message_parse_error(message, &error, &debug);
-        LOG(PR_LOG_ERROR, "read metadata error: %s: %s", error->message, debug);
+        LOG(LogLevel::Error, "read metadata error: %s: %s", error->message, debug);
         g_error_free(error);
         g_free(debug);
       }
@@ -472,15 +468,17 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
   /* report the duration */
   gint64 duration;
 
+  bool isMediaSeekable = false;
+
   if (isMP3 && mMP3FrameParser.IsMP3()) {
     // The MP3FrameParser has reported a duration; use that over the gstreamer
     // reported duration for inter-platform consistency.
-    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
     mUseParserDuration = true;
     mLastParserDuration = mMP3FrameParser.GetDuration();
-    mDecoder->SetMediaDuration(mLastParserDuration);
+    mInfo.mMetadataDuration.emplace(TimeUnit::FromMicroseconds(mLastParserDuration));
+    isMediaSeekable = true;
   } else {
-    LOG(PR_LOG_DEBUG, "querying duration");
+    LOG(LogLevel::Debug, "querying duration");
     // Otherwise use the gstreamer duration.
 #if GST_VERSION_MAJOR >= 1
     if (gst_element_query_duration(GST_ELEMENT(mPlayBin),
@@ -490,18 +488,24 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
     if (gst_element_query_duration(GST_ELEMENT(mPlayBin),
       &format, &duration) && format == GST_FORMAT_TIME) {
 #endif
-      ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-      LOG(PR_LOG_DEBUG, "have duration %" GST_TIME_FORMAT, GST_TIME_ARGS(duration));
+      LOG(LogLevel::Debug, "have duration %" GST_TIME_FORMAT, GST_TIME_ARGS(duration));
       duration = GST_TIME_AS_USECONDS (duration);
-      mDecoder->SetMediaDuration(duration);
+      mInfo.mMetadataDuration.emplace(TimeUnit::FromMicroseconds(duration));
+      isMediaSeekable = true;
     }
   }
 
+  mInfo.mMediaSeekable = isMediaSeekable;
+
   int n_video = 0, n_audio = 0;
   g_object_get(mPlayBin, "n-video", &n_video, "n-audio", &n_audio, nullptr);
-  mInfo.mVideo.mHasVideo = n_video != 0;
-  mInfo.mAudio.mHasAudio = n_audio != 0;
 
+  if (!n_video) {
+    mInfo.mVideo = VideoInfo();
+  }
+  if (!n_audio) {
+    mInfo.mAudio = AudioInfo();
+  }
   *aInfo = mInfo;
 
   *aTags = nullptr;
@@ -520,28 +524,6 @@ nsresult GStreamerReader::ReadMetadata(MediaInfo* aInfo,
   return NS_OK;
 }
 
-bool
-GStreamerReader::IsMediaSeekable()
-{
-  if (mUseParserDuration) {
-    return true;
-  }
-
-  gint64 duration;
-#if GST_VERSION_MAJOR >= 1
-  if (gst_element_query_duration(GST_ELEMENT(mPlayBin), GST_FORMAT_TIME,
-                                 &duration)) {
-#else
-  GstFormat format = GST_FORMAT_TIME;
-  if (gst_element_query_duration(GST_ELEMENT(mPlayBin), &format, &duration) &&
-      format == GST_FORMAT_TIME) {
-#endif
-    return true;
-  }
-
-  return false;
-}
-
 nsresult GStreamerReader::CheckSupportedFormats()
 {
   bool done = false;
@@ -600,7 +582,6 @@ nsresult GStreamerReader::CheckSupportedFormats()
       }
       case GST_ITERATOR_RESYNC:
         unsupported = false;
-        done = false;
         break;
       case GST_ITERATOR_ERROR:
         done = true;
@@ -611,6 +592,8 @@ nsresult GStreamerReader::CheckSupportedFormats()
     }
   }
 
+  gst_iterator_free(it);
+
   return unsupported ? NS_ERROR_FAILURE : NS_OK;
 }
 
@@ -618,7 +601,7 @@ nsresult GStreamerReader::ResetDecode()
 {
   nsresult res = NS_OK;
 
-  LOG(PR_LOG_DEBUG, "reset decode");
+  LOG(LogLevel::Debug, "reset decode");
 
   if (NS_FAILED(MediaDecoderReader::ResetDecode())) {
     res = NS_ERROR_FAILURE;
@@ -635,14 +618,14 @@ nsresult GStreamerReader::ResetDecode()
   mConfigureAlignment = true;
 #endif
 
-  LOG(PR_LOG_DEBUG, "reset decode done");
+  LOG(LogLevel::Debug, "reset decode done");
 
   return res;
 }
 
 bool GStreamerReader::DecodeAudioData()
 {
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
+  MOZ_ASSERT(OnTaskQueue());
 
   GstBuffer *buffer = nullptr;
 
@@ -728,7 +711,7 @@ bool GStreamerReader::DecodeAudioData()
 bool GStreamerReader::DecodeVideoFrame(bool &aKeyFrameSkip,
                                        int64_t aTimeThreshold)
 {
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
+  MOZ_ASSERT(OnTaskQueue());
 
   GstBuffer *buffer = nullptr;
 
@@ -798,7 +781,7 @@ bool GStreamerReader::DecodeVideoFrame(bool &aKeyFrameSkip,
     duration = gst_util_uint64_scale(GST_USECOND, fpsDen, fpsNum);
 
   if (timestamp < aTimeThreshold) {
-    LOG(PR_LOG_DEBUG, "skipping frame %" GST_TIME_FORMAT
+    LOG(LogLevel::Debug, "skipping frame %" GST_TIME_FORMAT
                       " threshold %" GST_TIME_FORMAT,
                       GST_TIME_ARGS(timestamp * 1000),
                       GST_TIME_ARGS(aTimeThreshold * 1000));
@@ -821,7 +804,7 @@ bool GStreamerReader::DecodeVideoFrame(bool &aKeyFrameSkip,
   }
 #endif
 
-  nsRefPtr<PlanarYCbCrImage> image = GetImageFromBuffer(buffer);
+  RefPtr<PlanarYCbCrImage> image = GetImageFromBuffer(buffer);
   if (!image) {
     /* Ugh, upstream is not calling gst_pad_alloc_buffer(). Fallback to
      * allocating a PlanarYCbCrImage backed GstBuffer here and memcpy.
@@ -832,8 +815,8 @@ bool GStreamerReader::DecodeVideoFrame(bool &aKeyFrameSkip,
     buffer = tmp;
   }
 
-  int64_t offset = mDecoder->GetResource()->Tell(); // Estimate location in media.
-  nsRefPtr<VideoData> video = VideoData::CreateFromImage(mInfo.mVideo,
+  int64_t offset = mResource.Tell(); // Estimate location in media.
+  RefPtr<VideoData> video = VideoData::CreateFromImage(mInfo.mVideo,
                                                          mDecoder->GetImageContainer(),
                                                          offset, timestamp, duration,
                                                          static_cast<Image*>(image.get()),
@@ -845,13 +828,13 @@ bool GStreamerReader::DecodeVideoFrame(bool &aKeyFrameSkip,
   return true;
 }
 
-nsRefPtr<MediaDecoderReader::SeekPromise>
+RefPtr<MediaDecoderReader::SeekPromise>
 GStreamerReader::Seek(int64_t aTarget, int64_t aEndTime)
 {
-  NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
+  MOZ_ASSERT(OnTaskQueue());
 
   gint64 seekPos = aTarget * GST_USECOND;
-  LOG(PR_LOG_DEBUG, "%p About to seek to %" GST_TIME_FORMAT,
+  LOG(LogLevel::Debug, "%p About to seek to %" GST_TIME_FORMAT,
         mDecoder, GST_TIME_ARGS(seekPos));
 
   int flags = GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT;
@@ -859,51 +842,53 @@ GStreamerReader::Seek(int64_t aTarget, int64_t aEndTime)
                                GST_FORMAT_TIME,
                                static_cast<GstSeekFlags>(flags),
                                seekPos)) {
-    LOG(PR_LOG_ERROR, "seek failed");
+    LOG(LogLevel::Error, "seek failed");
     return SeekPromise::CreateAndReject(NS_ERROR_FAILURE, __func__);
   }
-  LOG(PR_LOG_DEBUG, "seek succeeded");
+  LOG(LogLevel::Debug, "seek succeeded");
   GstMessage* message = gst_bus_timed_pop_filtered(mBus, GST_CLOCK_TIME_NONE,
                (GstMessageType)(GST_MESSAGE_ASYNC_DONE | GST_MESSAGE_ERROR));
   gst_message_unref(message);
-  LOG(PR_LOG_DEBUG, "seek completed");
+  LOG(LogLevel::Debug, "seek completed");
 
   return SeekPromise::CreateAndResolve(aTarget, __func__);
 }
 
-nsresult GStreamerReader::GetBuffered(dom::TimeRanges* aBuffered)
+media::TimeIntervals GStreamerReader::GetBuffered()
 {
+  MOZ_ASSERT(OnTaskQueue());
+  if (!HaveStartTime()) {
+    return media::TimeIntervals();
+  }
+  media::TimeIntervals buffered;
   if (!mInfo.HasValidMedia()) {
-    return NS_OK;
+    return buffered;
   }
 
 #if GST_VERSION_MAJOR == 0
   GstFormat format = GST_FORMAT_TIME;
 #endif
   AutoPinned<MediaResource> resource(mDecoder->GetResource());
-  nsTArray<MediaByteRange> ranges;
+  MediaByteRangeSet ranges;
   resource->GetCachedRanges(ranges);
 
   if (resource->IsDataCachedToEndOfResource(0)) {
     /* fast path for local or completely cached files */
-    gint64 duration = 0;
-
-    {
-      ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-      duration = mDecoder->GetMediaDuration();
-    }
-
-    double end = (double) duration / GST_MSECOND;
-    LOG(PR_LOG_DEBUG, "complete range [0, %f] for [0, %li]",
-          end, GetDataLength());
-    aBuffered->Add(0, end);
-    return NS_OK;
+    gint64 duration =
+       mDuration.Ref().refOr(media::TimeUnit::FromMicroseconds(0)).ToMicroseconds();
+    LOG(LogLevel::Debug, "complete range [0, %f] for [0, %li]",
+        (double) duration / GST_MSECOND, GetDataLength());
+    buffered +=
+      media::TimeInterval(media::TimeUnit::FromMicroseconds(0),
+                          media::TimeUnit::FromMicroseconds(duration));
+    return buffered;
   }
 
   for(uint32_t index = 0; index < ranges.Length(); index++) {
     int64_t startOffset = ranges[index].mStart;
     int64_t endOffset = ranges[index].mEnd;
-    gint64 startTime, endTime;
+    gint64 startTime, endTime, duration;
+    bool haveDuration = false;
 
 #if GST_VERSION_MAJOR >= 1
     if (!gst_element_query_convert(GST_ELEMENT(mPlayBin), GST_FORMAT_BYTES,
@@ -912,6 +897,10 @@ nsresult GStreamerReader::GetBuffered(dom::TimeRanges* aBuffered)
     if (!gst_element_query_convert(GST_ELEMENT(mPlayBin), GST_FORMAT_BYTES,
       endOffset, GST_FORMAT_TIME, &endTime))
       continue;
+    if (gst_element_query_duration(GST_ELEMENT(mPlayBin),
+      GST_FORMAT_TIME, &duration)) {
+      haveDuration = true;
+    }
 #else
     if (!gst_element_query_convert(GST_ELEMENT(mPlayBin), GST_FORMAT_BYTES,
       startOffset, &format, &startTime) || format != GST_FORMAT_TIME)
@@ -919,24 +908,35 @@ nsresult GStreamerReader::GetBuffered(dom::TimeRanges* aBuffered)
     if (!gst_element_query_convert(GST_ELEMENT(mPlayBin), GST_FORMAT_BYTES,
       endOffset, &format, &endTime) || format != GST_FORMAT_TIME)
       continue;
+    if (gst_element_query_duration(GST_ELEMENT(mPlayBin),
+      &format, &duration) && format == GST_FORMAT_TIME) {
+      haveDuration = true;
+    }
 #endif
+    // Check that the estimated time doesn't go beyond known duration
+    // as this indicates a buggy gst plugin.
+    if (haveDuration && endTime > duration) {
+      LOG(LogLevel::Debug, "Have duration %" GST_TIME_FORMAT "contradicting endTime %" GST_TIME_FORMAT,
+          GST_TIME_ARGS(duration), GST_TIME_ARGS(endTime));
+      endTime = std::min(endTime, duration);
+    }
 
-    double start = (double) GST_TIME_AS_USECONDS (startTime) / GST_MSECOND;
-    double end = (double) GST_TIME_AS_USECONDS (endTime) / GST_MSECOND;
-    LOG(PR_LOG_DEBUG, "adding range [%f, %f] for [%li %li] size %li",
-          start, end, startOffset, endOffset, GetDataLength());
-    aBuffered->Add(start, end);
+    LOG(LogLevel::Debug, "adding range [%f, %f] for [%li %li] size %li",
+        (double) GST_TIME_AS_USECONDS (startTime) / GST_MSECOND,
+        (double) GST_TIME_AS_USECONDS (endTime) / GST_MSECOND,
+        startOffset, endOffset, GetDataLength());
+    buffered +=
+      media::TimeInterval(media::TimeUnit::FromMicroseconds(GST_TIME_AS_USECONDS(startTime)),
+                          media::TimeUnit::FromMicroseconds(GST_TIME_AS_USECONDS(endTime)));
   }
 
-  return NS_OK;
+  return buffered;
 }
 
 void GStreamerReader::ReadAndPushData(guint aLength)
 {
-  MediaResource* resource = mDecoder->GetResource();
-  NS_ASSERTION(resource, "Decoder has no media resource");
-  int64_t offset1 = resource->Tell();
-  unused << offset1;
+  int64_t offset1 = mResource.Tell();
+  Unused << offset1;
   nsresult rv = NS_OK;
 
   GstBuffer* buffer = gst_buffer_new_and_alloc(aLength);
@@ -949,16 +949,16 @@ void GStreamerReader::ReadAndPushData(guint aLength)
 #endif
   uint32_t size = 0, bytesRead = 0;
   while(bytesRead < aLength) {
-    rv = resource->Read(reinterpret_cast<char*>(data + bytesRead),
-        aLength - bytesRead, &size);
+    rv = mResource.Read(reinterpret_cast<char*>(data + bytesRead),
+                        aLength - bytesRead, &size);
     if (NS_FAILED(rv) || size == 0)
       break;
 
     bytesRead += size;
   }
 
-  int64_t offset2 = resource->Tell();
-  unused << offset2;
+  int64_t offset2 = mResource.Tell();
+  Unused << offset2;
 
 #if GST_VERSION_MAJOR >= 1
   gst_buffer_unmap(buffer, &info);
@@ -969,16 +969,16 @@ void GStreamerReader::ReadAndPushData(guint aLength)
 
   GstFlowReturn ret = gst_app_src_push_buffer(mSource, gst_buffer_ref(buffer));
   if (ret != GST_FLOW_OK) {
-    LOG(PR_LOG_ERROR, "ReadAndPushData push ret %s(%d)", gst_flow_get_name(ret), ret);
+    LOG(LogLevel::Error, "ReadAndPushData push ret %s(%d)", gst_flow_get_name(ret), ret);
   }
 
   if (NS_FAILED(rv)) {
     /* Terminate the stream if there is an error in reading */
-    LOG(PR_LOG_ERROR, "ReadAndPushData read error, rv=%x", rv);
+    LOG(LogLevel::Error, "ReadAndPushData read error, rv=%x", rv);
     gst_app_src_end_of_stream(mSource);
   } else if (bytesRead < aLength) {
     /* If we read less than what we wanted, we reached the end */
-    LOG(PR_LOG_WARNING, "ReadAndPushData read underflow, "
+    LOG(LogLevel::Warning, "ReadAndPushData read underflow, "
         "bytesRead=%u, aLength=%u, offset(%lld,%lld)",
         bytesRead, aLength, offset1, offset2);
     gst_app_src_end_of_stream(mSource);
@@ -1031,8 +1031,7 @@ gboolean GStreamerReader::SeekData(GstAppSrc* aSrc, guint64 aOffset)
   aOffset += mDataOffset;
 
   ReentrantMonitorAutoEnter mon(mGstThreadsMonitor);
-  MediaResource* resource = mDecoder->GetResource();
-  int64_t resourceLength = resource->GetLength();
+  int64_t resourceLength = mResource.GetLength();
 
   if (gst_app_src_get_size(mSource) == -1) {
     /* It's possible that we didn't know the length when we initialized mSource
@@ -1043,20 +1042,14 @@ gboolean GStreamerReader::SeekData(GstAppSrc* aSrc, guint64 aOffset)
 
   nsresult rv = NS_ERROR_FAILURE;
   if (aOffset < static_cast<guint64>(resourceLength)) {
-    rv = resource->Seek(SEEK_SET, aOffset);
-  }
-
-  if (NS_FAILED(rv)) {
-    LOG(PR_LOG_ERROR, "seek at %lu failed", aOffset);
-  } else {
-    MOZ_ASSERT(aOffset == static_cast<guint64>(resource->Tell()));
+    rv = mResource.Seek(SEEK_SET, aOffset);
   }
 
   return NS_SUCCEEDED(rv);
 }
 
 GstFlowReturn GStreamerReader::NewPrerollCb(GstAppSink* aSink,
-                                              gpointer aUserData)
+                                            gpointer aUserData)
 {
   GStreamerReader* reader = reinterpret_cast<GStreamerReader*>(aUserData);
 
@@ -1070,7 +1063,7 @@ GstFlowReturn GStreamerReader::NewPrerollCb(GstAppSink* aSink,
 void GStreamerReader::AudioPreroll()
 {
   /* The first audio buffer has reached the audio sink. Get rate and channels */
-  LOG(PR_LOG_DEBUG, "Audio preroll");
+  LOG(LogLevel::Debug, "Audio preroll");
   GstPad* sinkpad = gst_element_get_static_pad(GST_ELEMENT(mAudioAppSink), "sink");
 #if GST_VERSION_MAJOR >= 1
   GstCaps *caps = gst_pad_get_current_caps(sinkpad);
@@ -1085,7 +1078,6 @@ void GStreamerReader::AudioPreroll()
   NS_ASSERTION(mInfo.mAudio.mChannels != 0, ("audio channels is zero"));
   NS_ASSERTION(mInfo.mAudio.mChannels > 0 && mInfo.mAudio.mChannels <= MAX_CHANNELS,
       "invalid audio channels number");
-  mInfo.mAudio.mHasAudio = true;
   gst_caps_unref(caps);
   gst_object_unref(sinkpad);
 }
@@ -1093,7 +1085,7 @@ void GStreamerReader::AudioPreroll()
 void GStreamerReader::VideoPreroll()
 {
   /* The first video buffer has reached the video sink. Get width and height */
-  LOG(PR_LOG_DEBUG, "Video preroll");
+  LOG(LogLevel::Debug, "Video preroll");
   GstPad* sinkpad = gst_element_get_static_pad(GST_ELEMENT(mVideoAppSink), "sink");
   int PARNumerator, PARDenominator;
 #if GST_VERSION_MAJOR >= 1
@@ -1125,10 +1117,9 @@ void GStreamerReader::VideoPreroll()
   if (IsValidVideoRegion(frameSize, pictureRect, displaySize)) {
     GstStructure* structure = gst_caps_get_structure(caps, 0);
     gst_structure_get_fraction(structure, "framerate", &fpsNum, &fpsDen);
-    mInfo.mVideo.mDisplay = ThebesIntSize(displaySize.ToIntSize());
-    mInfo.mVideo.mHasVideo = true;
+    mInfo.mVideo.mDisplay = displaySize;
   } else {
-    LOG(PR_LOG_DEBUG, "invalid video region");
+    LOG(LogLevel::Debug, "invalid video region");
     Eos();
   }
   gst_caps_unref(caps);
@@ -1193,19 +1184,13 @@ void GStreamerReader::Eos(GstAppSink* aSink)
     }
     mon.NotifyAll();
   }
-
-  {
-    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-    /* Potentially unblock the decode thread in ::DecodeLoop */
-    mon.NotifyAll();
-  }
 }
 
 /**
  * This callback is called while the pipeline is automatically built, after a
  * new element has been added to the pipeline. We use it to find the
  * uridecodebin instance used by playbin and connect to it to apply our
- * blacklist.
+ * block list.
  */
 void
 GStreamerReader::PlayElementAddedCb(GstBin *aBin, GstElement *aElement,
@@ -1242,7 +1227,7 @@ GStreamerReader::ShouldAutoplugFactory(GstElementFactory* aFactory, GstCaps* aCa
 
 /**
  * This is called by uridecodebin (running inside playbin), after it has found
- * candidate factories to continue decoding the stream. We apply the blacklist
+ * candidate factories to continue decoding the stream. We apply the block list
  * here, disallowing known-crashy plugins.
  */
 GValueArray*
@@ -1274,27 +1259,43 @@ GStreamerReader::AutoplugSortCb(GstElement* aElement, GstPad* aPad,
  * If this is an MP3 stream, pass any new data we get to the MP3 frame parser
  * for duration estimation.
  */
-void GStreamerReader::NotifyDataArrived(const char *aBuffer,
-                                        uint32_t aLength,
-                                        int64_t aOffset)
+void GStreamerReader::NotifyDataArrivedInternal()
 {
-  MOZ_ASSERT(NS_IsMainThread());
-
+  MOZ_ASSERT(OnTaskQueue());
   if (HasVideo()) {
     return;
   }
-
   if (!mMP3FrameParser.NeedsData()) {
     return;
   }
 
-  mMP3FrameParser.Parse(aBuffer, aLength, aOffset);
+  AutoPinned<MediaResource> resource(mResource.GetResource());
+  MediaByteRangeSet byteRanges;
+  nsresult rv = resource->GetCachedRanges(byteRanges);
 
+  if (NS_FAILED(rv)) {
+    return;
+  }
+  if (byteRanges == mLastCachedRanges) {
+    return;
+  }
+  MediaByteRangeSet intervals = byteRanges - mLastCachedRanges;
+  mLastCachedRanges = byteRanges;
+
+  for (const auto& interval : intervals) {
+    RefPtr<MediaByteBuffer> bytes =
+      resource->MediaReadAt(interval.mStart, interval.Length());
+    NS_ENSURE_TRUE_VOID(bytes);
+    mMP3FrameParser.Parse(bytes->Elements(), interval.Length(), interval.mStart);
+    if (!mMP3FrameParser.IsMP3()) {
+      return;
+    }
+  }
   int64_t duration = mMP3FrameParser.GetDuration();
   if (duration != mLastParserDuration && mUseParserDuration) {
-    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
+    MOZ_ASSERT(mDecoder);
     mLastParserDuration = duration;
-    mDecoder->UpdateEstimatedMediaDuration(mLastParserDuration);
+    mDecoder->DispatchUpdateEstimatedMediaDuration(mLastParserDuration);
   }
 }
 
@@ -1360,7 +1361,7 @@ GstPadProbeReturn GStreamerReader::EventProbe(GstPad *aPad, GstEvent *aEvent)
 {
   GstElement* parent = GST_ELEMENT(gst_pad_get_parent(aPad));
 
-  LOG(PR_LOG_DEBUG, "event probe %s", GST_EVENT_TYPE_NAME (aEvent));
+  LOG(LogLevel::Debug, "event probe %s", GST_EVENT_TYPE_NAME (aEvent));
 
   switch(GST_EVENT_TYPE(aEvent)) {
     case GST_EVENT_SEGMENT:
@@ -1451,9 +1452,9 @@ void GStreamerReader::ImageDataFromVideoFrame(GstVideoFrame *aFrame,
   aData->mCrSkip = GST_VIDEO_FRAME_COMP_PSTRIDE(aFrame, 2) - 1;
 }
 
-nsRefPtr<PlanarYCbCrImage> GStreamerReader::GetImageFromBuffer(GstBuffer* aBuffer)
+RefPtr<PlanarYCbCrImage> GStreamerReader::GetImageFromBuffer(GstBuffer* aBuffer)
 {
-  nsRefPtr<PlanarYCbCrImage> image = nullptr;
+  RefPtr<PlanarYCbCrImage> image = nullptr;
 
   if (gst_buffer_n_memory(aBuffer) == 1) {
     GstMemory* mem = gst_buffer_peek_memory(aBuffer, 0);
@@ -1474,7 +1475,7 @@ nsRefPtr<PlanarYCbCrImage> GStreamerReader::GetImageFromBuffer(GstBuffer* aBuffe
 
 void GStreamerReader::CopyIntoImageBuffer(GstBuffer* aBuffer,
                                           GstBuffer** aOutBuffer,
-                                          nsRefPtr<PlanarYCbCrImage> &image)
+                                          RefPtr<PlanarYCbCrImage> &image)
 {
   *aOutBuffer = gst_buffer_new_allocate(mAllocator, gst_buffer_get_size(aBuffer), nullptr);
   GstMemory *mem = gst_buffer_peek_memory(*aOutBuffer, 0);
