@@ -21,8 +21,7 @@ namespace mozilla {
 class RtspOmxDecoder : public MediaDecoder
 {
 public:
-  RtspOmxDecoder()
-  : MediaDecoder() {
+  explicit RtspOmxDecoder(MediaDecoderOwner* aOwner) : MediaDecoder(aOwner) {
     MOZ_COUNT_CTOR(RtspOmxDecoder);
   }
 
@@ -30,10 +29,9 @@ public:
     MOZ_COUNT_DTOR(RtspOmxDecoder);
   }
 
-  virtual MediaDecoder* Clone() override final;
+  virtual MediaDecoder* Clone(MediaDecoderOwner* aOwner) override final;
   virtual MediaDecoderStateMachine* CreateStateMachine() override final;
-  virtual void ApplyStateToStateMachine(PlayState aState)
-                                        override final;
+  virtual void ChangeState(PlayState aState) override final;
 };
 
 } // namespace mozilla
