@@ -1,5 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,13 +9,12 @@
 
 #include "mozilla/dom/DOMError.h"
 
-class nsIDOMMozMmsMessage;
-class nsIDOMMozSmsMessage;
-
 namespace mozilla {
 namespace dom {
 
-class OwningMozSmsMessageOrMozMmsMessage;
+class MmsMessage;
+class OwningSmsMessageOrMmsMessage;
+class SmsMessage;
 
 class DOMMobileMessageError final : public DOMError
 {
@@ -24,21 +23,21 @@ public:
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(DOMMobileMessageError, DOMError)
 
   DOMMobileMessageError(nsPIDOMWindow* aWindow, const nsAString& aName,
-                        nsIDOMMozSmsMessage* aSms);
+                        SmsMessage* aSms);
 
   DOMMobileMessageError(nsPIDOMWindow* aWindow, const nsAString& aName,
-                        nsIDOMMozMmsMessage* aMms);
+                        MmsMessage* aMms);
 
   virtual JSObject*
-  WrapObject(JSContext* aCx) override;
+  WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;
 
-  void GetData(OwningMozSmsMessageOrMozMmsMessage& aRetVal) const;
+  void GetData(OwningSmsMessageOrMmsMessage& aRetVal) const;
 
 private:
   ~DOMMobileMessageError() {}
 
-  nsCOMPtr<nsIDOMMozSmsMessage> mSms;
-  nsCOMPtr<nsIDOMMozMmsMessage> mMms;
+  RefPtr<SmsMessage> mSms;
+  RefPtr<MmsMessage> mMms;
 };
 
 } // namespace dom
