@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -24,7 +26,7 @@ namespace dom {
 namespace telephony {
 
 class TelephonyDialCallback final : public TelephonyCallback,
-                                        public nsITelephonyDialCallback
+                                    public nsITelephonyDialCallback
 {
 public:
   NS_DECL_ISUPPORTS_INHERITED
@@ -43,10 +45,10 @@ private:
 
 
   nsCOMPtr<nsPIDOMWindow> mWindow;
-  nsRefPtr<Telephony> mTelephony;
+  RefPtr<Telephony> mTelephony;
 
   nsString mServiceCode;
-  nsRefPtr<MMICall> mMMICall;
+  RefPtr<MMICall> mMMICall;
 };
 
 } // namespace telephony
