@@ -7,7 +7,6 @@
 #define nsHTMLEditor_h__
 
 #include "nsCOMPtr.h"
-#include "nsCOMArray.h"
 #include "nsPlaintextEditor.h"
 #include "nsIEditor.h"
 #include "nsIHTMLEditor.h"
@@ -41,13 +40,12 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/dom/Element.h"
 
+class nsDocumentFragment;
 class nsIDOMKeyEvent;
 class nsITransferable;
-class nsIDocumentEncoder;
 class nsIClipboard;
 class TypeInState;
 class nsIContentFilter;
-class nsIURL;
 class nsILinkHandler;
 class nsTableOuterFrame;
 class nsIDOMRange;
@@ -55,6 +53,10 @@ class nsRange;
 struct PropItem;
 
 namespace mozilla {
+template<class T> class OwningNonNull;
+namespace dom {
+class DocumentFragment;
+} // namespace dom
 namespace widget {
 struct IMEState;
 } // namespace widget
@@ -62,17 +64,17 @@ struct IMEState;
 
 /**
  * The HTML editor implementation.<br>
- * Use to edit HTML document represented as a DOM tree. 
+ * Use to edit HTML document represented as a DOM tree.
  */
 class nsHTMLEditor final : public nsPlaintextEditor,
-                               public nsIHTMLEditor,
-                               public nsIHTMLObjectResizer,
-                               public nsIHTMLAbsPosEditor,
-                               public nsITableEditor,
-                               public nsIHTMLInlineTableEditor,
-                               public nsIEditorStyleSheets,
-                               public nsICSSLoaderObserver,
-                               public nsStubMutationObserver
+                           public nsIHTMLEditor,
+                           public nsIHTMLObjectResizer,
+                           public nsIHTMLAbsPosEditor,
+                           public nsITableEditor,
+                           public nsIHTMLInlineTableEditor,
+                           public nsIEditorStyleSheets,
+                           public nsICSSLoaderObserver,
+                           public nsStubMutationObserver
 {
   typedef enum {eNoOp, eReplaceParent=1, eInsertParent=2} BlockTransformationType;
 
@@ -89,7 +91,7 @@ public:
   // see nsIHTMLEditor for documentation
 
 //Interfaces for addref and release and queryinterface
-//NOTE macro used is for classes that inherit from 
+//NOTE macro used is for classes that inherit from
 // another class. Only the base class should use NS_DECL_ISUPPORTS
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(nsHTMLEditor, nsPlaintextEditor)
@@ -193,14 +195,14 @@ public:
                            int32_t aRowIndex, int32_t aColIndex,
                            nsIDOMElement **aCell,
                            int32_t* aStartRowIndex, int32_t* aStartColIndex,
-                           int32_t* aRowSpan, int32_t* aColSpan, 
-                           int32_t* aActualRowSpan, int32_t* aActualColSpan, 
+                           int32_t* aRowSpan, int32_t* aColSpan,
+                           int32_t* aActualRowSpan, int32_t* aActualColSpan,
                            bool* aIsSelected) override;
   NS_IMETHOD GetFirstRow(nsIDOMElement* aTableElement, nsIDOMNode** aRowNode) override;
   NS_IMETHOD GetNextRow(nsIDOMNode* aCurrentRowNode, nsIDOMNode** aRowNode) override;
   nsresult GetLastCellInRow(nsIDOMNode* aRowNode, nsIDOMNode** aCellNode);
 
-  NS_IMETHOD SetSelectionAfterTableEdit(nsIDOMElement* aTable, int32_t aRow, int32_t aCol, 
+  NS_IMETHOD SetSelectionAfterTableEdit(nsIDOMElement* aTable, int32_t aRow, int32_t aCol,
                                         int32_t aDirection, bool aSelected) override;
   NS_IMETHOD GetSelectedOrParentTableElement(nsAString& aTagName,
                                              int32_t *aSelectedCount,
@@ -211,7 +213,7 @@ public:
 
   // Finds the first selected cell in first range of selection
   // This is in the *order of selection*, not order in the table
-  // (i.e., each cell added to selection is added in another range 
+  // (i.e., each cell added to selection is added in another range
   //  in the selection's rangelist, independent of location in table)
   // aRange is optional: returns the range around the cell
   NS_IMETHOD GetFirstSelectedCell(nsIDOMRange **aRange, nsIDOMElement **aCell) override;
@@ -221,7 +223,7 @@ public:
 
   // Upper-left-most selected cell in table
   NS_IMETHOD GetFirstSelectedCellInTable(int32_t *aRowIndex, int32_t *aColIndex, nsIDOMElement **aCell) override;
-    
+
   /* miscellaneous */
   // This sets background on the appropriate container element (table, cell,)
   //   or calls into nsTextEditor to set the page background
@@ -229,8 +231,9 @@ public:
   nsresult SetHTMLBackgroundColor(const nsAString& aColor);
 
   /* ------------ Block methods moved from nsEditor -------------- */
-  static already_AddRefed<mozilla::dom::Element> GetBlockNodeParent(nsINode* aNode);
-  static already_AddRefed<nsIDOMNode> GetBlockNodeParent(nsIDOMNode *aNode);
+  static mozilla::dom::Element* GetBlockNodeParent(nsINode* aNode);
+  static nsIDOMNode* GetBlockNodeParent(nsIDOMNode* aNode);
+  static mozilla::dom::Element* GetBlock(nsINode& aNode);
 
   void IsNextCharInNodeWhitespace(nsIContent* aContent,
                                   int32_t aOffset,
@@ -257,7 +260,7 @@ public:
 
   /** Internal, static version */
   // aElement must not be null.
-  static bool NodeIsBlockStatic(const mozilla::dom::Element* aElement);
+  static bool NodeIsBlockStatic(const nsINode* aElement);
   static nsresult NodeIsBlockStatic(nsIDOMNode *aNode, bool *aIsBlock);
 protected:
   virtual ~nsHTMLEditor();
@@ -288,7 +291,7 @@ public:
   /** returns true if aParentTag can contain a child of type aChildTag */
   virtual bool TagCanContainTag(nsIAtom& aParentTag, nsIAtom& aChildTag)
     override;
-  
+
   /** returns true if aNode is a container */
   virtual bool IsContainer(nsINode* aNode) override;
   virtual bool IsContainer(nsIDOMNode* aNode) override;
@@ -335,12 +338,12 @@ public:
 
   /* ------------ Utility Routines, not part of public API -------------- */
   NS_IMETHOD TypedText(const nsAString& aString, ETypingAction aAction) override;
-  nsresult InsertNodeAtPoint( nsIDOMNode *aNode, 
-                              nsCOMPtr<nsIDOMNode> *ioParent, 
-                              int32_t *ioOffset, 
+  nsresult InsertNodeAtPoint( nsIDOMNode *aNode,
+                              nsCOMPtr<nsIDOMNode> *ioParent,
+                              int32_t *ioOffset,
                               bool aNoEmptyNodes);
 
-  // Use this to assure that selection is set after attribute nodes when 
+  // Use this to assure that selection is set after attribute nodes when
   //  trying to collapse selection at begining of a block node
   //  e.g., when setting at beginning of a table cell
   // This will stop at a table, however, since we don't want to
@@ -356,7 +359,7 @@ public:
   nsresult IsVisTextNode(nsIContent* aNode,
                          bool* outIsEmptyNode,
                          bool aSafeToAskFrames);
-  nsresult IsEmptyNode(nsIDOMNode *aNode, bool *outIsEmptyBlock, 
+  nsresult IsEmptyNode(nsIDOMNode *aNode, bool *outIsEmptyBlock,
                        bool aMozBRDoesntCount = false,
                        bool aListOrCellNotEmpty = false,
                        bool aSafeToAskFrames = false);
@@ -365,7 +368,7 @@ public:
                        bool aListOrCellNotEmpty = false,
                        bool aSafeToAskFrames = false);
   nsresult IsEmptyNodeImpl(nsINode* aNode,
-                           bool *outIsEmptyBlock, 
+                           bool *outIsEmptyBlock,
                            bool aMozBRDoesntCount,
                            bool aListOrCellNotEmpty,
                            bool aSafeToAskFrames,
@@ -425,14 +428,14 @@ protected:
   NS_IMETHOD TabInTable(bool inIsShift, bool *outHandled);
   already_AddRefed<mozilla::dom::Element> CreateBR(nsINode* aNode,
       int32_t aOffset, EDirection aSelect = eNone);
-  NS_IMETHOD CreateBR(nsIDOMNode *aNode, int32_t aOffset, 
+  NS_IMETHOD CreateBR(nsIDOMNode *aNode, int32_t aOffset,
                       nsCOMPtr<nsIDOMNode> *outBRNode, nsIEditor::EDirection aSelect = nsIEditor::eNone) override;
 
 // Table Editing (implemented in nsTableEditor.cpp)
 
   // Table utilities
 
-  // Insert a new cell after or before supplied aCell. 
+  // Insert a new cell after or before supplied aCell.
   //  Optional: If aNewCell supplied, returns the newly-created cell (addref'd, of course)
   // This doesn't change or use the current selection
   NS_IMETHOD InsertCell(nsIDOMElement *aCell, int32_t aRowSpan, int32_t aColSpan,
@@ -472,7 +475,7 @@ protected:
                           nsIDOMNode** aCellParent, int32_t* aCellOffset,
                           int32_t* aRowIndex, int32_t* aColIndex);
 
-  NS_IMETHOD GetCellSpansAt(nsIDOMElement* aTable, int32_t aRowIndex, int32_t aColIndex, 
+  NS_IMETHOD GetCellSpansAt(nsIDOMElement* aTable, int32_t aRowIndex, int32_t aColIndex,
                             int32_t& aActualRowSpan, int32_t& aActualColSpan);
 
   NS_IMETHOD SplitCellIntoColumns(nsIDOMElement *aTable, int32_t aRowIndex, int32_t aColIndex,
@@ -492,13 +495,12 @@ protected:
   nsresult SetSelectionAtDocumentStart(mozilla::dom::Selection* aSelection);
 
 // End of Table Editing utilities
-  
-  static already_AddRefed<mozilla::dom::Element>
-    GetEnclosingTable(nsINode* aNode);
-  static nsCOMPtr<nsIDOMNode> GetEnclosingTable(nsIDOMNode *aNode);
+
+  static mozilla::dom::Element* GetEnclosingTable(nsINode* aNode);
+  static nsIDOMNode* GetEnclosingTable(nsIDOMNode *aNode);
 
   /** content-based query returns true if <aProperty aAttribute=aValue> effects aNode
-    * If <aProperty aAttribute=aValue> contains aNode, 
+    * If <aProperty aAttribute=aValue> contains aNode,
     * but <aProperty aAttribute=SomeOtherValue> also contains aNode and the second is
     * more deeply nested than the first, then the first does not effect aNode.
     *
@@ -513,7 +515,7 @@ protected:
     *
     * The nsIContent variant returns aIsSet instead of using an out parameter.
     */
-  bool IsTextPropertySetByContent(nsIContent*      aContent,
+  bool IsTextPropertySetByContent(nsINode*         aNode,
                                   nsIAtom*         aProperty,
                                   const nsAString* aAttribute,
                                   const nsAString* aValue,
@@ -550,11 +552,12 @@ protected:
 
   // factored methods for handling insertion of data from transferables (drag&drop or clipboard)
   NS_IMETHOD PrepareTransferable(nsITransferable **transferable) override;
-  nsresult PrepareHTMLTransferable(nsITransferable **transferable, bool havePrivFlavor);
-  nsresult InsertFromTransferable(nsITransferable *transferable, 
+  nsresult PrepareHTMLTransferable(nsITransferable **transferable);
+  nsresult InsertFromTransferable(nsITransferable *transferable,
                                     nsIDOMDocument *aSourceDoc,
                                     const nsAString & aContextStr,
                                     const nsAString & aInfoStr,
+                                    bool havePrivateHTMLFlavor,
                                     nsIDOMNode *aDestinationNode,
                                     int32_t aDestinationOffset,
                                     bool aDoDeleteSelection);
@@ -569,17 +572,17 @@ protected:
   nsresult   DoContentFilterCallback(const nsAString &aFlavor,
                                      nsIDOMDocument *aSourceDoc,
                                      bool aWillDeleteSelection,
-                                     nsIDOMNode **aFragmentAsNode,      
+                                     nsIDOMNode **aFragmentAsNode,
                                      nsIDOMNode **aFragStartNode,
                                      int32_t *aFragStartOffset,
                                      nsIDOMNode **aFragEndNode,
                                      int32_t *aFragEndOffset,
-                                     nsIDOMNode **aTargetNode,       
-                                     int32_t *aTargetOffset,   
+                                     nsIDOMNode **aTargetNode,
+                                     int32_t *aTargetOffset,
                                      bool *aDoContinue);
 
   bool       IsInLink(nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *outLink = nullptr);
-  nsresult   StripFormattingNodes(nsIDOMNode *aNode, bool aOnlyList = false);
+  nsresult   StripFormattingNodes(nsIContent& aNode, bool aOnlyList = false);
   nsresult   CreateDOMFragmentFromPaste(const nsAString & aInputString,
                                         const nsAString & aContextStr,
                                         const nsAString & aInfoStr,
@@ -591,37 +594,35 @@ protected:
                                         bool aTrustedInput);
   nsresult   ParseFragment(const nsAString & aStr, nsIAtom* aContextLocalName,
                            nsIDocument* aTargetDoc,
-                           nsCOMPtr<nsIDOMNode> *outNode,
+                           mozilla::dom::DocumentFragment** aFragment,
                            bool aTrustedInput);
-  nsresult   CreateListOfNodesToPaste(nsIDOMNode  *aFragmentAsNode,
-                                      nsCOMArray<nsIDOMNode>& outNodeList,
-                                      nsIDOMNode *aStartNode,
+  void       CreateListOfNodesToPaste(mozilla::dom::DocumentFragment& aFragment,
+                                      nsTArray<mozilla::OwningNonNull<nsINode>>& outNodeList,
+                                      nsINode* aStartNode,
                                       int32_t aStartOffset,
-                                      nsIDOMNode *aEndNode,
+                                      nsINode* aEndNode,
                                       int32_t aEndOffset);
   nsresult CreateTagStack(nsTArray<nsString> &aTagStack,
                           nsIDOMNode *aNode);
-  nsresult GetListAndTableParents( bool aEnd, 
-                                   nsCOMArray<nsIDOMNode>& aListOfNodes,
-                                   nsCOMArray<nsIDOMNode>& outArray);
-  nsresult DiscoverPartialListsAndTables(nsCOMArray<nsIDOMNode>& aPasteNodes,
-                                         nsCOMArray<nsIDOMNode>& aListsAndTables,
-                                         int32_t *outHighWaterMark);
-  nsresult ScanForListAndTableStructure(bool aEnd,
-                                        nsCOMArray<nsIDOMNode>& aNodes,
-                                        nsIDOMNode *aListOrTable,
-                                        nsCOMPtr<nsIDOMNode> *outReplaceNode);
-  nsresult ReplaceOrphanedStructure( bool aEnd,
-                                     nsCOMArray<nsIDOMNode>& aNodeArray,
-                                     nsCOMArray<nsIDOMNode>& aListAndTableArray,
-                                     int32_t aHighWaterMark);
-  nsIDOMNode* GetArrayEndpoint(bool aEnd, nsCOMArray<nsIDOMNode>& aNodeArray);
+  enum class StartOrEnd { start, end };
+  void GetListAndTableParents(StartOrEnd aStartOrEnd,
+                              nsTArray<mozilla::OwningNonNull<nsINode>>& aNodeList,
+                              nsTArray<mozilla::OwningNonNull<mozilla::dom::Element>>& outArray);
+  int32_t DiscoverPartialListsAndTables(nsTArray<mozilla::OwningNonNull<nsINode>>& aPasteNodes,
+                                        nsTArray<mozilla::OwningNonNull<mozilla::dom::Element>>& aListsAndTables);
+  nsINode* ScanForListAndTableStructure(StartOrEnd aStartOrEnd,
+                                        nsTArray<mozilla::OwningNonNull<nsINode>>& aNodes,
+                                        mozilla::dom::Element& aListOrTable);
+  void ReplaceOrphanedStructure(StartOrEnd aStartOrEnd,
+                                nsTArray<mozilla::OwningNonNull<nsINode>>& aNodeArray,
+                                nsTArray<mozilla::OwningNonNull<mozilla::dom::Element>>& aListAndTableArray,
+                                int32_t aHighWaterMark);
 
   /* small utility routine to test if a break node is visible to user */
   bool     IsVisBreak(nsINode* aNode);
   bool     IsVisBreak(nsIDOMNode *aNode);
 
-  /* utility routine to possibly adjust the insertion position when 
+  /* utility routine to possibly adjust the insertion position when
      inserting a block level element */
   void NormalizeEOLInsertPosition(nsIDOMNode *firstNodeToInsert,
                                   nsCOMPtr<nsIDOMNode> *insertParentNode,
@@ -633,51 +634,53 @@ protected:
   /* helpers for block transformations */
   nsresult MakeDefinitionItem(const nsAString & aItemType);
   nsresult InsertBasicBlock(const nsAString & aBlockType);
-  
+
   /* increase/decrease the font size of selection */
-  nsresult RelativeFontChange( int32_t aSizeChange);
-  
+  enum class FontSize { incr, decr };
+  nsresult RelativeFontChange(FontSize aDir);
+
   /* helper routines for font size changing */
-  nsresult RelativeFontChangeOnTextNode( int32_t aSizeChange, 
-                                         nsIDOMCharacterData *aTextNode, 
+  nsresult RelativeFontChangeOnTextNode( int32_t aSizeChange,
+                                         nsIDOMCharacterData *aTextNode,
                                          int32_t aStartOffset,
                                          int32_t aEndOffset);
   nsresult RelativeFontChangeOnNode(int32_t aSizeChange, nsIContent* aNode);
   nsresult RelativeFontChangeHelper(int32_t aSizeChange, nsINode* aNode);
 
   /* helper routines for inline style */
-  nsresult SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode, 
-                                        int32_t aStartOffset,
-                                        int32_t aEndOffset,
-                                        nsIAtom *aProperty, 
-                                        const nsAString *aAttribute,
-                                        const nsAString *aValue);
-  nsresult SetInlinePropertyOnNode( nsIDOMNode *aNode,
-                                    nsIAtom *aProperty, 
-                                    const nsAString *aAttribute,
-                                    const nsAString *aValue);
-  nsresult SetInlinePropertyOnNode(nsIContent* aNode,
-                                   nsIAtom* aProperty,
+  nsresult SetInlinePropertyOnTextNode(mozilla::dom::Text& aData,
+                                       int32_t aStartOffset,
+                                       int32_t aEndOffset,
+                                       nsIAtom& aProperty,
+                                       const nsAString* aAttribute,
+                                       const nsAString& aValue);
+  nsresult SetInlinePropertyOnNode(nsIContent& aNode,
+                                   nsIAtom& aProperty,
                                    const nsAString* aAttribute,
-                                   const nsAString* aValue);
+                                   const nsAString& aValue);
 
   nsresult PromoteInlineRange(nsRange* aRange);
   nsresult PromoteRangeIfStartsOrEndsInNamedAnchor(nsRange* aRange);
   nsresult SplitStyleAboveRange(nsRange* aRange,
-                                nsIAtom *aProperty, 
+                                nsIAtom *aProperty,
                                 const nsAString *aAttribute);
   nsresult SplitStyleAbovePoint(nsCOMPtr<nsIDOMNode> *aNode,
                                 int32_t *aOffset,
-                                nsIAtom *aProperty, 
+                                nsIAtom *aProperty,
                                 const nsAString *aAttribute,
                                 nsCOMPtr<nsIDOMNode> *outLeftNode = nullptr,
                                 nsCOMPtr<nsIDOMNode> *outRightNode = nullptr);
   nsresult ApplyDefaultProperties();
-  nsresult RemoveStyleInside(nsIDOMNode *aNode, 
-                             nsIAtom *aProperty, 
-                             const nsAString *aAttribute, 
+  nsresult RemoveStyleInside(nsIDOMNode *aNode,
+                             nsIAtom *aProperty,
+                             const nsAString *aAttribute,
                              const bool aChildrenOnly = false);
-  nsresult RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAString *aAttribute);
+  nsresult RemoveStyleInside(nsIContent& aNode,
+                             nsIAtom* aProperty,
+                             const nsAString* aAttribute,
+                             const bool aChildrenOnly = false);
+  nsresult RemoveInlinePropertyImpl(nsIAtom* aProperty,
+                                    const nsAString* aAttribute);
 
   bool NodeIsProperty(nsIDOMNode *aNode);
   bool HasAttr(nsIDOMNode *aNode, const nsAString *aAttribute);
@@ -718,16 +721,16 @@ protected:
   nsIContent* GetFirstEditableLeaf(nsINode& aNode);
   nsIContent* GetLastEditableLeaf(nsINode& aNode);
 
-  nsresult GetInlinePropertyBase(nsIAtom *aProperty, 
-                             const nsAString *aAttribute,
-                             const nsAString *aValue,
-                             bool *aFirst, 
-                             bool *aAny, 
-                             bool *aAll,
-                             nsAString *outValue,
-                             bool aCheckDefaults = true);
+  nsresult GetInlinePropertyBase(nsIAtom& aProperty,
+                                 const nsAString* aAttribute,
+                                 const nsAString* aValue,
+                                 bool* aFirst,
+                                 bool* aAny,
+                                 bool* aAll,
+                                 nsAString* outValue,
+                                 bool aCheckDefaults = true);
   bool HasStyleOrIdOrClass(mozilla::dom::Element* aElement);
-  nsresult RemoveElementIfNoStyleOrIdOrClass(nsIDOMNode* aElement);
+  nsresult RemoveElementIfNoStyleOrIdOrClass(mozilla::dom::Element& aElement);
 
   // Whether the outer window of the DOM event target has focus or not.
   bool     OurWindowHasFocus();
@@ -756,12 +759,15 @@ protected:
   nsresult ClearStyle(nsCOMPtr<nsIDOMNode>* aNode, int32_t* aOffset,
                       nsIAtom* aProperty, const nsAString* aAttribute);
 
+  void SetElementPosition(mozilla::dom::Element& aElement,
+                          int32_t aX, int32_t aY);
+
 // Data members
 protected:
 
-  nsCOMArray<nsIContentFilter> mContentFilters;
+  nsTArray<mozilla::OwningNonNull<nsIContentFilter>> mContentFilters;
 
-  nsRefPtr<TypeInState>        mTypeInState;
+  RefPtr<TypeInState>        mTypeInState;
 
   bool mCRInParagraphCreatesParagraph;
 
@@ -776,8 +782,8 @@ protected:
 
   // Maintain a list of associated style sheets and their urls.
   nsTArray<nsString> mStyleSheetURLs;
-  nsTArray<nsRefPtr<mozilla::CSSStyleSheet>> mStyleSheets;
-  
+  nsTArray<RefPtr<mozilla::CSSStyleSheet>> mStyleSheets;
+
   // an array for holding default style settings
   nsTArray<PropItem*> mDefaultStyles;
 
@@ -787,7 +793,7 @@ protected:
   void     RemoveListenerAndDeleteRef(const nsAString& aEvent,
                                       nsIDOMEventListener* aListener,
                                       bool aUseCapture,
-                                      nsIDOMElement* aElement,
+                                      mozilla::dom::Element* aElement,
                                       nsIContent* aParentContent,
                                       nsIPresShell* aShell);
   void     DeleteRefToAnonymousNode(nsIDOMElement* aElement,
@@ -809,6 +815,10 @@ protected:
   /* PACKED BOOLEANS FOR RESIZING, ABSOLUTE POSITIONING AND */
   /* INLINE TABLE EDITING */
 
+  bool IsInObservedSubtree(nsIDocument* aDocument,
+                           nsIContent* aContainer,
+                           nsIContent* aChild);
+
   // resizing
   bool mIsObjectResizingEnabled;
   bool mIsResizing;
@@ -829,27 +839,27 @@ protected:
 
   /* RESIZING */
 
-  nsCOMPtr<nsIDOMElement> mTopLeftHandle;
-  nsCOMPtr<nsIDOMElement> mTopHandle;
-  nsCOMPtr<nsIDOMElement> mTopRightHandle;
-  nsCOMPtr<nsIDOMElement> mLeftHandle;
-  nsCOMPtr<nsIDOMElement> mRightHandle;
-  nsCOMPtr<nsIDOMElement> mBottomLeftHandle;
-  nsCOMPtr<nsIDOMElement> mBottomHandle;
-  nsCOMPtr<nsIDOMElement> mBottomRightHandle;
+  nsCOMPtr<mozilla::dom::Element> mTopLeftHandle;
+  nsCOMPtr<mozilla::dom::Element> mTopHandle;
+  nsCOMPtr<mozilla::dom::Element> mTopRightHandle;
+  nsCOMPtr<mozilla::dom::Element> mLeftHandle;
+  nsCOMPtr<mozilla::dom::Element> mRightHandle;
+  nsCOMPtr<mozilla::dom::Element> mBottomLeftHandle;
+  nsCOMPtr<mozilla::dom::Element> mBottomHandle;
+  nsCOMPtr<mozilla::dom::Element> mBottomRightHandle;
 
-  nsCOMPtr<nsIDOMElement> mActivatedHandle;
+  nsCOMPtr<mozilla::dom::Element> mActivatedHandle;
 
-  nsCOMPtr<nsIDOMElement> mResizingShadow;
-  nsCOMPtr<nsIDOMElement> mResizingInfo;
+  nsCOMPtr<mozilla::dom::Element> mResizingShadow;
+  nsCOMPtr<mozilla::dom::Element> mResizingInfo;
 
-  nsCOMPtr<nsIDOMElement> mResizedObject;
+  nsCOMPtr<mozilla::dom::Element> mResizedObject;
 
   nsCOMPtr<nsIDOMEventListener>  mMouseMotionListenerP;
   nsCOMPtr<nsISelectionListener> mSelectionListenerP;
   nsCOMPtr<nsIDOMEventListener>  mResizeEventListenerP;
 
-  nsCOMArray<nsIHTMLObjectResizeListener> objectResizeEventListeners;
+  nsTArray<mozilla::OwningNonNull<nsIHTMLObjectResizeListener>> mObjectResizeEventListeners;
 
   int32_t mOriginalX;
   int32_t mOriginalY;
@@ -874,17 +884,18 @@ protected:
 
   nsresult SetAllResizersPosition();
 
-  nsresult CreateResizer(nsIDOMElement ** aReturn, int16_t aLocation, nsIDOMNode * aParentNode);
+  already_AddRefed<mozilla::dom::Element>
+    CreateResizer(int16_t aLocation, nsIDOMNode* aParentNode);
   void     SetAnonymousElementPosition(int32_t aX, int32_t aY, nsIDOMElement *aResizer);
 
-  nsresult CreateShadow(nsIDOMElement ** aReturn, nsIDOMNode * aParentNode,
-                        nsIDOMElement * aOriginalObject);
-  nsresult SetShadowPosition(nsIDOMElement * aShadow,
-                             nsIDOMElement * aOriginalObject,
+  already_AddRefed<mozilla::dom::Element>
+    CreateShadow(nsIDOMNode* aParentNode, nsIDOMElement* aOriginalObject);
+  nsresult SetShadowPosition(mozilla::dom::Element* aShadow,
+                             mozilla::dom::Element* aOriginalObject,
                              int32_t aOriginalObjectX,
                              int32_t aOriginalObjectY);
 
-  nsresult CreateResizingInfo(nsIDOMElement ** aReturn, nsIDOMNode * aParentNode);
+  already_AddRefed<mozilla::dom::Element> CreateResizingInfo(nsIDOMNode* aParentNode);
   nsresult SetResizingInfoPosition(int32_t aX, int32_t aY,
                                    int32_t aW, int32_t aH);
 
@@ -912,13 +923,13 @@ protected:
   int32_t mPositionedObjectBorderLeft;
   int32_t mPositionedObjectBorderTop;
 
-  nsCOMPtr<nsIDOMElement> mAbsolutelyPositionedObject;
-  nsCOMPtr<nsIDOMElement> mGrabber;
-  nsCOMPtr<nsIDOMElement> mPositioningShadow;
+  nsCOMPtr<mozilla::dom::Element> mAbsolutelyPositionedObject;
+  nsCOMPtr<mozilla::dom::Element> mGrabber;
+  nsCOMPtr<mozilla::dom::Element> mPositioningShadow;
 
   int32_t      mGridSize;
 
-  nsresult CreateGrabber(nsIDOMNode * aParentNode, nsIDOMElement ** aReturn);
+  already_AddRefed<mozilla::dom::Element> CreateGrabber(nsINode* aParentNode);
   nsresult StartMoving(nsIDOMElement * aHandle);
   nsresult SetFinalPosition(int32_t aX, int32_t aY);
   void     AddPositioningOffset(int32_t & aX, int32_t & aY);
@@ -959,10 +970,10 @@ private:
                               nsIAtom* aProperty,
                               const nsAString* aAttribute,
                               const nsAString* aValue);
-  nsresult SetInlinePropertyOnNodeImpl(nsIContent* aNode,
-                                       nsIAtom* aProperty,
+  nsresult SetInlinePropertyOnNodeImpl(nsIContent& aNode,
+                                       nsIAtom& aProperty,
                                        const nsAString* aAttribute,
-                                       const nsAString* aValue);
+                                       const nsAString& aValue);
   typedef enum { eInserted, eAppended } InsertedOrAppended;
   void DoContentInserted(nsIDocument* aDocument, nsIContent* aContainer,
                          nsIContent* aChild, int32_t aIndexInContainer,
