@@ -22,13 +22,13 @@ public:
   NS_IMETHOD Convert(const char* aSrc,
                      int32_t* aSrcLength,
                      char16_t* aDest,
-                     int32_t* aDestLength);
+                     int32_t* aDestLength) override;
 
-  NS_IMETHOD GetMaxLength(const char* aSrc,
-                          int32_t aSrcLength,
-                          int32_t* aDestLength);
+  MOZ_WARN_UNUSED_RESULT NS_IMETHOD GetMaxLength(const char* aSrc,
+                                                 int32_t aSrcLength,
+                                                 int32_t* aDestLength) override;
 
-  NS_IMETHOD Reset();
+  NS_IMETHOD Reset() override;
 
 private:
   bool mSeenByte;
