@@ -81,4 +81,4 @@ cp $1/lib/books/floor/floor_books.h ./lib/books/floor/
 cp $1/lib/books/uncoupled/res_books_uncoupled.h ./lib/books/uncoupled/
 
 # Add any patches against upstream here.
-patch -p3 < ./bug1117571-r19419.patch
+# ...nothing to apply...
