@@ -9,6 +9,7 @@
  */
 
 #include "./vpx_config.h"
+#include "vpx_ports/mem.h"
 #include "vp9/common/vp9_common.h"
 #include "vp9/common/vp9_loopfilter.h"
 #include "vp9/common/vp9_onyxc_int.h"
@@ -354,11 +355,11 @@ void vp9_lpf_vertical_16_dual_c(uint8_t *s, int p, const uint8_t *blimit,
 
 #if CONFIG_VP9_HIGHBITDEPTH
 // Should we apply any filter at all: 11111111 yes, 00000000 no ?
-static INLINE int8_t high_filter_mask(uint8_t limit, uint8_t blimit,
-                                      uint16_t p3, uint16_t p2,
-                                      uint16_t p1, uint16_t p0,
-                                      uint16_t q0, uint16_t q1,
-                                      uint16_t q2, uint16_t q3, int bd) {
+static INLINE int8_t highbd_filter_mask(uint8_t limit, uint8_t blimit,
+                                        uint16_t p3, uint16_t p2,
+                                        uint16_t p1, uint16_t p0,
+                                        uint16_t q0, uint16_t q1,
+                                        uint16_t q2, uint16_t q3, int bd) {
   int8_t mask = 0;
   int16_t limit16 = (uint16_t)limit << (bd - 8);
   int16_t blimit16 = (uint16_t)blimit << (bd - 8);
@@ -372,11 +373,11 @@ static INLINE int8_t high_filter_mask(uint8_t limit, uint8_t blimit,
   return ~mask;
 }
 
-static INLINE int8_t high_flat_mask4(uint8_t thresh,
-                                     uint16_t p3, uint16_t p2,
-                                     uint16_t p1, uint16_t p0,
-                                     uint16_t q0, uint16_t q1,
-                                     uint16_t q2, uint16_t q3, int bd) {
+static INLINE int8_t highbd_flat_mask4(uint8_t thresh,
+                                       uint16_t p3, uint16_t p2,
+                                       uint16_t p1, uint16_t p0,
+                                       uint16_t q0, uint16_t q1,
+                                       uint16_t q2, uint16_t q3, int bd) {
   int8_t mask = 0;
   int16_t thresh16 = (uint16_t)thresh << (bd - 8);
   mask |= (abs(p1 - p0) > thresh16) * -1;
@@ -388,13 +389,13 @@ static INLINE int8_t high_flat_mask4(uint8_t thresh,
   return ~mask;
 }
 
-static INLINE int8_t high_flat_mask5(uint8_t thresh,
-                                     uint16_t p4, uint16_t p3,
-                                     uint16_t p2, uint16_t p1,
-                                     uint16_t p0, uint16_t q0,
-                                     uint16_t q1, uint16_t q2,
-                                     uint16_t q3, uint16_t q4, int bd) {
-  int8_t mask = ~high_flat_mask4(thresh, p3, p2, p1, p0, q0, q1, q2, q3, bd);
+static INLINE int8_t highbd_flat_mask5(uint8_t thresh,
+                                       uint16_t p4, uint16_t p3,
+                                       uint16_t p2, uint16_t p1,
+                                       uint16_t p0, uint16_t q0,
+                                       uint16_t q1, uint16_t q2,
+                                       uint16_t q3, uint16_t q4, int bd) {
+  int8_t mask = ~highbd_flat_mask4(thresh, p3, p2, p1, p0, q0, q1, q2, q3, bd);
   int16_t thresh16 = (uint16_t)thresh << (bd - 8);
   mask |= (abs(p4 - p0) > thresh16) * -1;
   mask |= (abs(q4 - q0) > thresh16) * -1;
@@ -403,8 +404,8 @@ static INLINE int8_t high_flat_mask5(uint8_t thresh,
 
 // Is there high edge variance internal edge:
 // 11111111_11111111 yes, 00000000_00000000 no ?
-static INLINE int16_t high_hev_mask(uint8_t thresh, uint16_t p1, uint16_t p0,
-                                    uint16_t q0, uint16_t q1, int bd) {
+static INLINE int16_t highbd_hev_mask(uint8_t thresh, uint16_t p1, uint16_t p0,
+                                      uint16_t q0, uint16_t q1, int bd) {
   int16_t hev = 0;
   int16_t thresh16 = (uint16_t)thresh << (bd - 8);
   hev |= (abs(p1 - p0) > thresh16) * -1;
@@ -412,9 +413,9 @@ static INLINE int16_t high_hev_mask(uint8_t thresh, uint16_t p1, uint16_t p0,
   return hev;
 }
 
-static INLINE void high_filter4(int8_t mask, uint8_t thresh, uint16_t *op1,
-                                uint16_t *op0, uint16_t *oq0, uint16_t *oq1,
-                                int bd) {
+static INLINE void highbd_filter4(int8_t mask, uint8_t thresh, uint16_t *op1,
+                                  uint16_t *op0, uint16_t *oq0, uint16_t *oq1,
+                                  int bd) {
   int16_t filter1, filter2;
   // ^0x80 equivalent to subtracting 0x80 from the values to turn them
   // into -128 to +127 instead of 0 to 255.
@@ -423,7 +424,7 @@ static INLINE void high_filter4(int8_t mask, uint8_t thresh, uint16_t *op1,
   const int16_t ps0 = (int16_t)*op0 - (0x80 << shift);
   const int16_t qs0 = (int16_t)*oq0 - (0x80 << shift);
   const int16_t qs1 = (int16_t)*oq1 - (0x80 << shift);
-  const uint16_t hev = high_hev_mask(thresh, *op1, *op0, *oq0, *oq1, bd);
+  const uint16_t hev = highbd_hev_mask(thresh, *op1, *op0, *oq0, *oq1, bd);
 
   // Add outer taps if we have high edge variance.
   int16_t filter = signed_char_clamp_high(ps1 - qs1, bd) & hev;
@@ -463,9 +464,9 @@ void vp9_highbd_lpf_horizontal_4_c(uint16_t *s, int p /* pitch */,
     const uint16_t q1 = s[1 * p];
     const uint16_t q2 = s[2 * p];
     const uint16_t q3 = s[3 * p];
-    const int8_t mask = high_filter_mask(*limit, *blimit,
-                                         p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    high_filter4(mask, *thresh, s - 2 * p, s - 1 * p, s, s + 1 * p, bd);
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
+                                           p3, p2, p1, p0, q0, q1, q2, q3, bd);
+    highbd_filter4(mask, *thresh, s - 2 * p, s - 1 * p, s, s + 1 * p, bd);
     ++s;
   }
 }
@@ -492,9 +493,9 @@ void vp9_highbd_lpf_vertical_4_c(uint16_t *s, int pitch, const uint8_t *blimit,
   for (i = 0; i < 8 * count; ++i) {
     const uint16_t p3 = s[-4], p2 = s[-3], p1 = s[-2], p0 = s[-1];
     const uint16_t q0 = s[0],  q1 = s[1],  q2 = s[2],  q3 = s[3];
-    const int8_t mask = high_filter_mask(*limit, *blimit,
-                                         p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    high_filter4(mask, *thresh, s - 2, s - 1, s, s + 1, bd);
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
+                                           p3, p2, p1, p0, q0, q1, q2, q3, bd);
+    highbd_filter4(mask, *thresh, s - 2, s - 1, s, s + 1, bd);
     s += pitch;
   }
 }
@@ -512,11 +513,11 @@ void vp9_highbd_lpf_vertical_4_dual_c(uint16_t *s, int pitch,
                               thresh1, 1, bd);
 }
 
-static INLINE void high_filter8(int8_t mask, uint8_t thresh, uint8_t flat,
-                                uint16_t *op3, uint16_t *op2,
-                                uint16_t *op1, uint16_t *op0,
-                                uint16_t *oq0, uint16_t *oq1,
-                                uint16_t *oq2, uint16_t *oq3, int bd) {
+static INLINE void highbd_filter8(int8_t mask, uint8_t thresh, uint8_t flat,
+                                  uint16_t *op3, uint16_t *op2,
+                                  uint16_t *op1, uint16_t *op0,
+                                  uint16_t *oq0, uint16_t *oq1,
+                                  uint16_t *oq2, uint16_t *oq3, int bd) {
   if (flat && mask) {
     const uint16_t p3 = *op3, p2 = *op2, p1 = *op1, p0 = *op0;
     const uint16_t q0 = *oq0, q1 = *oq1, q2 = *oq2, q3 = *oq3;
@@ -529,7 +530,7 @@ static INLINE void high_filter8(int8_t mask, uint8_t thresh, uint8_t flat,
     *oq1 = ROUND_POWER_OF_TWO(p1 + p0 + q0 + 2 * q1 + q2 + q3 + q3, 3);
     *oq2 = ROUND_POWER_OF_TWO(p0 + q0 + q1 + 2 * q2 + q3 + q3 + q3, 3);
   } else {
-    high_filter4(mask, thresh, op1,  op0, oq0, oq1, bd);
+    highbd_filter4(mask, thresh, op1,  op0, oq0, oq1, bd);
   }
 }
 
@@ -544,10 +545,11 @@ void vp9_highbd_lpf_horizontal_8_c(uint16_t *s, int p, const uint8_t *blimit,
     const uint16_t p3 = s[-4 * p], p2 = s[-3 * p], p1 = s[-2 * p], p0 = s[-p];
     const uint16_t q0 = s[0 * p], q1 = s[1 * p], q2 = s[2 * p], q3 = s[3 * p];
 
-    const int8_t mask = high_filter_mask(*limit, *blimit,
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
                                          p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat = high_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    high_filter8(mask, *thresh, flat,
+    const int8_t flat = highbd_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3,
+                                          bd);
+    highbd_filter8(mask, *thresh, flat,
                  s - 4 * p, s - 3 * p, s - 2 * p, s - 1 * p,
                  s, s + 1 * p, s + 2 * p, s + 3 * p, bd);
     ++s;
@@ -574,10 +576,11 @@ void vp9_highbd_lpf_vertical_8_c(uint16_t *s, int pitch, const uint8_t *blimit,
   for (i = 0; i < 8 * count; ++i) {
     const uint16_t p3 = s[-4], p2 = s[-3], p1 = s[-2], p0 = s[-1];
     const uint16_t q0 = s[0], q1 = s[1], q2 = s[2], q3 = s[3];
-    const int8_t mask = high_filter_mask(*limit, *blimit,
-                                         p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat = high_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    high_filter8(mask, *thresh, flat,
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
+                                           p3, p2, p1, p0, q0, q1, q2, q3, bd);
+    const int8_t flat = highbd_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3,
+                                          bd);
+    highbd_filter8(mask, *thresh, flat,
                  s - 4, s - 3, s - 2, s - 1,
                  s, s + 1, s + 2, s + 3,
                  bd);
@@ -598,16 +601,16 @@ void vp9_highbd_lpf_vertical_8_dual_c(uint16_t *s, int pitch,
                               thresh1, 1, bd);
 }
 
-static INLINE void high_filter16(int8_t mask, uint8_t thresh,
-                                 uint8_t flat, uint8_t flat2,
-                                 uint16_t *op7, uint16_t *op6,
-                                 uint16_t *op5, uint16_t *op4,
-                                 uint16_t *op3, uint16_t *op2,
-                                 uint16_t *op1, uint16_t *op0,
-                                 uint16_t *oq0, uint16_t *oq1,
-                                 uint16_t *oq2, uint16_t *oq3,
-                                 uint16_t *oq4, uint16_t *oq5,
-                                 uint16_t *oq6, uint16_t *oq7, int bd) {
+static INLINE void highbd_filter16(int8_t mask, uint8_t thresh,
+                                   uint8_t flat, uint8_t flat2,
+                                   uint16_t *op7, uint16_t *op6,
+                                   uint16_t *op5, uint16_t *op4,
+                                   uint16_t *op3, uint16_t *op2,
+                                   uint16_t *op1, uint16_t *op0,
+                                   uint16_t *oq0, uint16_t *oq1,
+                                   uint16_t *oq2, uint16_t *oq3,
+                                   uint16_t *oq4, uint16_t *oq5,
+                                   uint16_t *oq6, uint16_t *oq7, int bd) {
   if (flat2 && flat && mask) {
     const uint16_t p7 = *op7;
     const uint16_t p6 = *op6;
@@ -656,8 +659,8 @@ static INLINE void high_filter16(int8_t mask, uint8_t thresh,
     *oq6 = ROUND_POWER_OF_TWO(p0 +
                               q0 + q1 + q2 + q3 + q4 + q5 + q6 * 2 + q7 * 7, 4);
   } else {
-    high_filter8(mask, thresh, flat, op3, op2, op1, op0, oq0, oq1, oq2, oq3,
-                 bd);
+    highbd_filter8(mask, thresh, flat, op3, op2, op1, op0, oq0, oq1, oq2, oq3,
+                   bd);
   }
 }
 
@@ -677,28 +680,29 @@ void vp9_highbd_lpf_horizontal_16_c(uint16_t *s, int p, const uint8_t *blimit,
     const uint16_t q1 = s[1 * p];
     const uint16_t q2 = s[2 * p];
     const uint16_t q3 = s[3 * p];
-    const int8_t mask = high_filter_mask(*limit, *blimit,
-                                    p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat = high_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat2 = high_flat_mask5(
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
+                                           p3, p2, p1, p0, q0, q1, q2, q3, bd);
+    const int8_t flat = highbd_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3,
+                                          bd);
+    const int8_t flat2 = highbd_flat_mask5(
         1, s[-8 * p], s[-7 * p], s[-6 * p], s[-5 * p], p0,
         q0, s[4 * p], s[5 * p], s[6 * p], s[7 * p], bd);
 
-    high_filter16(mask, *thresh, flat, flat2,
-                  s - 8 * p, s - 7 * p, s - 6 * p, s - 5 * p,
-                  s - 4 * p, s - 3 * p, s - 2 * p, s - 1 * p,
-                  s, s + 1 * p, s + 2 * p, s + 3 * p,
-                  s + 4 * p, s + 5 * p, s + 6 * p, s + 7 * p,
-                  bd);
+    highbd_filter16(mask, *thresh, flat, flat2,
+                    s - 8 * p, s - 7 * p, s - 6 * p, s - 5 * p,
+                    s - 4 * p, s - 3 * p, s - 2 * p, s - 1 * p,
+                    s, s + 1 * p, s + 2 * p, s + 3 * p,
+                    s + 4 * p, s + 5 * p, s + 6 * p, s + 7 * p,
+                    bd);
     ++s;
   }
 }
 
-static void high_mb_lpf_vertical_edge_w(uint16_t *s, int p,
-                                        const uint8_t *blimit,
-                                        const uint8_t *limit,
-                                        const uint8_t *thresh,
-                                        int count, int bd) {
+static void highbd_mb_lpf_vertical_edge_w(uint16_t *s, int p,
+                                          const uint8_t *blimit,
+                                          const uint8_t *limit,
+                                          const uint8_t *thresh,
+                                          int count, int bd) {
   int i;
 
   for (i = 0; i < count; ++i) {
@@ -710,16 +714,17 @@ static void high_mb_lpf_vertical_edge_w(uint16_t *s, int p,
     const uint16_t q1 = s[1];
     const uint16_t q2 = s[2];
     const uint16_t q3 = s[3];
-    const int8_t mask = high_filter_mask(*limit, *blimit,
-                                         p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat = high_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3, bd);
-    const int8_t flat2 = high_flat_mask5(1, s[-8], s[-7], s[-6], s[-5], p0,
-                                         q0, s[4], s[5], s[6], s[7], bd);
+    const int8_t mask = highbd_filter_mask(*limit, *blimit,
+                                           p3, p2, p1, p0, q0, q1, q2, q3, bd);
+    const int8_t flat = highbd_flat_mask4(1, p3, p2, p1, p0, q0, q1, q2, q3,
+                                          bd);
+    const int8_t flat2 = highbd_flat_mask5(1, s[-8], s[-7], s[-6], s[-5], p0,
+                                           q0, s[4], s[5], s[6], s[7], bd);
 
-    high_filter16(mask, *thresh, flat, flat2,
-                  s - 8, s - 7, s - 6, s - 5, s - 4, s - 3, s - 2, s - 1,
-                  s, s + 1, s + 2, s + 3, s + 4, s + 5, s + 6, s + 7,
-                  bd);
+    highbd_filter16(mask, *thresh, flat, flat2,
+                    s - 8, s - 7, s - 6, s - 5, s - 4, s - 3, s - 2, s - 1,
+                    s, s + 1, s + 2, s + 3, s + 4, s + 5, s + 6, s + 7,
+                    bd);
     s += p;
   }
 }
@@ -727,7 +732,7 @@ static void high_mb_lpf_vertical_edge_w(uint16_t *s, int p,
 void vp9_highbd_lpf_vertical_16_c(uint16_t *s, int p, const uint8_t *blimit,
                                   const uint8_t *limit, const uint8_t *thresh,
                                   int bd) {
-  high_mb_lpf_vertical_edge_w(s, p, blimit, limit, thresh, 8, bd);
+  highbd_mb_lpf_vertical_edge_w(s, p, blimit, limit, thresh, 8, bd);
 }
 
 void vp9_highbd_lpf_vertical_16_dual_c(uint16_t *s, int p,
@@ -735,6 +740,6 @@ void vp9_highbd_lpf_vertical_16_dual_c(uint16_t *s, int p,
                                        const uint8_t *limit,
                                        const uint8_t *thresh,
                                        int bd) {
-  high_mb_lpf_vertical_edge_w(s, p, blimit, limit, thresh, 16, bd);
+  highbd_mb_lpf_vertical_edge_w(s, p, blimit, limit, thresh, 16, bd);
 }
 #endif  // CONFIG_VP9_HIGHBITDEPTH
