@@ -57,6 +57,11 @@
 namespace rtc {
 namespace {
 
+// Turning on IPv6 could make many IPv6 interfaces available for connectivity
+// check and delay the call setup time. kMaxIPv6Networks is the default upper
+// limit of IPv6 networks but could be changed by set_max_ipv6_networks().
+const int kMaxIPv6Networks = 5;
+
 const uint32 kUpdateNetworksMessage = 1;
 const uint32 kSignalNetworksMessage = 2;
 
@@ -115,12 +120,34 @@ std::string AdapterTypeToString(AdapterType type) {
       return "Cellular";
     case ADAPTER_TYPE_VPN:
       return "VPN";
+    case ADAPTER_TYPE_LOOPBACK:
+      return "Loopback";
     default:
-      ASSERT(false);
+      DCHECK(false) << "Invalid type " << type;
       return std::string();
   }
 }
 
+bool IsIgnoredIPv6(const IPAddress& ip) {
+  if (ip.family() != AF_INET6) {
+    return false;
+  }
+
+  // Link-local addresses require scope id to be bound successfully.
+  // However, our IPAddress structure doesn't carry that so the
+  // information is lost and causes binding failure.
+  if (IPIsLinkLocal(ip)) {
+    return true;
+  }
+
+  // Any MAC based IPv6 should be avoided to prevent the MAC tracking.
+  if (IPIsMacBased(ip)) {
+    return true;
+  }
+
+  return false;
+}
+
 }  // namespace
 
 std::string MakeNetworkKey(const std::string& name, const IPAddress& prefix,
@@ -136,22 +163,60 @@ NetworkManager::NetworkManager() {
 NetworkManager::~NetworkManager() {
 }
 
-NetworkManagerBase::NetworkManagerBase() : ipv6_enabled_(true) {
+NetworkManagerBase::NetworkManagerBase()
+    : max_ipv6_networks_(kMaxIPv6Networks), ipv6_enabled_(true) {
 }
 
 NetworkManagerBase::~NetworkManagerBase() {
-  for (NetworkMap::iterator i = networks_map_.begin();
-       i != networks_map_.end(); ++i) {
-    delete i->second;
+  for (const auto& kv : networks_map_) {
+    delete kv.second;
+  }
+}
+
+void NetworkManagerBase::GetAnyAddressNetworks(NetworkList* networks) {
+  if (!ipv4_any_address_network_) {
+    const rtc::IPAddress ipv4_any_address(INADDR_ANY);
+    ipv4_any_address_network_.reset(
+        new rtc::Network("any", "any", ipv4_any_address, 0));
+    ipv4_any_address_network_->AddIP(ipv4_any_address);
+  }
+  networks->push_back(ipv4_any_address_network_.get());
+
+  if (ipv6_enabled()) {
+    if (!ipv6_any_address_network_) {
+      const rtc::IPAddress ipv6_any_address(in6addr_any);
+      ipv6_any_address_network_.reset(
+          new rtc::Network("any", "any", ipv6_any_address, 0));
+      ipv6_any_address_network_->AddIP(ipv6_any_address);
+    }
+    networks->push_back(ipv6_any_address_network_.get());
   }
 }
 
 void NetworkManagerBase::GetNetworks(NetworkList* result) const {
-  *result = networks_;
+  int ipv6_networks = 0;
+  result->clear();
+  for (Network* network : networks_) {
+    // Keep the number of IPv6 networks under |max_ipv6_networks_|.
+    if (network->prefix().family() == AF_INET6) {
+      if (ipv6_networks >= max_ipv6_networks_) {
+        continue;
+      }
+      ++ipv6_networks;
+    }
+    result->push_back(network);
+  }
 }
 
 void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
                                           bool* changed) {
+  NetworkManager::Stats stats;
+  MergeNetworkList(new_networks, changed, &stats);
+}
+
+void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
+                                          bool* changed,
+                                          NetworkManager::Stats* stats) {
   // AddressList in this map will track IP addresses for all Networks
   // with the same key.
   std::map<std::string, AddressList> consolidated_address_list;
@@ -167,49 +232,51 @@ void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
     *changed = true;
 
   // First, build a set of network-keys to the ipaddresses.
-  for (uint32 i = 0; i < list.size(); ++i) {
+  for (Network* network : list) {
     bool might_add_to_merged_list = false;
-    std::string key = MakeNetworkKey(list[i]->name(),
-                                     list[i]->prefix(),
-                                     list[i]->prefix_length());
+    std::string key = MakeNetworkKey(network->name(),
+                                     network->prefix(),
+                                     network->prefix_length());
     if (consolidated_address_list.find(key) ==
         consolidated_address_list.end()) {
       AddressList addrlist;
-      addrlist.net = list[i];
+      addrlist.net = network;
       consolidated_address_list[key] = addrlist;
       might_add_to_merged_list = true;
     }
-    const std::vector<InterfaceAddress>& addresses = list[i]->GetIPs();
+    const std::vector<InterfaceAddress>& addresses = network->GetIPs();
     AddressList& current_list = consolidated_address_list[key];
-    for (std::vector<InterfaceAddress>::const_iterator it = addresses.begin();
-         it != addresses.end();
-         ++it) {
-      current_list.ips.push_back(*it);
+    for (const InterfaceAddress& address : addresses) {
+      current_list.ips.push_back(address);
     }
     if (!might_add_to_merged_list) {
-      delete list[i];
+      delete network;
+    } else {
+      if (current_list.ips[0].family() == AF_INET) {
+        stats->ipv4_network_count++;
+      } else {
+        ASSERT(current_list.ips[0].family() == AF_INET6);
+        stats->ipv6_network_count++;
+      }
     }
   }
 
   // Next, look for existing network objects to re-use.
-  for (std::map<std::string, AddressList>::iterator it =
-         consolidated_address_list.begin();
-       it != consolidated_address_list.end();
-       ++it) {
-    const std::string& key = it->first;
-    Network* net = it->second.net;
-    NetworkMap::iterator existing = networks_map_.find(key);
+  for (const auto& kv : consolidated_address_list) {
+    const std::string& key = kv.first;
+    Network* net = kv.second.net;
+    auto existing = networks_map_.find(key);
     if (existing == networks_map_.end()) {
       // This network is new. Place it in the network map.
       merged_list.push_back(net);
       networks_map_[key] = net;
       // Also, we might have accumulated IPAddresses from the first
       // step, set it here.
-      net->SetIPs(it->second.ips, true);
+      net->SetIPs(kv.second.ips, true);
       *changed = true;
     } else {
       // This network exists in the map already. Reset its IP addresses.
-      *changed = existing->second->SetIPs(it->second.ips, *changed);
+      *changed = existing->second->SetIPs(kv.second.ips, *changed);
       merged_list.push_back(existing->second);
       if (existing->second != net) {
         delete net;
@@ -219,7 +286,7 @@ void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
   networks_ = merged_list;
 
   // If the network lists changes, we resort it.
-  if (changed) {
+  if (*changed) {
     std::sort(networks_.begin(), networks_.end(), SortNetworks);
     // Now network interfaces are sorted, we should set the preference value
     // for each of the interfaces we are planning to use.
@@ -229,9 +296,8 @@ void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
     // requirements, we will just assign a preference value starting with 127,
     // in decreasing order.
     int pref = kHighestNetworkPreference;
-    for (NetworkList::const_iterator iter = networks_.begin();
-         iter != networks_.end(); ++iter) {
-      (*iter)->set_preference(pref);
+    for (Network* network : networks_) {
+      network->set_preference(pref);
       if (pref > 0) {
         --pref;
       } else {
@@ -244,6 +310,7 @@ void NetworkManagerBase::MergeNetworkList(const NetworkList& new_networks,
 
 BasicNetworkManager::BasicNetworkManager()
     : thread_(NULL), sent_first_update_(false), start_count_(0),
+      network_ignore_mask_(kDefaultNetworkIgnoreMask),
       ignore_non_default_routes_(false) {
 }
 
@@ -287,6 +354,11 @@ void BasicNetworkManager::ConvertIfAddrs(struct ifaddrs* interfaces,
         if (ipv6_enabled()) {
           ip = IPAddress(
               reinterpret_cast<sockaddr_in6*>(cursor->ifa_addr)->sin6_addr);
+
+          if (IsIgnoredIPv6(ip)) {
+            continue;
+          }
+
           mask = IPAddress(
               reinterpret_cast<sockaddr_in6*>(cursor->ifa_netmask)->sin6_addr);
           scope_id =
@@ -305,17 +377,21 @@ void BasicNetworkManager::ConvertIfAddrs(struct ifaddrs* interfaces,
     prefix = TruncateIP(ip, prefix_length);
     std::string key = MakeNetworkKey(std::string(cursor->ifa_name),
                                      prefix, prefix_length);
-    NetworkMap::iterator existing_network = current_networks.find(key);
+    auto existing_network = current_networks.find(key);
     if (existing_network == current_networks.end()) {
+      AdapterType adapter_type = ADAPTER_TYPE_UNKNOWN;
+      if (cursor->ifa_flags & IFF_LOOPBACK) {
+        // TODO(phoglund): Need to recognize other types as well.
+        adapter_type = ADAPTER_TYPE_LOOPBACK;
+      }
       scoped_ptr<Network> network(new Network(cursor->ifa_name,
                                               cursor->ifa_name,
                                               prefix,
-                                              prefix_length));
+                                              prefix_length,
+                                              adapter_type));
       network->set_scope_id(scope_id);
       network->AddIP(ip);
-      bool ignored = ((cursor->ifa_flags & IFF_LOOPBACK) ||
-                      IsIgnoredNetwork(*network));
-      network->set_ignored(ignored);
+      network->set_ignored(IsIgnoredNetwork(*network));
       if (include_ignored || !network->ignored()) {
         networks->push_back(network.release());
       }
@@ -438,6 +514,11 @@ bool BasicNetworkManager::CreateNetworks(bool include_ignored,
                   reinterpret_cast<sockaddr_in6*>(address->Address.lpSockaddr);
               scope_id = v6_addr->sin6_scope_id;
               ip = IPAddress(v6_addr->sin6_addr);
+
+              if (IsIgnoredIPv6(ip)) {
+                continue;
+              }
+
               break;
             } else {
               continue;
@@ -451,17 +532,22 @@ bool BasicNetworkManager::CreateNetworks(bool include_ignored,
         IPAddress prefix;
         int prefix_length = GetPrefix(prefixlist, ip, &prefix);
         std::string key = MakeNetworkKey(name, prefix, prefix_length);
-        NetworkMap::iterator existing_network = current_networks.find(key);
+        auto existing_network = current_networks.find(key);
         if (existing_network == current_networks.end()) {
+          AdapterType adapter_type = ADAPTER_TYPE_UNKNOWN;
+          if (adapter_addrs->IfType == IF_TYPE_SOFTWARE_LOOPBACK) {
+            // TODO(phoglund): Need to recognize other types as well.
+            adapter_type = ADAPTER_TYPE_LOOPBACK;
+          }
           scoped_ptr<Network> network(new Network(name,
                                                   description,
                                                   prefix,
-                                                  prefix_length));
+                                                  prefix_length,
+                                                  adapter_type));
           network->set_scope_id(scope_id);
           network->AddIP(ip);
-          bool ignore = ((adapter_addrs->IfType == IF_TYPE_SOFTWARE_LOOPBACK) ||
-                         IsIgnoredNetwork(*network));
-          network->set_ignored(ignore);
+          bool ignored = IsIgnoredNetwork(*network);
+          network->set_ignored(ignored);
           if (include_ignored || !network->ignored()) {
             networks->push_back(network.release());
           }
@@ -508,15 +594,21 @@ bool IsDefaultRoute(const std::string& network_name) {
 
 bool BasicNetworkManager::IsIgnoredNetwork(const Network& network) const {
   // Ignore networks on the explicit ignore list.
-  for (size_t i = 0; i < network_ignore_list_.size(); ++i) {
-    if (network.name() == network_ignore_list_[i]) {
+  for (const std::string& ignored_name : network_ignore_list_) {
+    if (network.name() == ignored_name) {
       return true;
     }
   }
+
+  if (network_ignore_mask_ & network.type()) {
+    return true;
+  }
 #if defined(WEBRTC_POSIX)
-  // Filter out VMware interfaces, typically named vmnet1 and vmnet8
+  // Filter out VMware/VirtualBox interfaces, typically named vmnet1,
+  // vmnet8, or vboxnet0.
   if (strncmp(network.name().c_str(), "vmnet", 5) == 0 ||
-      strncmp(network.name().c_str(), "vnic", 4) == 0) {
+      strncmp(network.name().c_str(), "vnic", 4) == 0 ||
+      strncmp(network.name().c_str(), "vboxnet", 7) == 0) {
     return true;
   }
 #if defined(WEBRTC_LINUX)
@@ -539,6 +631,7 @@ bool BasicNetworkManager::IsIgnoredNetwork(const Network& network) const {
   if (network.prefix().family() == AF_INET) {
     return (network.prefix().v4AddressAsHostOrderInteger() < 0x01000000);
   }
+
   return false;
 }
 
@@ -608,8 +701,7 @@ void BasicNetworkManager::DumpNetworks(bool include_ignored) {
   NetworkList list;
   CreateNetworks(include_ignored, &list);
   LOG(LS_INFO) << "NetworkManager detected " << list.size() << " networks:";
-  for (size_t i = 0; i < list.size(); ++i) {
-    const Network* network = list[i];
+  for (const Network* network : list) {
     if (!network->ignored() || include_ignored) {
       LOG(LS_INFO) << network->ToString() << ": "
                    << network->description()
@@ -618,8 +710,8 @@ void BasicNetworkManager::DumpNetworks(bool include_ignored) {
   }
   // Release the network list created previously.
   // Do this in a seperated for loop for better readability.
-  for (size_t i = 0; i < list.size(); ++i) {
-    delete list[i];
+  for (Network* network : list) {
+    delete network;
   }
 }
 
@@ -639,25 +731,23 @@ Network::Network(const std::string& name, const std::string& desc,
       ignored_(false), type_(type), preference_(0) {
 }
 
+Network::~Network() = default;
+
 // Sets the addresses of this network. Returns true if the address set changed.
 // Change detection is short circuited if the changed argument is true.
 bool Network::SetIPs(const std::vector<InterfaceAddress>& ips, bool changed) {
-  changed = changed || ips.size() != ips_.size();
   // Detect changes with a nested loop; n-squared but we expect on the order
   // of 2-3 addresses per network.
-  for (std::vector<InterfaceAddress>::const_iterator it = ips.begin();
-      !changed && it != ips.end();
-      ++it) {
-    bool found = false;
-    for (std::vector<InterfaceAddress>::iterator inner_it = ips_.begin();
-         !found && inner_it != ips_.end();
-         ++inner_it) {
-      if (*it == *inner_it) {
-        found = true;
+  changed = changed || ips.size() != ips_.size();
+  if (!changed) {
+    for (const InterfaceAddress& ip : ips) {
+      if (std::find(ips_.begin(), ips_.end(), ip) == ips_.end()) {
+        changed = true;
+        break;
       }
     }
-    changed = !found;
   }
+
   ips_ = ips;
   return changed;
 }
@@ -674,21 +764,21 @@ IPAddress Network::GetBestIP() const {
 
   InterfaceAddress selected_ip, ula_ip;
 
-  for (size_t i = 0; i < ips_.size(); i++) {
+  for (const InterfaceAddress& ip : ips_) {
     // Ignore any address which has been deprecated already.
-    if (ips_[i].ipv6_flags() & IPV6_ADDRESS_FLAG_DEPRECATED)
+    if (ip.ipv6_flags() & IPV6_ADDRESS_FLAG_DEPRECATED)
       continue;
 
     // ULA address should only be returned when we have no other
     // global IP.
-    if (IPIsULA(static_cast<const IPAddress&>(ips_[i]))) {
-      ula_ip = ips_[i];
+    if (IPIsULA(static_cast<const IPAddress&>(ip))) {
+      ula_ip = ip;
       continue;
     }
-    selected_ip = ips_[i];
+    selected_ip = ip;
 
     // Search could stop once a temporary non-deprecated one is found.
-    if (ips_[i].ipv6_flags() & IPV6_ADDRESS_FLAG_TEMPORARY)
+    if (ip.ipv6_flags() & IPV6_ADDRESS_FLAG_TEMPORARY)
       break;
   }
 
