@@ -13,6 +13,7 @@
 #include <security.h>
 #include <schannel.h>
 
+#include <algorithm>
 #include <iomanip>
 #include <vector>
 
@@ -89,7 +90,7 @@ struct SChannelAdapter::SSLImpl {
 };
 
 SChannelAdapter::SChannelAdapter(AsyncSocket* socket)
-  : SSLAdapter(socket), state_(SSL_NONE),
+  : SSLAdapter(socket), state_(SSL_NONE), mode_(SSL_MODE_TLS),
     restartable_(false), signal_close_(false), message_pending_(false),
     impl_(new SSLImpl) {
 }
@@ -98,10 +99,20 @@ SChannelAdapter::~SChannelAdapter() {
   Cleanup();
 }
 
+void
+SChannelAdapter::SetMode(SSLMode mode) {
+  // SSL_MODE_DTLS isn't supported.
+  ASSERT(mode == SSL_MODE_TLS);
+  mode_ = mode;
+}
+
 int
 SChannelAdapter::StartSSL(const char* hostname, bool restartable) {
   if (state_ != SSL_NONE)
-    return ERROR_ALREADY_INITIALIZED;
+    return -1;
+
+  if (mode_ != SSL_MODE_TLS)
+    return -1;
 
   ssl_host_name_ = hostname;
   restartable_ = restartable;
@@ -558,7 +569,7 @@ SChannelAdapter::Recv(void* pv, size_t cb) {
     SetError(EWOULDBLOCK);
     return SOCKET_ERROR;
   }
-  size_t read = _min(cb, readable.size());
+  size_t read = std::min(cb, readable.size());
   memcpy(pv, &readable[0], read);
   if (size_t remaining = readable.size() - read) {
     memmove(&readable[0], &readable[read], remaining);
