@@ -38,12 +38,12 @@ void WebRtcSpl_AToK_JSK(
   k16[useOrder-1] = a16[useOrder] << 4;  // Q11<<4 => Q15
 
   for (m=useOrder-1; m>0; m--) {
-    tmp_inv_denum32 = ((int32_t) 1073741823) - WEBRTC_SPL_MUL_16_16(k16[m], k16[m]); // (1 - k^2) in Q30
+    // (1 - k^2) in Q30
+    tmp_inv_denum32 = 1073741823 - k16[m] * k16[m];
     tmp_inv_denum16 = (int16_t)(tmp_inv_denum32 >> 15);  // (1 - k^2) in Q15.
 
     for (k=1; k<=m; k++) {
-      tmp32b = WEBRTC_SPL_LSHIFT_W32((int32_t)a16[k], 16) -
-          WEBRTC_SPL_LSHIFT_W32(WEBRTC_SPL_MUL_16_16(k16[m], a16[m-k+1]), 1);
+      tmp32b = (a16[k] << 16) - ((k16[m] * a16[m - k + 1]) << 1);
 
       tmp32[k] = WebRtcSpl_DivW32W16(tmp32b, tmp_inv_denum16); //Q27/Q15 = Q12
     }
@@ -53,7 +53,7 @@ void WebRtcSpl_AToK_JSK(
     }
 
     tmp32[m] = WEBRTC_SPL_SAT(4092, tmp32[m], -4092);
-    k16[m-1] = (int16_t) WEBRTC_SPL_LSHIFT_W32(tmp32[m], 3); //Q12<<3 => Q15
+    k16[m - 1] = (int16_t)(tmp32[m] << 3);  // Q12<<3 => Q15
   }
 
   return;
@@ -88,7 +88,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
   norm = WebRtcSpl_NormW32(R[0]);
 
   for (i=order;i>=0;i--) {
-    temp1W32 = WEBRTC_SPL_LSHIFT_W32(R[i], norm);
+    temp1W32 = R[i] << norm;
     /* Put R in hi and low format */
     R_hi[i] = (int16_t)(temp1W32 >> 16);
     R_low[i] = (int16_t)((temp1W32 - ((int32_t)R_hi[i] << 16)) >> 1);
@@ -96,8 +96,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
 
   /* K = A[1] = -R[1] / R[0] */
 
-  temp2W32  = WEBRTC_SPL_LSHIFT_W32((int32_t)R_hi[1],16) +
-      WEBRTC_SPL_LSHIFT_W32((int32_t)R_low[1],1);     /* R[1] in Q31      */
+  temp2W32 = (R_hi[1] << 16) + (R_low[1] << 1);  /* R[1] in Q31      */
   temp3W32  = WEBRTC_SPL_ABS_W32(temp2W32);      /* abs R[1]         */
   temp1W32  = WebRtcSpl_DivW32HiLow(temp3W32, R_hi[0], R_low[0]); /* abs(R[1])/R[0] in Q31 */
   /* Put back the sign on R[1] */
@@ -136,7 +135,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
   /* Normalize Alpha and put it in hi and low format */
 
   Alpha_exp = WebRtcSpl_NormW32(temp1W32);
-  temp1W32 = WEBRTC_SPL_LSHIFT_W32(temp1W32, Alpha_exp);
+  temp1W32 <<= Alpha_exp;
   Alpha_hi = (int16_t)(temp1W32 >> 16);
   Alpha_low = (int16_t)((temp1W32 - ((int32_t)Alpha_hi<< 16)) >> 1);
 
@@ -163,9 +162,8 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
               ((R_low[j] * A_hi[i - j]) >> 15)) << 1);
     }
 
-    temp1W32  = WEBRTC_SPL_LSHIFT_W32(temp1W32, 4);
-    temp1W32 += (WEBRTC_SPL_LSHIFT_W32((int32_t)R_hi[i], 16) +
-                 WEBRTC_SPL_LSHIFT_W32((int32_t)R_low[i], 1));
+    temp1W32 <<= 4;
+    temp1W32 += (R_hi[i] << 16) + (R_low[i] << 1);
 
     /* K = -temp1W32 / Alpha */
     temp2W32 = WEBRTC_SPL_ABS_W32(temp1W32);      /* abs(temp1W32) */
@@ -179,7 +177,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
     /* Use the Alpha shifts from earlier to denormalize */
     norm = WebRtcSpl_NormW32(temp3W32);
     if ((Alpha_exp <= norm)||(temp3W32==0)) {
-      temp3W32 = WEBRTC_SPL_LSHIFT_W32(temp3W32, Alpha_exp);
+      temp3W32 <<= Alpha_exp;
     } else {
       if (temp3W32 > 0)
       {
@@ -213,8 +211,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
 
     for(j=1; j<i; j++)
     {
-      temp1W32  = WEBRTC_SPL_LSHIFT_W32((int32_t)A_hi[j],16) +
-          WEBRTC_SPL_LSHIFT_W32((int32_t)A_low[j],1);    /* temp1W32 = A[j] in Q27 */
+      temp1W32 = (A_hi[j] << 16) + (A_low[j] << 1);  // temp1W32 = A[j] in Q27
 
       temp1W32 += (K_hi * A_hi[i - j] + ((K_hi * A_low[i - j]) >> 15) +
           ((K_low * A_hi[i - j]) >> 15)) << 1;  // temp1W32 += K*A[i-j] in Q27.
@@ -248,7 +245,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
     /* Normalize Alpha and store it on hi and low format */
 
     norm = WebRtcSpl_NormW32(temp1W32);
-    temp1W32 = WEBRTC_SPL_LSHIFT_W32(temp1W32, norm);
+    temp1W32 <<= norm;
 
     Alpha_hi = (int16_t)(temp1W32 >> 16);
     Alpha_low = (int16_t)((temp1W32 - ((int32_t)Alpha_hi << 16)) >> 1);
@@ -274,8 +271,7 @@ int16_t WebRtcSpl_LevinsonW32_JSK(
 
   for(i=1; i<=order; i++) {
     /* temp1W32 in Q27 */
-    temp1W32 = WEBRTC_SPL_LSHIFT_W32((int32_t)A_hi[i], 16) +
-        WEBRTC_SPL_LSHIFT_W32((int32_t)A_low[i], 1);
+    temp1W32 = (A_hi[i] << 16) + (A_low[i] << 1);
     /* Round and store upper word */
     A[i] = (int16_t)((temp1W32 + 32768) >> 16);
   }
@@ -371,19 +367,19 @@ void WebRtcIsacfix_GetVars(const int16_t *input, const int16_t *pitchGains_Q12,
   /* Calculate energies of first and second frame halfs */
   nrgQ[0]=0;
   for (k = QLOOKAHEAD/2; k < (FRAMESAMPLES/4 + QLOOKAHEAD) / 2; k++) {
-    nrgQ[0] +=WEBRTC_SPL_MUL_16_16(input[k],input[k]);
+    nrgQ[0] += (uint32_t)(input[k] * input[k]);
   }
   nrgQ[1]=0;
   for ( ; k < (FRAMESAMPLES/2 + QLOOKAHEAD) / 2; k++) {
-    nrgQ[1] +=WEBRTC_SPL_MUL_16_16(input[k],input[k]);
+    nrgQ[1] += (uint32_t)(input[k] * input[k]);
   }
   nrgQ[2]=0;
-  for ( ; k < (WEBRTC_SPL_MUL_16_16(FRAMESAMPLES, 3)/4 + QLOOKAHEAD) / 2; k++) {
-    nrgQ[2] +=WEBRTC_SPL_MUL_16_16(input[k],input[k]);
+  for ( ; k < (FRAMESAMPLES * 3 / 4 + QLOOKAHEAD) / 2; k++) {
+    nrgQ[2] += (uint32_t)(input[k] * input[k]);
   }
   nrgQ[3]=0;
   for ( ; k < (FRAMESAMPLES + QLOOKAHEAD) / 2; k++) {
-    nrgQ[3] +=WEBRTC_SPL_MUL_16_16(input[k],input[k]);
+    nrgQ[3] += (uint32_t)(input[k] * input[k]);
   }
 
   for ( k=0; k<4; k++) {
@@ -482,7 +478,7 @@ int32_t WebRtcIsacfix_CalculateResidualEnergyC(int lpc_order,
        *    a_polynomial[j] * corr_coeffs[i] * a_polynomial[j - i] * 2;
        */
 
-      tmp32 = WEBRTC_SPL_MUL_16_16(a_polynomial[j], a_polynomial[j - i]);
+      tmp32 = a_polynomial[j] * a_polynomial[j - i];
                                    /* tmp32 in Q(q_val_polynomial * 2). */
       if (i != 0) {
         tmp32 <<= 1;
@@ -622,7 +618,7 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
       DataHiQ6[pos1] = (int16_t) WEBRTC_SPL_MUL_16_16_RSFT(
           maskdata->DataBufferHiQ0[pos1], kWindowAutocorr[pos1], 15); // Q0*Q21>>15 = Q6
     }
-    pos2 = (int16_t)(WEBRTC_SPL_MUL_16_16(k, UPDATE)/2);
+    pos2 = (int16_t)(k * UPDATE / 2);
     for (n = 0; n < UPDATE/2; n++, pos1++) {
       maskdata->DataBufferLoQ0[pos1] = inLoQ0[QLOOKAHEAD + pos2];
       maskdata->DataBufferHiQ0[pos1] = inHiQ0[pos2++];
@@ -649,7 +645,7 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
     sh_lo = WebRtcSpl_NormW32(corrloQQ[0]);
     QdomLO += sh_lo;
     for (ii=0; ii<ORDERLO+2; ii++) {
-      corrloQQ[ii] = WEBRTC_SPL_LSHIFT_W32(corrloQQ[ii], sh_lo);
+      corrloQQ[ii] <<= sh_lo;
     }
     /* It is investigated whether it was possible to use 16 bits for the
        32-bit vector corrloQQ, but it didn't work. */
@@ -660,7 +656,7 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
     sh_hi = WebRtcSpl_NormW32(corrhiQQ[0]);
     QdomHI += sh_hi;
     for (ii=0; ii<ORDERHI+1; ii++) {
-      corrhiQQ[ii] = WEBRTC_SPL_LSHIFT_W32(corrhiQQ[ii], sh_hi);
+      corrhiQQ[ii] <<= sh_hi;
     }
 
     /* less noise for lower frequencies, by filtering/scaling autocorrelation sequences */
@@ -830,22 +826,22 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
 
     /* bandwidth expansion */
     for (n = 1; n <= ORDERLO; n++) {
-      a_LOQ11[n] = (int16_t) ((WEBRTC_SPL_MUL_16_16(
-          kPolyVecLo[n-1], a_LOQ11[n]) + ((int32_t) (1 << 14))) >> 15);
+      a_LOQ11[n] = (int16_t)((kPolyVecLo[n - 1] * a_LOQ11[n] + (1 << 14)) >>
+          15);
     }
 
 
     polyHI[0] = a_HIQ12[0];
     for (n = 1; n <= ORDERHI; n++) {
-      a_HIQ12[n] = (int16_t) ((WEBRTC_SPL_MUL_16_16(
-          kPolyVecHi[n-1], a_HIQ12[n]) + ((int32_t) (1 << 14))) >> 15);
+      a_HIQ12[n] = (int16_t)(((int32_t)(kPolyVecHi[n - 1] * a_HIQ12[n]) +
+        (1 << 14)) >> 15);
       polyHI[n] = a_HIQ12[n];
     }
 
     /* Normalize the corrlo2 vector */
     sh = WebRtcSpl_NormW32(corrlo2QQ[0]);
     for (n = 0; n <= ORDERLO; n++) {
-      corrlo2QQ[n] = WEBRTC_SPL_LSHIFT_W32(corrlo2QQ[n], sh);
+      corrlo2QQ[n] <<= sh;
     }
     QdomLO += sh; /* Now, corrlo2QQ is still in Q(QdomLO) */
 
@@ -891,7 +887,7 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
     }
     else
     {
-      *gain_lo_hiQ17 = 100; //(int32_t)WEBRTC_SPL_LSHIFT_W32( (int32_t)1, 17);  // Gains in Q17
+      *gain_lo_hiQ17 = 100;  // Gains in Q17
     }
     gain_lo_hiQ17++;
 
@@ -941,7 +937,7 @@ void WebRtcIsacfix_GetLpcCoef(int16_t *inLoQ0,
     }
     else
     {
-      *gain_lo_hiQ17 = 100; //(int32_t)WEBRTC_SPL_LSHIFT_W32( (int32_t)1, 17);  // Gains in Q17
+      *gain_lo_hiQ17 = 100;  // Gains in Q17
     }
     gain_lo_hiQ17++;
 
