@@ -299,7 +299,7 @@ typedef struct {
   /* Used in adaptive mode only */
   int     minBytes;
 
-} ISAC_SaveEncData_t;
+} IsacSaveEncoderData;
 
 typedef struct {
 
@@ -327,14 +327,14 @@ typedef struct {
   PostFiltBankstr     interpolatorstr_obj;
 #endif
 
-  ISAC_SaveEncData_t *SaveEnc_ptr;
+  IsacSaveEncoderData *SaveEnc_ptr;
   int16_t         payloadLimitBytes30; /* Maximum allowed number of bits for a 30 msec packet */
   int16_t         payloadLimitBytes60; /* Maximum allowed number of bits for a 30 msec packet */
   int16_t         maxPayloadBytes;     /* Maximum allowed number of bits for both 30 and 60 msec packet */
   int16_t         maxRateInBytes;      /* Maximum allowed rate in bytes per 30 msec packet */
   int16_t         enforceFrameSize;    /* If set iSAC will never change packet size */
 
-} ISACFIX_EncInst_t;
+} IsacFixEncoderInstance;
 
 
 typedef struct {
@@ -349,14 +349,14 @@ typedef struct {
   PreFiltBankstr      decimatorstr_obj;
 #endif
 
-} ISACFIX_DecInst_t;
+} IsacFixDecoderInstance;
 
 
 
 typedef struct {
 
-  ISACFIX_EncInst_t ISACenc_obj;
-  ISACFIX_DecInst_t ISACdec_obj;
+  IsacFixEncoderInstance ISACenc_obj;
+  IsacFixDecoderInstance ISACdec_obj;
   BwEstimatorstr     bwestimator_obj;
   int16_t         CodingMode;       /* 0 = adaptive; 1 = instantaneous */
   int16_t   errorcode;
