@@ -14,7 +14,8 @@
 
 namespace webrtc {
 
-enum { kMtu = 1200, kAcceptedBitrateErrorBps = 50000u };
+const size_t kMtu = 1200;
+const unsigned int kAcceptedBitrateErrorBps = 50000;
 
 namespace testing {
 
@@ -54,11 +55,11 @@ int64_t RtpStream::GenerateFrame(int64_t time_now_us, PacketList* packets) {
     return next_rtp_time_;
   }
   assert(packets != NULL);
-  int bits_per_frame = (bitrate_bps_ + fps_ / 2) / fps_;
-  int n_packets = std::max((bits_per_frame + 4 * kMtu) / (8 * kMtu), 1);
-  int packet_size = (bits_per_frame + 4 * n_packets) / (8 * n_packets);
-  assert(n_packets >= 0);
-  for (int i = 0; i < n_packets; ++i) {
+  size_t bits_per_frame = (bitrate_bps_ + fps_ / 2) / fps_;
+  size_t n_packets =
+      std::max<size_t>((bits_per_frame + 4 * kMtu) / (8 * kMtu), 1u);
+  size_t packet_size = (bits_per_frame + 4 * n_packets) / (8 * n_packets);
+  for (size_t i = 0; i < n_packets; ++i) {
     RtpPacket* packet = new RtpPacket;
     packet->send_time = time_now_us + kSendSideOffsetUs;
     packet->size = packet_size;
@@ -217,7 +218,7 @@ uint32_t RemoteBitrateEstimatorTest::AddAbsSendTime(uint32_t t1, uint32_t t2) {
 const unsigned int RemoteBitrateEstimatorTest::kDefaultSsrc = 1;
 
 void RemoteBitrateEstimatorTest::IncomingPacket(uint32_t ssrc,
-                                                uint32_t payload_size,
+                                                size_t payload_size,
                                                 int64_t arrival_time,
                                                 uint32_t rtp_timestamp,
                                                 uint32_t absolute_send_time) {
@@ -511,4 +512,155 @@ void RemoteBitrateEstimatorTest::CapacityDropTestHelper(
   EXPECT_EQ(0u, ssrcs.size());
   EXPECT_EQ(0u, latest_bps);
 }
+
+void RemoteBitrateEstimatorTest::TestTimestampGroupingTestHelper() {
+  const int kFramerate = 50;  // 50 fps to avoid rounding errors.
+  const int kFrameIntervalMs = 1000 / kFramerate;
+  const uint32_t kFrameIntervalAbsSendTime = AbsSendTime(1, kFramerate);
+  uint32_t timestamp = 0;
+  // Initialize absolute_send_time (24 bits) so that it will definitely wrap
+  // during the test.
+  uint32_t absolute_send_time =
+      AddAbsSendTime((1 << 24), -int(50 * kFrameIntervalAbsSendTime));
+  // Initial set of frames to increase the bitrate.
+  for (int i = 0; i <= 100; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    bitrate_estimator_->Process();
+    clock_.AdvanceTimeMilliseconds(kFrameIntervalMs);
+    timestamp += 90 * kFrameIntervalMs;
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        kFrameIntervalAbsSendTime);
+  }
+  EXPECT_TRUE(bitrate_observer_->updated());
+  EXPECT_NEAR(450000u, bitrate_observer_->latest_bitrate(), 20000u);
+
+  // Insert batches of frames which were sent very close in time. Also simulate
+  // capacity over-use to see that we back off correctly.
+  const int kTimestampGroupLength = 15;
+  const uint32_t kTimestampGroupLengthAbsSendTime =
+      AbsSendTime(kTimestampGroupLength, 90000);
+  const uint32_t kSingleRtpTickAbsSendTime = AbsSendTime(1, 90000);
+  for (int i = 0; i < 100; ++i) {
+    for (int j = 0; j < kTimestampGroupLength; ++j) {
+      // Insert |kTimestampGroupLength| frames with just 1 timestamp ticks in
+      // between. Should be treated as part of the same group by the estimator.
+      IncomingPacket(kDefaultSsrc, 100, clock_.TimeInMilliseconds(), timestamp,
+                     absolute_send_time);
+      clock_.AdvanceTimeMilliseconds(kFrameIntervalMs / kTimestampGroupLength);
+      timestamp += 1;
+      absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                          kSingleRtpTickAbsSendTime);
+    }
+    // Increase time until next batch to simulate over-use.
+    clock_.AdvanceTimeMilliseconds(10);
+    timestamp += 90 * kFrameIntervalMs - kTimestampGroupLength;
+    absolute_send_time = AddAbsSendTime(absolute_send_time, AddAbsSendTime(
+        kFrameIntervalAbsSendTime, -int(kTimestampGroupLengthAbsSendTime)));
+    bitrate_estimator_->Process();
+  }
+  EXPECT_TRUE(bitrate_observer_->updated());
+  // Should have reduced the estimate.
+  EXPECT_EQ(378720u, bitrate_observer_->latest_bitrate());
+}
+
+void RemoteBitrateEstimatorTest::TestGetStatsHelper() {
+  const int kFramerate = 100;
+  const int kFrameIntervalMs = 1000 / kFramerate;
+  const int kBurstThresholdMs = 5;
+  const uint32_t kFrameIntervalAbsSendTime = AbsSendTime(1, kFramerate);
+  uint32_t timestamp = 0;
+  // Initialize absolute_send_time (24 bits) so that it will definitely wrap
+  // during the test.
+  uint32_t absolute_send_time =
+      AddAbsSendTime((1 << 24),
+      -(50 * static_cast<int>(kFrameIntervalAbsSendTime)));
+
+  // Inject propagation_time_delta of kFrameIntervalMs.
+  for (size_t i = 0; i < 3; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    timestamp += kFrameIntervalMs;
+    // Insert a kFrameIntervalMs propagation_time_delta.
+    clock_.AdvanceTimeMilliseconds(kFrameIntervalMs * 2);
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        kFrameIntervalAbsSendTime);
+  }
+  ReceiveBandwidthEstimatorStats stats;
+  EXPECT_TRUE(bitrate_estimator_->GetStats(&stats));
+  EXPECT_EQ(1U, stats.recent_propagation_time_delta_ms.size());
+  EXPECT_EQ(kFrameIntervalMs, stats.recent_propagation_time_delta_ms[0]);
+  EXPECT_EQ(1U, stats.recent_arrival_time_ms.size());
+  EXPECT_EQ(kFrameIntervalMs, stats.total_propagation_time_delta_ms);
+
+  // Inject negative propagation_time_deltas. The total propagation_time_delta
+  // should be adjusted to 0.
+  for (size_t i = 0; i < 3; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    timestamp += 10 * kFrameIntervalMs;
+    clock_.AdvanceTimeMilliseconds(kBurstThresholdMs + 1);
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        10 * kFrameIntervalAbsSendTime);
+  }
+  EXPECT_TRUE(bitrate_estimator_->GetStats(&stats));
+  EXPECT_EQ(0, stats.total_propagation_time_delta_ms);
+
+  // Send more than 1000 frames and make sure the stats queues stays within
+  // limits.
+  for (size_t i = 0; i < 1001; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    timestamp += kFrameIntervalMs;
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        kFrameIntervalAbsSendTime);
+  }
+  EXPECT_TRUE(bitrate_estimator_->GetStats(&stats));
+  EXPECT_LE(stats.recent_propagation_time_delta_ms.size(), 1000U);
+  EXPECT_LE(stats.recent_arrival_time_ms.size(), 1000U);
+
+  // Move the clock over the 1000ms limit.
+  clock_.AdvanceTimeMilliseconds(2000);
+  EXPECT_TRUE(bitrate_estimator_->GetStats(&stats));
+  EXPECT_EQ(0U, stats.recent_propagation_time_delta_ms.size());
+}
+
+void RemoteBitrateEstimatorTest::TestWrappingHelper(
+    int silence_time_s) {
+  const int kFramerate = 100;
+  const int kFrameIntervalMs = 1000 / kFramerate;
+  const uint32_t kFrameIntervalAbsSendTime = AbsSendTime(1, kFramerate);
+  uint32_t absolute_send_time = 0;
+  uint32_t timestamp = 0;
+
+  for (size_t i = 0; i < 3000; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    timestamp += kFrameIntervalMs;
+    clock_.AdvanceTimeMilliseconds(kFrameIntervalMs);
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        kFrameIntervalAbsSendTime);
+    bitrate_estimator_->Process();
+  }
+  unsigned int bitrate_before = 0;
+  std::vector<unsigned int> ssrcs;
+  bitrate_estimator_->LatestEstimate(&ssrcs, &bitrate_before);
+
+  clock_.AdvanceTimeMilliseconds(silence_time_s * 1000);
+  absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                      AbsSendTime(silence_time_s, 1));
+  bitrate_estimator_->Process();
+  for (size_t i = 0; i < 100; ++i) {
+    IncomingPacket(kDefaultSsrc, 1000, clock_.TimeInMilliseconds(), timestamp,
+                   absolute_send_time);
+    timestamp += kFrameIntervalMs;
+    clock_.AdvanceTimeMilliseconds(2 * kFrameIntervalMs);
+    absolute_send_time = AddAbsSendTime(absolute_send_time,
+                                        kFrameIntervalAbsSendTime);
+    bitrate_estimator_->Process();
+  }
+  unsigned int bitrate_after = 0;
+  bitrate_estimator_->LatestEstimate(&ssrcs, &bitrate_after);
+  EXPECT_LT(bitrate_after, bitrate_before);
+}
 }  // namespace webrtc
