@@ -17,6 +17,7 @@
 namespace webrtc {
 namespace testing {
 namespace bwe {
+
 class DelayCapHelper {
  public:
   DelayCapHelper() : max_delay_us_(0), delay_stats_() {}
@@ -80,6 +81,7 @@ class RateCounter {
   uint32_t bits_per_second() const {
     return bytes_per_second_ * 8;
   }
+
   uint32_t packets_per_second() const { return packets_per_second_; }
 
  private:
@@ -120,29 +122,17 @@ int Random::Gaussian(int mean, int standard_deviation) {
 }
 
 Packet::Packet()
-    : flow_id_(0),
-      creation_time_us_(-1),
-      send_time_us_(-1),
-      payload_size_(0) {
-  memset(&header_, 0, sizeof(header_));
+    : flow_id_(0), creation_time_us_(-1), send_time_us_(-1), payload_size_(0) {
 }
 
-Packet::Packet(int flow_id, int64_t send_time_us, uint32_t payload_size,
-               const RTPHeader& header)
+Packet::Packet(int flow_id, int64_t send_time_us, size_t payload_size)
     : flow_id_(flow_id),
       creation_time_us_(send_time_us),
       send_time_us_(send_time_us),
-      payload_size_(payload_size),
-      header_(header) {
+      payload_size_(payload_size) {
 }
 
-Packet::Packet(int64_t send_time_us, uint32_t sequence_number)
-    : flow_id_(0),
-      creation_time_us_(send_time_us),
-      send_time_us_(send_time_us),
-      payload_size_(0) {
-  memset(&header_, 0, sizeof(header_));
-  header_.sequenceNumber = sequence_number;
+Packet::~Packet() {
 }
 
 bool Packet::operator<(const Packet& rhs) const {
@@ -154,16 +144,50 @@ void Packet::set_send_time_us(int64_t send_time_us) {
   send_time_us_ = send_time_us;
 }
 
-void Packet::SetAbsSendTimeMs(int64_t abs_send_time_ms) {
+MediaPacket::MediaPacket() {
+  memset(&header_, 0, sizeof(header_));
+}
+
+MediaPacket::MediaPacket(int flow_id,
+                         int64_t send_time_us,
+                         size_t payload_size,
+                         const RTPHeader& header)
+    : Packet(flow_id, send_time_us, payload_size), header_(header) {
+}
+
+MediaPacket::MediaPacket(int64_t send_time_us, uint32_t sequence_number)
+    : Packet(0, send_time_us, 0) {
+  memset(&header_, 0, sizeof(header_));
+  header_.sequenceNumber = sequence_number;
+}
+
+void MediaPacket::SetAbsSendTimeMs(int64_t abs_send_time_ms) {
   header_.extension.hasAbsoluteSendTime = true;
   header_.extension.absoluteSendTime = ((static_cast<int64_t>(abs_send_time_ms *
     (1 << 18)) + 500) / 1000) & 0x00fffffful;
 }
 
+RembFeedback::RembFeedback(int flow_id,
+                           int64_t send_time_us,
+                           uint32_t estimated_bps,
+                           RTCPReportBlock report_block)
+    : FeedbackPacket(flow_id, send_time_us),
+      estimated_bps_(estimated_bps),
+      report_block_(report_block) {
+}
+
+SendSideBweFeedback::SendSideBweFeedback(
+    int flow_id,
+    int64_t send_time_us,
+    const std::vector<PacketInfo>& packet_feedback_vector)
+    : FeedbackPacket(flow_id, send_time_us),
+      packet_feedback_vector_(packet_feedback_vector) {
+}
+
 bool IsTimeSorted(const Packets& packets) {
   PacketsConstIt last_it = packets.begin();
   for (PacketsConstIt it = last_it; it != packets.end(); ++it) {
-    if (it != last_it && *it < *last_it) {
+    if (it != last_it && **it < **last_it) {
       return false;
     }
     last_it = it;
@@ -172,19 +196,21 @@ bool IsTimeSorted(const Packets& packets) {
 }
 
 PacketProcessor::PacketProcessor(PacketProcessorListener* listener,
-                                 bool is_sender)
-    : listener_(listener), flow_ids_(1, 0) {
+                                 int flow_id,
+                                 ProcessorType type)
+    : listener_(listener) {
+  flow_ids_.insert(flow_id);
   if (listener_) {
-    listener_->AddPacketProcessor(this, is_sender);
+    listener_->AddPacketProcessor(this, type);
   }
 }
 
 PacketProcessor::PacketProcessor(PacketProcessorListener* listener,
                                  const FlowIds& flow_ids,
-                                 bool is_sender)
+                                 ProcessorType type)
     : listener_(listener), flow_ids_(flow_ids) {
   if (listener_) {
-    listener_->AddPacketProcessor(this, is_sender);
+    listener_->AddPacketProcessor(this, type);
   }
 }
 
@@ -194,33 +220,28 @@ PacketProcessor::~PacketProcessor() {
   }
 }
 
-RateCounterFilter::RateCounterFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
-      rate_counter_(new RateCounter()),
-      packets_per_second_stats_(),
-      kbps_stats_(),
-      name_("") {}
-
 RateCounterFilter::RateCounterFilter(PacketProcessorListener* listener,
-                                     const std::string& name)
-    : PacketProcessor(listener, false),
+                                     int flow_id,
+                                     const char* name)
+    : PacketProcessor(listener, flow_id, kRegular),
       rate_counter_(new RateCounter()),
       packets_per_second_stats_(),
       kbps_stats_(),
-      name_(name) {}
+      name_(name) {
+}
 
 RateCounterFilter::RateCounterFilter(PacketProcessorListener* listener,
                                      const FlowIds& flow_ids,
-                                     const std::string& name)
-    : PacketProcessor(listener, flow_ids, false),
+                                     const char* name)
+    : PacketProcessor(listener, flow_ids, kRegular),
       rate_counter_(new RateCounter()),
       packets_per_second_stats_(),
       kbps_stats_(),
       name_(name) {
   std::stringstream ss;
   ss << name_ << "_";
-  for (size_t i = 0; i < flow_ids.size(); ++i) {
-    ss << flow_ids[i] << ",";
+  for (int flow_id : flow_ids) {
+    ss << flow_id << ",";
   }
   name_ = ss.str();
 }
@@ -255,15 +276,22 @@ void RateCounterFilter::Plot(int64_t timestamp_ms) {
 
 void RateCounterFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   assert(in_out);
-  for (PacketsConstIt it = in_out->begin(); it != in_out->end(); ++it) {
-    rate_counter_->UpdateRates(it->send_time_us(), it->payload_size());
+  for (const Packet* packet : *in_out) {
+    rate_counter_->UpdateRates(packet->send_time_us(), packet->payload_size());
   }
   packets_per_second_stats_.Push(rate_counter_->packets_per_second());
   kbps_stats_.Push(rate_counter_->bits_per_second() / 1000.0);
 }
 
-LossFilter::LossFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+LossFilter::LossFilter(PacketProcessorListener* listener, int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
+      random_(0x12345678),
+      loss_fraction_(0.0f) {
+}
+
+LossFilter::LossFilter(PacketProcessorListener* listener,
+                       const FlowIds& flow_ids)
+    : PacketProcessor(listener, flow_ids, kRegular),
       random_(0x12345678),
       loss_fraction_(0.0f) {
 }
@@ -280,6 +308,7 @@ void LossFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   assert(in_out);
   for (PacketsIt it = in_out->begin(); it != in_out->end(); ) {
     if (random_.Rand() < loss_fraction_) {
+      delete *it;
       it = in_out->erase(it);
     } else {
       ++it;
@@ -287,8 +316,15 @@ void LossFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   }
 }
 
-DelayFilter::DelayFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+DelayFilter::DelayFilter(PacketProcessorListener* listener, int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
+      delay_us_(0),
+      last_send_time_us_(0) {
+}
+
+DelayFilter::DelayFilter(PacketProcessorListener* listener,
+                         const FlowIds& flow_ids)
+    : PacketProcessor(listener, flow_ids, kRegular),
       delay_us_(0),
       last_send_time_us_(0) {
 }
@@ -302,15 +338,23 @@ void DelayFilter::SetDelay(int64_t delay_ms) {
 
 void DelayFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   assert(in_out);
-  for (PacketsIt it = in_out->begin(); it != in_out->end(); ++it) {
-    int64_t new_send_time_us = it->send_time_us() + delay_us_;
+  for (Packet* packet : *in_out) {
+    int64_t new_send_time_us = packet->send_time_us() + delay_us_;
     last_send_time_us_ = std::max(last_send_time_us_, new_send_time_us);
-    it->set_send_time_us(last_send_time_us_);
+    packet->set_send_time_us(last_send_time_us_);
   }
 }
 
-JitterFilter::JitterFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+JitterFilter::JitterFilter(PacketProcessorListener* listener, int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
+      random_(0x89674523),
+      stddev_jitter_us_(0),
+      last_send_time_us_(0) {
+}
+
+JitterFilter::JitterFilter(PacketProcessorListener* listener,
+                           const FlowIds& flow_ids)
+    : PacketProcessor(listener, flow_ids, kRegular),
       random_(0x89674523),
       stddev_jitter_us_(0),
       last_send_time_us_(0) {
@@ -326,16 +370,23 @@ void JitterFilter::SetJitter(int64_t stddev_jitter_ms) {
 
 void JitterFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   assert(in_out);
-  for (PacketsIt it = in_out->begin(); it != in_out->end(); ++it) {
-    int64_t new_send_time_us = it->send_time_us();
+  for (Packet* packet : *in_out) {
+    int64_t new_send_time_us = packet->send_time_us();
     new_send_time_us += random_.Gaussian(0, stddev_jitter_us_);
     last_send_time_us_ = std::max(last_send_time_us_, new_send_time_us);
-    it->set_send_time_us(last_send_time_us_);
+    packet->set_send_time_us(last_send_time_us_);
   }
 }
 
-ReorderFilter::ReorderFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+ReorderFilter::ReorderFilter(PacketProcessorListener* listener, int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
+      random_(0x27452389),
+      reorder_fraction_(0.0f) {
+}
+
+ReorderFilter::ReorderFilter(PacketProcessorListener* listener,
+                             const FlowIds& flow_ids)
+    : PacketProcessor(listener, flow_ids, kRegular),
       random_(0x27452389),
       reorder_fraction_(0.0f) {
 }
@@ -355,19 +406,19 @@ void ReorderFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
     PacketsIt it = last_it;
     while (++it != in_out->end()) {
       if (random_.Rand() < reorder_fraction_) {
-        int64_t t1 = last_it->send_time_us();
-        int64_t t2 = it->send_time_us();
+        int64_t t1 = (*last_it)->send_time_us();
+        int64_t t2 = (*it)->send_time_us();
         std::swap(*last_it, *it);
-        last_it->set_send_time_us(t1);
-        it->set_send_time_us(t2);
+        (*last_it)->set_send_time_us(t1);
+        (*it)->set_send_time_us(t2);
       }
       last_it = it;
     }
   }
 }
 
-ChokeFilter::ChokeFilter(PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+ChokeFilter::ChokeFilter(PacketProcessorListener* listener, int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
       kbps_(1200),
       last_send_time_us_(0),
       delay_cap_helper_(new DelayCapHelper()) {
@@ -375,7 +426,7 @@ ChokeFilter::ChokeFilter(PacketProcessorListener* listener)
 
 ChokeFilter::ChokeFilter(PacketProcessorListener* listener,
                          const FlowIds& flow_ids)
-    : PacketProcessor(listener, flow_ids, false),
+    : PacketProcessor(listener, flow_ids, kRegular),
       kbps_(1200),
       last_send_time_us_(0),
       delay_cap_helper_(new DelayCapHelper()) {
@@ -392,16 +443,18 @@ void ChokeFilter::SetCapacity(uint32_t kbps) {
 void ChokeFilter::RunFor(int64_t /*time_ms*/, Packets* in_out) {
   assert(in_out);
   for (PacketsIt it = in_out->begin(); it != in_out->end(); ) {
-    int64_t earliest_send_time_us = last_send_time_us_ +
-        (it->payload_size() * 8 * 1000 + kbps_ / 2) / kbps_;
-    int64_t new_send_time_us = std::max(it->send_time_us(),
-                                        earliest_send_time_us);
+    int64_t earliest_send_time_us =
+        last_send_time_us_ +
+        ((*it)->payload_size() * 8 * 1000 + kbps_ / 2) / kbps_;
+    int64_t new_send_time_us =
+        std::max((*it)->send_time_us(), earliest_send_time_us);
     if (delay_cap_helper_->ShouldSendPacket(new_send_time_us,
-                                            it->send_time_us())) {
-      it->set_send_time_us(new_send_time_us);
+                                            (*it)->send_time_us())) {
+      (*it)->set_send_time_us(new_send_time_us);
       last_send_time_us_ = new_send_time_us;
       ++it;
     } else {
+      delete *it;
       it = in_out->erase(it);
     }
   }
@@ -416,8 +469,9 @@ Stats<double> ChokeFilter::GetDelayStats() const {
 }
 
 TraceBasedDeliveryFilter::TraceBasedDeliveryFilter(
-    PacketProcessorListener* listener)
-    : PacketProcessor(listener, false),
+    PacketProcessorListener* listener,
+    int flow_id)
+    : PacketProcessor(listener, flow_id, kRegular),
       current_offset_us_(0),
       delivery_times_us_(),
       next_delivery_it_(),
@@ -426,12 +480,29 @@ TraceBasedDeliveryFilter::TraceBasedDeliveryFilter(
       name_(""),
       delay_cap_helper_(new DelayCapHelper()),
       packets_per_second_stats_(),
-      kbps_stats_() {}
+      kbps_stats_() {
+}
 
 TraceBasedDeliveryFilter::TraceBasedDeliveryFilter(
     PacketProcessorListener* listener,
-    const std::string& name)
-    : PacketProcessor(listener, false),
+    const FlowIds& flow_ids)
+    : PacketProcessor(listener, flow_ids, kRegular),
+      current_offset_us_(0),
+      delivery_times_us_(),
+      next_delivery_it_(),
+      local_time_us_(-1),
+      rate_counter_(new RateCounter),
+      name_(""),
+      delay_cap_helper_(new DelayCapHelper()),
+      packets_per_second_stats_(),
+      kbps_stats_() {
+}
+
+TraceBasedDeliveryFilter::TraceBasedDeliveryFilter(
+    PacketProcessorListener* listener,
+    int flow_id,
+    const char* name)
+    : PacketProcessor(listener, flow_id, kRegular),
       current_offset_us_(0),
       delivery_times_us_(),
       next_delivery_it_(),
@@ -440,7 +511,8 @@ TraceBasedDeliveryFilter::TraceBasedDeliveryFilter(
       name_(name),
       delay_cap_helper_(new DelayCapHelper()),
       packets_per_second_stats_(),
-      kbps_stats_() {}
+      kbps_stats_() {
+}
 
 TraceBasedDeliveryFilter::~TraceBasedDeliveryFilter() {
 }
@@ -484,19 +556,20 @@ void TraceBasedDeliveryFilter::Plot(int64_t timestamp_ms) {
 void TraceBasedDeliveryFilter::RunFor(int64_t time_ms, Packets* in_out) {
   assert(in_out);
   for (PacketsIt it = in_out->begin(); it != in_out->end();) {
-    while (local_time_us_ < it->send_time_us()) {
+    while (local_time_us_ < (*it)->send_time_us()) {
       ProceedToNextSlot();
     }
     // Drop any packets that have been queued for too long.
     while (!delay_cap_helper_->ShouldSendPacket(local_time_us_,
-                                                it->send_time_us())) {
+                                                (*it)->send_time_us())) {
+      delete *it;
       it = in_out->erase(it);
       if (it == in_out->end()) {
         return;
       }
     }
-    if (local_time_us_ >= it->send_time_us()) {
-      it->set_send_time_us(local_time_us_);
+    if (local_time_us_ >= (*it)->send_time_us()) {
+      (*it)->set_send_time_us(local_time_us_);
       ProceedToNextSlot();
     }
     ++it;
@@ -522,9 +595,8 @@ void TraceBasedDeliveryFilter::ProceedToNextSlot() {
     ++next_delivery_it_;
     if (next_delivery_it_ == delivery_times_us_.end()) {
       // When the trace wraps we allow two packets to be sent back-to-back.
-      for (TimeList::iterator it = delivery_times_us_.begin();
-           it != delivery_times_us_.end(); ++it) {
-        *it += local_time_us_ - current_offset_us_;
+      for (int64_t& delivery_time_us : delivery_times_us_) {
+        delivery_time_us += local_time_us_ - current_offset_us_;
       }
       current_offset_us_ += local_time_us_ - current_offset_us_;
       next_delivery_it_ = delivery_times_us_.begin();
@@ -535,51 +607,35 @@ void TraceBasedDeliveryFilter::ProceedToNextSlot() {
   rate_counter_->UpdateRates(local_time_us_, kPayloadSize);
 }
 
-PacketSender::PacketSender(PacketProcessorListener* listener)
-    : PacketProcessor(listener, true) {}
-
-PacketSender::PacketSender(PacketProcessorListener* listener,
-                           const FlowIds& flow_ids)
-    : PacketProcessor(listener, flow_ids, true) {
-
-}
-
-VideoSender::VideoSender(int flow_id,
-                         PacketProcessorListener* listener,
+VideoSource::VideoSource(int flow_id,
                          float fps,
                          uint32_t kbps,
                          uint32_t ssrc,
-                         float first_frame_offset)
-    : PacketSender(listener, FlowIds(1, flow_id)),
-      kMaxPayloadSizeBytes(1200),
+                         int64_t first_frame_offset_ms)
+    : kMaxPayloadSizeBytes(1200),
       kTimestampBase(0xff80ff00ul),
       frame_period_ms_(1000.0 / fps),
-      bytes_per_second_((1000 * kbps) / 8),
-      frame_size_bytes_(bytes_per_second_ / fps),
-      next_frame_ms_(frame_period_ms_ * first_frame_offset),
-      now_ms_(0.0),
+      bits_per_second_(1000 * kbps),
+      frame_size_bytes_(bits_per_second_ / 8 / fps),
+      flow_id_(flow_id),
+      next_frame_ms_(first_frame_offset_ms),
+      now_ms_(0),
       prototype_header_() {
-  assert(first_frame_offset >= 0.0f);
-  assert(first_frame_offset < 1.0f);
   memset(&prototype_header_, 0, sizeof(prototype_header_));
   prototype_header_.ssrc = ssrc;
   prototype_header_.sequenceNumber = 0xf000u;
 }
 
-uint32_t VideoSender::GetCapacityKbps() const {
-  return (bytes_per_second_ * 8) / 1000;
-}
-
-uint32_t VideoSender::NextFrameSize() {
+uint32_t VideoSource::NextFrameSize() {
   return frame_size_bytes_;
 }
 
-uint32_t VideoSender::NextPacketSize(uint32_t frame_size,
+uint32_t VideoSource::NextPacketSize(uint32_t frame_size,
                                      uint32_t remaining_payload) {
   return std::min(kMaxPayloadSizeBytes, remaining_payload);
 }
 
-void VideoSender::RunFor(int64_t time_ms, Packets* in_out) {
+void VideoSource::RunFor(int64_t time_ms, Packets* in_out) {
   assert(in_out);
   now_ms_ += time_ms;
   Packets new_packets;
@@ -599,52 +655,45 @@ void VideoSender::RunFor(int64_t time_ms, Packets* in_out) {
     while (payload_size > 0) {
       ++prototype_header_.sequenceNumber;
       uint32_t size = NextPacketSize(frame_size, payload_size);
-      new_packets.push_back(Packet(flow_ids()[0], send_time_us, size,
-                                   prototype_header_));
-      new_packets.back().SetAbsSendTimeMs(next_frame_ms_);
+      MediaPacket* new_packet =
+          new MediaPacket(flow_id_, send_time_us, size, prototype_header_);
+      new_packets.push_back(new_packet);
+      new_packet->SetAbsSendTimeMs(next_frame_ms_);
       payload_size -= size;
     }
 
     next_frame_ms_ += frame_period_ms_;
   }
-  in_out->merge(new_packets);
+  in_out->merge(new_packets, DereferencingComparator<Packet>);
 }
 
-AdaptiveVideoSender::AdaptiveVideoSender(int flow_id,
-                                         PacketProcessorListener* listener,
+AdaptiveVideoSource::AdaptiveVideoSource(int flow_id,
                                          float fps,
                                          uint32_t kbps,
                                          uint32_t ssrc,
-                                         float first_frame_offset)
-    : VideoSender(flow_id, listener, fps, kbps, ssrc, first_frame_offset) {
+                                         int64_t first_frame_offset_ms)
+    : VideoSource(flow_id, fps, kbps, ssrc, first_frame_offset_ms) {
 }
 
-void AdaptiveVideoSender::GiveFeedback(const PacketSender::Feedback& feedback) {
-  bytes_per_second_ = std::min(feedback.estimated_bps / 8, 2500000u / 8);
-  frame_size_bytes_ = (bytes_per_second_ * frame_period_ms_ + 500) / 1000;
+void AdaptiveVideoSource::SetBitrateBps(int bitrate_bps) {
+  bits_per_second_ = std::min(bitrate_bps, 2500000);
+  frame_size_bytes_ = (bits_per_second_ / 8 * frame_period_ms_ + 500) / 1000;
 }
 
-PeriodicKeyFrameSender::PeriodicKeyFrameSender(
-    int flow_id,
-    PacketProcessorListener* listener,
-    float fps,
-    uint32_t kbps,
-    uint32_t ssrc,
-    float first_frame_offset,
-    int key_frame_interval)
-    : AdaptiveVideoSender(flow_id,
-                          listener,
-                          fps,
-                          kbps,
-                          ssrc,
-                          first_frame_offset),
+PeriodicKeyFrameSource::PeriodicKeyFrameSource(int flow_id,
+                                               float fps,
+                                               uint32_t kbps,
+                                               uint32_t ssrc,
+                                               int64_t first_frame_offset_ms,
+                                               int key_frame_interval)
+    : AdaptiveVideoSource(flow_id, fps, kbps, ssrc, first_frame_offset_ms),
       key_frame_interval_(key_frame_interval),
       frame_counter_(0),
       compensation_bytes_(0),
       compensation_per_frame_(0) {
 }
 
-uint32_t PeriodicKeyFrameSender::NextFrameSize() {
+uint32_t PeriodicKeyFrameSource::NextFrameSize() {
   uint32_t payload_size = frame_size_bytes_;
   if (frame_counter_ == 0) {
     payload_size = kMaxPayloadSizeBytes * 12;
@@ -671,123 +720,13 @@ uint32_t PeriodicKeyFrameSender::NextFrameSize() {
   return payload_size;
 }
 
-uint32_t PeriodicKeyFrameSender::NextPacketSize(uint32_t frame_size,
+uint32_t PeriodicKeyFrameSource::NextPacketSize(uint32_t frame_size,
                                                 uint32_t remaining_payload) {
   uint32_t fragments =
       (frame_size + (kMaxPayloadSizeBytes - 1)) / kMaxPayloadSizeBytes;
   uint32_t avg_size = (frame_size + fragments - 1) / fragments;
   return std::min(avg_size, remaining_payload);
 }
-
-PacedVideoSender::PacedVideoSender(PacketProcessorListener* listener,
-                                   uint32_t kbps,
-                                   AdaptiveVideoSender* source)
-    // It is important that the first_frame_offset and the initial time of
-    // clock_ are both zero, otherwise we can't have absolute time in this
-    // class.
-    : PacketSender(listener, source->flow_ids()),
-      clock_(0),
-      start_of_run_ms_(0),
-      pacer_(&clock_, this, kbps, PacedSender::kDefaultPaceMultiplier* kbps, 0),
-      source_(source) {
-}
-
-void PacedVideoSender::RunFor(int64_t time_ms, Packets* in_out) {
-  start_of_run_ms_ = clock_.TimeInMilliseconds();
-  Packets generated_packets;
-  source_->RunFor(time_ms, &generated_packets);
-  // Run process periodically to allow the packets to be paced out.
-  int64_t end_time_ms = clock_.TimeInMilliseconds() + time_ms;
-  Packets::iterator it = generated_packets.begin();
-  while (clock_.TimeInMilliseconds() <= end_time_ms) {
-    int time_until_process_ms = pacer_.TimeUntilNextProcess();
-    if (time_until_process_ms < 0)
-      time_until_process_ms = 0;
-    int time_until_packet_ms = time_ms;
-    if (it != generated_packets.end())
-      time_until_packet_ms =
-          (it->send_time_us() + 500) / 1000 - clock_.TimeInMilliseconds();
-    assert(time_until_packet_ms >= 0);
-    int time_until_next_event_ms = time_until_packet_ms;
-    if (time_until_process_ms < time_until_packet_ms &&
-        pacer_.QueueSizePackets() > 0)
-      time_until_next_event_ms = time_until_process_ms;
-
-    if (clock_.TimeInMilliseconds() + time_until_next_event_ms > end_time_ms) {
-      clock_.AdvanceTimeMilliseconds(end_time_ms - clock_.TimeInMilliseconds());
-      break;
-    }
-    clock_.AdvanceTimeMilliseconds(time_until_next_event_ms);
-    if (time_until_process_ms < time_until_packet_ms) {
-      // Time to process.
-      pacer_.Process();
-    } else {
-      // Time to send next packet to pacer.
-      pacer_.SendPacket(PacedSender::kNormalPriority,
-                        it->header().ssrc,
-                        it->header().sequenceNumber,
-                        (it->send_time_us() + 500) / 1000,
-                        it->payload_size(),
-                        false);
-      pacer_queue_.push_back(*it);
-      const size_t kMaxPacerQueueSize = 10000;
-      if (pacer_queue_.size() > kMaxPacerQueueSize) {
-        pacer_queue_.pop_front();
-      }
-      ++it;
-    }
-  }
-  QueuePackets(in_out, end_time_ms * 1000);
-}
-
-void PacedVideoSender::QueuePackets(Packets* batch,
-                                    int64_t end_of_batch_time_us) {
-  queue_.merge(*batch);
-  if (queue_.empty()) {
-    return;
-  }
-  Packets::iterator it = queue_.begin();
-  for (; it != queue_.end(); ++it) {
-    if (it->send_time_us() > end_of_batch_time_us) {
-      break;
-    }
-  }
-  Packets to_transfer;
-  to_transfer.splice(to_transfer.begin(), queue_, queue_.begin(), it);
-  batch->merge(to_transfer);
-}
-
-void PacedVideoSender::GiveFeedback(const PacketSender::Feedback& feedback) {
-  source_->GiveFeedback(feedback);
-  pacer_.UpdateBitrate(
-      feedback.estimated_bps / 1000,
-      PacedSender::kDefaultPaceMultiplier * feedback.estimated_bps / 1000,
-      0);
-}
-
-bool PacedVideoSender::TimeToSendPacket(uint32_t ssrc,
-                                        uint16_t sequence_number,
-                                        int64_t capture_time_ms,
-                                        bool retransmission) {
-  for (Packets::iterator it = pacer_queue_.begin(); it != pacer_queue_.end();
-       ++it) {
-    if (it->header().sequenceNumber == sequence_number) {
-      int64_t pace_out_time_ms = clock_.TimeInMilliseconds();
-      // Make sure a packet is never paced out earlier than when it was put into
-      // the pacer.
-      assert(pace_out_time_ms >= (it->send_time_us() + 500) / 1000);
-      it->SetAbsSendTimeMs(pace_out_time_ms);
-      it->set_send_time_us(1000 * pace_out_time_ms);
-      queue_.push_back(*it);
-      return true;
-    }
-  }
-  return false;
-}
-
-int PacedVideoSender::TimeToSendPadding(int bytes) {
-  return 0;
-}
 }  // namespace bwe
 }  // namespace testing
 }  // namespace webrtc
