@@ -14,6 +14,7 @@
 #include <sstream>
 
 #include "testing/gtest/include/gtest/gtest.h"
+#include "webrtc/base/scoped_ptr.h"
 #include "webrtc/common_video/interface/i420_video_frame.h"
 #include "webrtc/common_video/libyuv/include/webrtc_libyuv.h"
 #include "webrtc/modules/utility/interface/process_thread.h"
@@ -21,15 +22,14 @@
 #include "webrtc/modules/video_capture/include/video_capture.h"
 #include "webrtc/modules/video_capture/include/video_capture_factory.h"
 #include "webrtc/system_wrappers/interface/critical_section_wrapper.h"
-#include "webrtc/system_wrappers/interface/scoped_ptr.h"
 #include "webrtc/system_wrappers/interface/scoped_refptr.h"
 #include "webrtc/system_wrappers/interface/sleep.h"
 #include "webrtc/system_wrappers/interface/tick_util.h"
 #include "webrtc/test/testsupport/gtest_disable.h"
 
+using rtc::scoped_ptr;
 using webrtc::CriticalSectionWrapper;
 using webrtc::CriticalSectionScoped;
-using webrtc::scoped_ptr;
 using webrtc::SleepMs;
 using webrtc::TickTime;
 using webrtc::VideoCaptureAlarm;
@@ -92,21 +92,21 @@ static bool CompareFrames(const webrtc::I420VideoFrame& frame1,
 class TestVideoCaptureCallback : public VideoCaptureDataCallback {
  public:
   TestVideoCaptureCallback()
-    : capture_cs_(CriticalSectionWrapper::CreateCriticalSection()),
-      capture_delay_(-1),
-      last_render_time_ms_(0),
-      incoming_frames_(0),
-      timing_warnings_(0),
-      rotate_frame_(webrtc::kCameraRotate0){
-  }
+      : capture_cs_(CriticalSectionWrapper::CreateCriticalSection()),
+        capture_delay_(-1),
+        last_render_time_ms_(0),
+        incoming_frames_(0),
+        timing_warnings_(0),
+        rotate_frame_(webrtc::kVideoRotation_0) {}
 
   ~TestVideoCaptureCallback() {
     if (timing_warnings_ > 0)
       printf("No of timing warnings %d\n", timing_warnings_);
   }
 
-  virtual void OnIncomingCapturedFrame(const int32_t id,
-                                       webrtc::I420VideoFrame& videoFrame) {
+  virtual void OnIncomingCapturedFrame(
+      const int32_t id,
+      const webrtc::I420VideoFrame& videoFrame) {
     CriticalSectionScoped cs(capture_cs_.get());
     int height = videoFrame.height();
     int width = videoFrame.width();
@@ -116,8 +116,8 @@ class TestVideoCaptureCallback : public VideoCaptureDataCallback {
     EXPECT_TRUE(height == capability_.height || height == capability_.width);
     EXPECT_TRUE(width == capability_.width || width == capability_.height);
 #else
-    if (rotate_frame_ == webrtc::kCameraRotate90 ||
-        rotate_frame_ == webrtc::kCameraRotate270) {
+    if (rotate_frame_ == webrtc::kVideoRotation_90 ||
+        rotate_frame_ == webrtc::kVideoRotation_270) {
       EXPECT_EQ(width, capability_.height);
       EXPECT_EQ(height, capability_.width);
     } else {
@@ -143,12 +143,6 @@ class TestVideoCaptureCallback : public VideoCaptureDataCallback {
     last_render_time_ms_ = videoFrame.render_time_ms();
     last_frame_.CopyFrame(videoFrame);
   }
-  virtual void OnIncomingCapturedEncodedFrame(const int32_t id,
-                                              webrtc::VideoFrame& videoFrame,
-                                              webrtc::VideoCodecType codecType)
- {
-     assert(false);
- }
 
   virtual void OnCaptureDelayChanged(const int32_t id,
                                      const int32_t delay) {
@@ -186,7 +180,7 @@ class TestVideoCaptureCallback : public VideoCaptureDataCallback {
     return CompareFrames(last_frame_, frame);
   }
 
-  void SetExpectedCaptureRotation(webrtc::VideoCaptureRotation rotation) {
+  void SetExpectedCaptureRotation(webrtc::VideoRotation rotation) {
     CriticalSectionScoped cs(capture_cs_.get());
     rotate_frame_ = rotation;
   }
@@ -199,7 +193,7 @@ class TestVideoCaptureCallback : public VideoCaptureDataCallback {
   int incoming_frames_;
   int timing_warnings_;
   webrtc::I420VideoFrame last_frame_;
-  webrtc::VideoCaptureRotation rotate_frame_;
+  webrtc::VideoRotation rotate_frame_;
 };
 
 class TestVideoCaptureFeedBack : public VideoCaptureFeedBack {
@@ -436,7 +430,7 @@ class VideoCaptureExternalTest : public testing::Test {
  public:
   void SetUp() {
     capture_module_ = VideoCaptureFactory::Create(0, capture_input_interface_);
-    process_module_ = webrtc::ProcessThread::CreateProcessThread();
+    process_module_ = webrtc::ProcessThread::Create();
     process_module_->Start();
     process_module_->RegisterModule(capture_module_);
 
@@ -464,12 +458,11 @@ class VideoCaptureExternalTest : public testing::Test {
 
   void TearDown() {
     process_module_->Stop();
-    webrtc::ProcessThread::DestroyProcessThread(process_module_);
   }
 
   webrtc::VideoCaptureExternal* capture_input_interface_;
   webrtc::scoped_refptr<VideoCaptureModule> capture_module_;
-  webrtc::ProcessThread* process_module_;
+  rtc::scoped_ptr<webrtc::ProcessThread> process_module_;
   webrtc::I420VideoFrame test_frame_;
   TestVideoCaptureCallback capture_callback_;
   TestVideoCaptureFeedBack capture_feedback_;
@@ -477,85 +470,16 @@ class VideoCaptureExternalTest : public testing::Test {
 
 // Test input of external video frames.
 TEST_F(VideoCaptureExternalTest, TestExternalCapture) {
-  unsigned int length = webrtc::CalcBufferSize(webrtc::kI420,
-                                               test_frame_.width(),
-                                               test_frame_.height());
-  webrtc::scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
+  size_t length = webrtc::CalcBufferSize(webrtc::kI420,
+                                         test_frame_.width(),
+                                         test_frame_.height());
+  scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
   webrtc::ExtractBuffer(test_frame_, length, test_buffer.get());
   EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
       length, capture_callback_.capability(), 0));
   EXPECT_TRUE(capture_callback_.CompareLastFrame(test_frame_));
 }
 
-// Test input of planar I420 frames.
-// NOTE: flaky, sometimes fails on the last CompareLastFrame.
-// http://code.google.com/p/webrtc/issues/detail?id=777
-TEST_F(VideoCaptureExternalTest, DISABLED_TestExternalCaptureI420) {
-  webrtc::I420VideoFrame frame_i420;
-  frame_i420.CopyFrame(test_frame_);
-
-  EXPECT_EQ(0,
-            capture_input_interface_->IncomingI420VideoFrame(&frame_i420, 0));
-  EXPECT_TRUE(capture_callback_.CompareLastFrame(frame_i420));
-
-  // Test with a frame with pitch not equal to width
-  memset(test_frame_.buffer(webrtc::kYPlane), 0xAA,
-         test_frame_.allocated_size(webrtc::kYPlane));
-  memset(test_frame_.buffer(webrtc::kUPlane), 0xAA,
-         test_frame_.allocated_size(webrtc::kUPlane));
-  memset(test_frame_.buffer(webrtc::kVPlane), 0xAA,
-         test_frame_.allocated_size(webrtc::kVPlane));
-  webrtc::I420VideoFrame aligned_test_frame;
-  int y_pitch = kTestWidth + 2;
-  int u_pitch = kTestWidth / 2 + 1;
-  int v_pitch = u_pitch;
-  aligned_test_frame.CreateEmptyFrame(kTestWidth, kTestHeight,
-                                      y_pitch, u_pitch, v_pitch);
-  memset(aligned_test_frame.buffer(webrtc::kYPlane), 0,
-         kTestWidth * kTestHeight);
-  memset(aligned_test_frame.buffer(webrtc::kUPlane), 0,
-         (kTestWidth + 1) / 2  * (kTestHeight + 1) / 2);
-  memset(aligned_test_frame.buffer(webrtc::kVPlane), 0,
-         (kTestWidth + 1) / 2  * (kTestHeight + 1) / 2);
-  // Copy the test_frame_ to aligned_test_frame.
-  int y_width = kTestWidth;
-  int uv_width = kTestWidth / 2;
-  int y_rows = kTestHeight;
-  int uv_rows = kTestHeight / 2;
-  unsigned char* y_plane = test_frame_.buffer(webrtc::kYPlane);
-  unsigned char* u_plane = test_frame_.buffer(webrtc::kUPlane);
-  unsigned char* v_plane = test_frame_.buffer(webrtc::kVPlane);
-  // Copy Y
-  unsigned char* current_pointer = aligned_test_frame.buffer(webrtc::kYPlane);
-  for (int i = 0; i < y_rows; ++i) {
-    memcpy(current_pointer, y_plane, y_width);
-    // Remove the alignment which ViE doesn't support.
-    current_pointer += y_pitch;
-    y_plane += y_width;
-  }
-  // Copy U
-  current_pointer = aligned_test_frame.buffer(webrtc::kUPlane);
-  for (int i = 0; i < uv_rows; ++i) {
-    memcpy(current_pointer, u_plane, uv_width);
-    // Remove the alignment which ViE doesn't support.
-    current_pointer += u_pitch;
-    u_plane += uv_width;
-  }
-  // Copy V
-  current_pointer = aligned_test_frame.buffer(webrtc::kVPlane);
-  for (int i = 0; i < uv_rows; ++i) {
-    memcpy(current_pointer, v_plane, uv_width);
-    // Remove the alignment which ViE doesn't support.
-    current_pointer += v_pitch;
-    v_plane += uv_width;
-  }
-  frame_i420.CopyFrame(aligned_test_frame);
-
-  EXPECT_EQ(0,
-            capture_input_interface_->IncomingI420VideoFrame(&frame_i420, 0));
-  EXPECT_TRUE(capture_callback_.CompareLastFrame(test_frame_));
-}
-
 // Test frame rate and no picture alarm.
 // Flaky on Win32, see webrtc:3270.
 TEST_F(VideoCaptureExternalTest, DISABLED_ON_WIN(FrameRate)) {
@@ -563,10 +487,10 @@ TEST_F(VideoCaptureExternalTest, DISABLED_ON_WIN(FrameRate)) {
   TickTime startTime = TickTime::Now();
 
   while ((TickTime::Now() - startTime).Milliseconds() < testTime * 1000) {
-     unsigned int length = webrtc::CalcBufferSize(webrtc::kI420,
-                                                 test_frame_.width(),
-                                                 test_frame_.height());
-     webrtc::scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
+     size_t length = webrtc::CalcBufferSize(webrtc::kI420,
+                                            test_frame_.width(),
+                                            test_frame_.height());
+     scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
      webrtc::ExtractBuffer(test_frame_, length, test_buffer.get());
      EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
        length, capture_callback_.capability(), 0));
@@ -579,10 +503,10 @@ TEST_F(VideoCaptureExternalTest, DISABLED_ON_WIN(FrameRate)) {
 
   startTime = TickTime::Now();
   while ((TickTime::Now() - startTime).Milliseconds() < testTime * 1000) {
-    unsigned int length = webrtc::CalcBufferSize(webrtc::kI420,
-                                                 test_frame_.width(),
-                                                 test_frame_.height());
-    webrtc::scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
+    size_t length = webrtc::CalcBufferSize(webrtc::kI420,
+                                           test_frame_.width(),
+                                           test_frame_.height());
+    scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
     webrtc::ExtractBuffer(test_frame_, length, test_buffer.get());
     EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
       length, capture_callback_.capability(), 0));
@@ -596,24 +520,24 @@ TEST_F(VideoCaptureExternalTest, DISABLED_ON_WIN(FrameRate)) {
 }
 
 TEST_F(VideoCaptureExternalTest, Rotation) {
-  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kCameraRotate0));
-  unsigned int length = webrtc::CalcBufferSize(webrtc::kI420,
-                                               test_frame_.width(),
-                                               test_frame_.height());
-  webrtc::scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
+  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kVideoRotation_0));
+  size_t length = webrtc::CalcBufferSize(webrtc::kI420,
+                                         test_frame_.width(),
+                                         test_frame_.height());
+  scoped_ptr<uint8_t[]> test_buffer(new uint8_t[length]);
   webrtc::ExtractBuffer(test_frame_, length, test_buffer.get());
   EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
     length, capture_callback_.capability(), 0));
-  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kCameraRotate90));
-  capture_callback_.SetExpectedCaptureRotation(webrtc::kCameraRotate90);
+  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kVideoRotation_90));
+  capture_callback_.SetExpectedCaptureRotation(webrtc::kVideoRotation_90);
   EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
     length, capture_callback_.capability(), 0));
-  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kCameraRotate180));
-  capture_callback_.SetExpectedCaptureRotation(webrtc::kCameraRotate180);
+  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kVideoRotation_180));
+  capture_callback_.SetExpectedCaptureRotation(webrtc::kVideoRotation_180);
   EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
     length, capture_callback_.capability(), 0));
-  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kCameraRotate270));
-  capture_callback_.SetExpectedCaptureRotation(webrtc::kCameraRotate270);
+  EXPECT_EQ(0, capture_module_->SetCaptureRotation(webrtc::kVideoRotation_270));
+  capture_callback_.SetExpectedCaptureRotation(webrtc::kVideoRotation_270);
   EXPECT_EQ(0, capture_input_interface_->IncomingFrame(test_buffer.get(),
     length, capture_callback_.capability(), 0));
 }
