@@ -11,38 +11,46 @@
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
-
-   Bit reading helpers
 */
 
-#include <assert.h>
+/* Bit reading helpers */
+
 #include <stdlib.h>
 
 #include "./bit_reader.h"
+#include "./port.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-int BrotliInitBitReader(BrotliBitReader* const br, BrotliInput input) {
-  size_t i;
-  assert(br != NULL);
+void BrotliInitBitReader(BrotliBitReader* const br, BrotliInput input) {
+  BROTLI_DCHECK(br != NULL);
 
-  br->buf_ptr_ = br->buf_;
   br->input_ = input;
   br->val_ = 0;
-  br->pos_ = 0;
-  br->bit_pos_ = 0;
-  br->bit_end_pos_ = 0;
+  br->bit_pos_ = sizeof(br->val_) << 3;
+  br->avail_in = 0;
   br->eos_ = 0;
-  if (!BrotliReadMoreInput(br)) {
-    return 0;
+  br->next_in = br->buf_;
+}
+
+int BrotliWarmupBitReader(BrotliBitReader* const br) {
+  size_t aligned_read_mask = (sizeof(br->val_) >> 1) - 1;
+  /* Fixing alignment after unaligned BrotliFillWindow would result accumulator
+     overflow. If unalignment is caused by BrotliSafeReadBits, then there is
+     enough space in accumulator to fix aligment. */
+  if (!BROTLI_ALIGNED_READ) {
+    aligned_read_mask = 0;
   }
-  for (i = 0; i < sizeof(br->val_); ++i) {
-    br->val_ |= ((uint64_t)br->buf_[br->pos_]) << (8 * i);
-    ++br->pos_;
+  while (br->bit_pos_ == (sizeof(br->val_) << 3) ||
+      (((size_t)br->next_in) & aligned_read_mask) != 0) {
+    if (!br->avail_in) {
+      return 0;
+    }
+    BrotliPullByte(br);
   }
-  return (br->bit_end_pos_ > 0);
+  return 1;
 }
 
 #if defined(__cplusplus) || defined(c_plusplus)
