@@ -202,6 +202,11 @@ nsresult
 nsFileStreamBase::Read(char* aBuf, uint32_t aCount, uint32_t* aResult)
 {
     nsresult rv = DoPendingOpen();
+    if (rv == NS_ERROR_FILE_NOT_FOUND) {
+      // Don't warn if this is just a deferred file not found.
+      return rv;
+    }
+
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (!mFD) {
@@ -272,7 +277,7 @@ nsFileStreamBase::Write(const char *buf, uint32_t count, uint32_t *result)
     *result = cnt;
     return NS_OK;
 }
-    
+
 nsresult
 nsFileStreamBase::WriteFrom(nsIInputStream *inStr, uint32_t count, uint32_t *_retval)
 {
@@ -408,7 +413,7 @@ nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 
 nsresult
 nsFileInputStream::Open(nsIFile* aFile, int32_t aIOFlags, int32_t aPerm)
-{   
+{
     nsresult rv = NS_OK;
 
     // If the previous file is open, close it
@@ -489,6 +494,11 @@ NS_IMETHODIMP
 nsFileInputStream::Read(char* aBuf, uint32_t aCount, uint32_t* _retval)
 {
     nsresult rv = nsFileStreamBase::Read(aBuf, aCount, _retval);
+    if (rv == NS_ERROR_FILE_NOT_FOUND) {
+      // Don't warn if this is a deffered file not found.
+      return rv;
+    }
+
     NS_ENSURE_SUCCESS(rv, rv);
 
     // Check if we're at the end of file and need to close
@@ -622,7 +632,7 @@ nsFileInputStream::Deserialize(const InputStreamParams& aParams,
 
     mBehaviorFlags = params.behaviorFlags();
 
-    if (XRE_GetProcessType() != GeckoProcessType_Default) {
+    if (!XRE_IsParentProcess()) {
         // A child process shouldn't close when it reads the end because it will
         // not be able to reopen the file later.
         mBehaviorFlags &= ~nsIFileInputStream::CLOSE_ON_EOF;
@@ -688,7 +698,7 @@ nsPartialFileInputStream::Init(nsIFile* aFile, uint64_t aStart,
     nsresult rv = nsFileInputStream::Init(aFile, aIOFlags, aPerm,
                                           aBehaviorFlags);
     NS_ENSURE_SUCCESS(rv, rv);
-    
+
     return nsFileInputStream::Seek(NS_SEEK_SET, mStart);
 }
 
@@ -823,7 +833,7 @@ NS_IMPL_ISUPPORTS_INHERITED(nsFileOutputStream,
                             nsFileStreamBase,
                             nsIOutputStream,
                             nsIFileOutputStream)
- 
+
 nsresult
 nsFileOutputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 {
