@@ -20,9 +20,9 @@
 namespace mozilla {
 namespace net {
 
-const char kContextEvictionPrefix[] = "ce_";
+#define CONTEXT_EVICTION_PREFIX "ce_"
 const uint32_t kContextEvictionPrefixLength =
-  sizeof(kContextEvictionPrefix) - 1;
+  sizeof(CONTEXT_EVICTION_PREFIX) - 1;
 
 bool CacheFileContextEvictor::sDiskAlreadySearched = false;
 
@@ -56,7 +56,7 @@ CacheFileContextEvictor::Init(nsIFile *aCacheDirectory)
     return rv;
   }
 
-  rv = mEntriesDir->AppendNative(NS_LITERAL_CSTRING(kEntriesDir));
+  rv = mEntriesDir->AppendNative(NS_LITERAL_CSTRING(ENTRIES_DIR));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
@@ -81,32 +81,49 @@ CacheFileContextEvictor::ContextsCount()
 }
 
 nsresult
-CacheFileContextEvictor::AddContext(nsILoadContextInfo *aLoadContextInfo)
+CacheFileContextEvictor::AddContext(nsILoadContextInfo *aLoadContextInfo,
+                                    bool aPinned)
 {
-  LOG(("CacheFileContextEvictor::AddContext() [this=%p, loadContextInfo=%p]",
-       this, aLoadContextInfo));
+  LOG(("CacheFileContextEvictor::AddContext() [this=%p, loadContextInfo=%p, pinned=%d]",
+       this, aLoadContextInfo, aPinned));
 
   nsresult rv;
 
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   CacheFileContextEvictorEntry *entry = nullptr;
-  for (uint32_t i = 0; i < mEntries.Length(); ++i) {
-    if (mEntries[i]->mInfo->Equals(aLoadContextInfo)) {
-      entry = mEntries[i];
-      break;
+  if (aLoadContextInfo) {
+    for (uint32_t i = 0; i < mEntries.Length(); ++i) {
+      if (mEntries[i]->mInfo &&
+          mEntries[i]->mInfo->Equals(aLoadContextInfo) &&
+          mEntries[i]->mPinned == aPinned) {
+        entry = mEntries[i];
+        break;
+      }
+    }
+  } else {
+    // Not providing load context info means we want to delete everything,
+    // so let's not bother with any currently running context cleanups
+    // for the same pinning state.
+    for (uint32_t i = mEntries.Length(); i > 0;) {
+      --i;
+      if (mEntries[i]->mInfo && mEntries[i]->mPinned == aPinned) {
+        RemoveEvictInfoFromDisk(mEntries[i]->mInfo, mEntries[i]->mPinned);
+        mEntries.RemoveElementAt(i);
+      }
     }
   }
 
   if (!entry) {
     entry = new CacheFileContextEvictorEntry();
     entry->mInfo = aLoadContextInfo;
+    entry->mPinned = aPinned;
     mEntries.AppendElement(entry);
   }
 
   entry->mTimeStamp = PR_Now() / PR_USEC_PER_MSEC;
 
-  PersistEvictionInfoToDisk(aLoadContextInfo);
+  PersistEvictionInfoToDisk(aLoadContextInfo, aPinned);
 
   if (mIndexIsUpToDate) {
     // Already existing context could be added again, in this case the iterator
@@ -180,58 +197,62 @@ CacheFileContextEvictor::CacheIndexStateChanged()
 
 nsresult
 CacheFileContextEvictor::WasEvicted(const nsACString &aKey, nsIFile *aFile,
-                                    bool *_retval)
+                                    bool *aEvictedAsPinned, bool *aEvictedAsNonPinned)
 {
   LOG(("CacheFileContextEvictor::WasEvicted() [key=%s]",
        PromiseFlatCString(aKey).get()));
 
   nsresult rv;
 
+  *aEvictedAsPinned = false;
+  *aEvictedAsNonPinned = false;
+
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   nsCOMPtr<nsILoadContextInfo> info = CacheFileUtils::ParseKey(aKey);
   MOZ_ASSERT(info);
   if (!info) {
     LOG(("CacheFileContextEvictor::WasEvicted() - Cannot parse key!"));
-    *_retval = false;
     return NS_OK;
   }
 
-  CacheFileContextEvictorEntry *entry = nullptr;
   for (uint32_t i = 0; i < mEntries.Length(); ++i) {
-    if (info->Equals(mEntries[i]->mInfo)) {
-      entry = mEntries[i];
-      break;
+    CacheFileContextEvictorEntry *entry = mEntries[i];
+
+    if (entry->mInfo && !info->Equals(entry->mInfo)) {
+      continue;
+    }
+
+    PRTime lastModifiedTime;
+    rv = aFile->GetLastModifiedTime(&lastModifiedTime);
+    if (NS_FAILED(rv)) {
+      LOG(("CacheFileContextEvictor::WasEvicted() - Cannot get last modified time"
+            ", returning false."));
+      return NS_OK;
+    }
+
+    if (lastModifiedTime > entry->mTimeStamp) {
+      // File has been modified since context eviction.
+      continue;
+    }
+
+    LOG(("CacheFileContextEvictor::WasEvicted() - evicted [pinning=%d, "
+         "mTimeStamp=%lld, lastModifiedTime=%lld]",
+         entry->mPinned, entry->mTimeStamp, lastModifiedTime));
+
+    if (entry->mPinned) {
+      *aEvictedAsPinned = true;
+    } else {
+      *aEvictedAsNonPinned = true;
     }
   }
 
-  if (!entry) {
-    LOG(("CacheFileContextEvictor::WasEvicted() - Didn't find equal context, "
-         "returning false."));
-    *_retval = false;
-    return NS_OK;
-  }
-
-  PRTime lastModifiedTime;
-  rv = aFile->GetLastModifiedTime(&lastModifiedTime);
-  if (NS_FAILED(rv)) {
-    LOG(("CacheFileContextEvictor::WasEvicted() - Cannot get last modified time"
-         ", returning false."));
-    *_retval = false;
-    return NS_OK;
-  }
-
-  *_retval = !(lastModifiedTime > entry->mTimeStamp);
-  LOG(("CacheFileContextEvictor::WasEvicted() - returning %s. [mTimeStamp=%lld,"
-       " lastModifiedTime=%lld]", *_retval ? "true" : "false",
-       mEntries[0]->mTimeStamp, lastModifiedTime));
-
   return NS_OK;
 }
 
 nsresult
 CacheFileContextEvictor::PersistEvictionInfoToDisk(
-  nsILoadContextInfo *aLoadContextInfo)
+  nsILoadContextInfo *aLoadContextInfo, bool aPinned)
 {
   LOG(("CacheFileContextEvictor::PersistEvictionInfoToDisk() [this=%p, "
        "loadContextInfo=%p]", this, aLoadContextInfo));
@@ -241,15 +262,13 @@ CacheFileContextEvictor::PersistEvictionInfoToDisk(
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   nsCOMPtr<nsIFile> file;
-  rv = GetContextFile(aLoadContextInfo, getter_AddRefs(file));
+  rv = GetContextFile(aLoadContextInfo, aPinned, getter_AddRefs(file));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-#ifdef PR_LOGGING
   nsAutoCString path;
   file->GetNativePath(path);
-#endif
 
   PRFileDesc *fd;
   rv = file->OpenNSPRFileDesc(PR_RDWR | PR_CREATE_FILE | PR_TRUNCATE, 0600,
@@ -270,7 +289,7 @@ CacheFileContextEvictor::PersistEvictionInfoToDisk(
 
 nsresult
 CacheFileContextEvictor::RemoveEvictInfoFromDisk(
-  nsILoadContextInfo *aLoadContextInfo)
+  nsILoadContextInfo *aLoadContextInfo, bool aPinned)
 {
   LOG(("CacheFileContextEvictor::RemoveEvictInfoFromDisk() [this=%p, "
        "loadContextInfo=%p]", this, aLoadContextInfo));
@@ -280,15 +299,13 @@ CacheFileContextEvictor::RemoveEvictInfoFromDisk(
   MOZ_ASSERT(CacheFileIOManager::IsOnIOThread());
 
   nsCOMPtr<nsIFile> file;
-  rv = GetContextFile(aLoadContextInfo, getter_AddRefs(file));
+  rv = GetContextFile(aLoadContextInfo, aPinned, getter_AddRefs(file));
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-#ifdef PR_LOGGING
   nsAutoCString path;
   file->GetNativePath(path);
-#endif
 
   rv = file->Remove(false);
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -350,7 +367,7 @@ CacheFileContextEvictor::LoadEvictInfoFromDisk()
       continue;
     }
 
-    if (!StringBeginsWith(leaf, NS_LITERAL_CSTRING(kContextEvictionPrefix))) {
+    if (!StringBeginsWith(leaf, NS_LITERAL_CSTRING(CONTEXT_EVICTION_PREFIX))) {
       continue;
     }
 
@@ -367,16 +384,26 @@ CacheFileContextEvictor::LoadEvictInfoFromDisk()
       continue;
     }
 
-    nsCOMPtr<nsILoadContextInfo> info = CacheFileUtils::ParseKey(decoded);
-
-    if (!info) {
-      LOG(("CacheFileContextEvictor::LoadEvictInfoFromDisk() - Cannot parse "
-           "context key, removing file. [contextKey=%s, file=%s]",
-           decoded.get(), leaf.get()));
-      file->Remove(false);
-      continue;
+    bool pinned = decoded[0] == '\t';
+    if (pinned) {
+      decoded = Substring(decoded, 1);
     }
 
+    nsCOMPtr<nsILoadContextInfo> info;
+    if (!NS_LITERAL_CSTRING("*").Equals(decoded)) {
+      // "*" is indication of 'delete all', info left null will pass
+      // to CacheFileContextEvictor::AddContext and clear all the cache data.
+      info = CacheFileUtils::ParseKey(decoded);
+      if (!info) {
+        LOG(("CacheFileContextEvictor::LoadEvictInfoFromDisk() - Cannot parse "
+             "context key, removing file. [contextKey=%s, file=%s]",
+             decoded.get(), leaf.get()));
+        file->Remove(false);
+        continue;
+      }
+    }
+
+
     PRTime lastModifiedTime;
     rv = file->GetLastModifiedTime(&lastModifiedTime);
     if (NS_FAILED(rv)) {
@@ -385,6 +412,7 @@ CacheFileContextEvictor::LoadEvictInfoFromDisk()
 
     CacheFileContextEvictorEntry *entry = new CacheFileContextEvictorEntry();
     entry->mInfo = info;
+    entry->mPinned = pinned;
     entry->mTimeStamp = lastModifiedTime;
     mEntries.AppendElement(entry);
   }
@@ -394,15 +422,25 @@ CacheFileContextEvictor::LoadEvictInfoFromDisk()
 
 nsresult
 CacheFileContextEvictor::GetContextFile(nsILoadContextInfo *aLoadContextInfo,
+                                        bool aPinned,
                                         nsIFile **_retval)
 {
   nsresult rv;
 
   nsAutoCString leafName;
-  leafName.Assign(NS_LITERAL_CSTRING(kContextEvictionPrefix));
+  leafName.AssignLiteral(CONTEXT_EVICTION_PREFIX);
 
   nsAutoCString keyPrefix;
-  CacheFileUtils::AppendKeyPrefix(aLoadContextInfo, keyPrefix);
+  if (aPinned) {
+    // Mark pinned context files with a tab char at the start.
+    // Tab is chosen because it can never be used as a context key tag.
+    keyPrefix.Append('\t');
+  }
+  if (aLoadContextInfo) {
+    CacheFileUtils::AppendKeyPrefix(aLoadContextInfo, keyPrefix);
+  } else {
+    keyPrefix.Append('*');
+  }
 
   nsAutoCString data64;
   rv = Base64Encode(keyPrefix, data64);
@@ -486,7 +524,7 @@ CacheFileContextEvictor::StartEvicting()
   nsCOMPtr<nsIRunnable> ev;
   ev = NS_NewRunnableMethod(this, &CacheFileContextEvictor::EvictEntries);
 
-  nsRefPtr<CacheIOThread> ioThread = CacheFileIOManager::IOThread();
+  RefPtr<CacheIOThread> ioThread = CacheFileIOManager::IOThread();
 
   nsresult rv = ioThread->Dispatch(ev, CacheIOThread::EVICT);
   if (NS_FAILED(rv)) {
@@ -534,7 +572,7 @@ CacheFileContextEvictor::EvictEntries()
       LOG(("CacheFileContextEvictor::EvictEntries() - No more entries left in "
            "iterator. [iterator=%p, info=%p]", mEntries[0]->mIterator.get(),
            mEntries[0]->mInfo.get()));
-      RemoveEvictInfoFromDisk(mEntries[0]->mInfo);
+      RemoveEvictInfoFromDisk(mEntries[0]->mInfo, mEntries[0]->mPinned);
       mEntries.RemoveElementAt(0);
       continue;
     } else if (NS_FAILED(rv)) {
@@ -550,8 +588,8 @@ CacheFileContextEvictor::EvictEntries()
          "[hash=%08x%08x%08x%08x%08x, iterator=%p, info=%p]", LOGSHA1(&hash),
          mEntries[0]->mIterator.get(), mEntries[0]->mInfo.get()));
 
-    nsRefPtr<CacheFileHandle> handle;
-    CacheFileIOManager::gInstance->mHandles.GetHandle(&hash, false,
+    RefPtr<CacheFileHandle> handle;
+    CacheFileIOManager::gInstance->mHandles.GetHandle(&hash,
                                                       getter_AddRefs(handle));
     if (handle) {
       // We doom any active handle in CacheFileIOManager::EvictByContext(), so
@@ -561,6 +599,20 @@ CacheFileContextEvictor::EvictEntries()
       continue;
     }
 
+    CacheIndex::EntryStatus status;
+    bool pinned;
+    rv = CacheIndex::HasEntry(hash, &status, &pinned);
+    // This must never fail, since eviction (this code) happens only when the index
+    // is up-to-date and thus the informatin is known.
+    MOZ_ASSERT(NS_SUCCEEDED(rv));
+
+    if (pinned != mEntries[0]->mPinned) {
+      LOG(("CacheFileContextEvictor::EvictEntries() - Skipping entry since pinning "
+           "doesn't match [evicting pinned=%d, entry pinned=%d]",
+           mEntries[0]->mPinned, pinned));
+      continue;
+    }
+
     nsAutoCString leafName;
     CacheFileIOManager::HashToStr(&hash, leafName);
 
@@ -595,5 +647,5 @@ CacheFileContextEvictor::EvictEntries()
   return NS_OK;
 }
 
-} // net
-} // mozilla
+} // namespace net
+} // namespace mozilla
