@@ -27,15 +27,11 @@
 #include "nsHttpHandler.h"
 #include "nsHttpRequestHead.h"
 #include "nsIClassOfService.h"
+#include "nsIPipe.h"
 #include "nsISocketTransport.h"
 #include "nsStandardURL.h"
 #include "prnetdb.h"
 
-#ifdef DEBUG
-// defined by the socket transport service while active
-extern PRThread *gSocketThread;
-#endif
-
 namespace mozilla {
 namespace net {
 
@@ -57,12 +53,14 @@ Http2Stream::Http2Stream(nsAHttpTransaction *httpTransaction,
   , mChunkSize(session->SendingChunkSize())
   , mRequestBlockedOnRead(0)
   , mRecvdFin(0)
+  , mReceivedData(0)
   , mRecvdReset(0)
   , mSentReset(0)
   , mCountAsActive(0)
   , mSentFin(0)
   , mSentWaitingFor(0)
   , mSetTCPSocketBuffer(0)
+  , mBypassInputBuffer(0)
   , mTxInlineFrameSize(Http2Session::kDefaultBufferSize)
   , mTxInlineFrameUsed(0)
   , mTxStreamFrameSize(0)
@@ -82,7 +80,7 @@ Http2Stream::Http2Stream(nsAHttpTransaction *httpTransaction,
   mServerReceiveWindow = session->GetServerInitialStreamWindow();
   mClientReceiveWindow = session->PushAllowance();
 
-  mTxInlineFrame = new uint8_t[mTxInlineFrameSize];
+  mTxInlineFrame = MakeUnique<uint8_t[]>(mTxInlineFrameSize);
 
   PR_STATIC_ASSERT(nsISupportsPriority::PRIORITY_LOWEST <= kNormalPriority);
 
@@ -236,6 +234,69 @@ Http2Stream::ReadSegments(nsAHttpSegmentReader *reader,
   return rv;
 }
 
+static bool
+IsDataAvailable(nsIInputStream *stream)
+{
+  if (!stream) {
+    return false;
+  }
+  uint64_t avail;
+  if (NS_FAILED(stream->Available(&avail))) {
+    return false;
+  }
+  return (avail > 0);
+}
+
+uint64_t
+Http2Stream::LocalUnAcked()
+{
+  // reduce unacked by the amount of undelivered data
+  // to help assert flow control
+  uint64_t undelivered = 0;
+  if (mInputBufferIn) {
+    mInputBufferIn->Available(&undelivered);
+  }
+
+  if (undelivered > mLocalUnacked) {
+    return 0;
+  }
+  return mLocalUnacked - undelivered;
+}
+
+nsresult
+Http2Stream::BufferInput(uint32_t count, uint32_t *countWritten)
+{
+  static const uint32_t segmentSize = 32768;
+  char buf[segmentSize];
+
+  count = std::min(segmentSize, count);
+  if (!mInputBufferOut) {
+    NS_NewPipe(getter_AddRefs(mInputBufferIn), getter_AddRefs(mInputBufferOut),
+               segmentSize, UINT32_MAX);
+    if (!mInputBufferOut) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+  }
+  mBypassInputBuffer = 1;
+  nsresult rv = mSegmentWriter->OnWriteSegment(buf, count, countWritten);
+  mBypassInputBuffer = 0;
+  if (NS_SUCCEEDED(rv)) {
+    uint32_t buffered;
+    rv = mInputBufferOut->Write(buf, *countWritten, &buffered);
+    if (NS_SUCCEEDED(rv) && (buffered != *countWritten)) {
+      rv = NS_ERROR_OUT_OF_MEMORY;
+    }
+  }
+  return rv;
+}
+
+bool
+Http2Stream::DeferCleanup(nsresult status)
+{
+  // do not cleanup a stream that has data buffered for the transaction
+  return (NS_SUCCEEDED(status) && IsDataAvailable(mInputBufferIn));
+}
+
 // WriteSegments() is used to read data off the socket. Generally this is
 // just a call through to the associated nsHttpTransaction for this stream
 // for the remaining data bytes indicated by the current DATA frame.
@@ -253,13 +314,33 @@ Http2Stream::WriteSegments(nsAHttpSegmentWriter *writer,
 
   mSegmentWriter = writer;
   nsresult rv = mTransaction->WriteSegments(this, count, countWritten);
-  mSegmentWriter = nullptr;
 
+  if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
+    // consuming transaction won't take data. but we need to read it into a buffer so that it
+    // won't block other streams. but we should not advance the flow control window
+    // so that we'll eventually push back on the sender.
+
+    // with tunnels you need to make sure that this is an underlying connction established
+    // that can be meaningfully giving this signal
+    bool doBuffer = true;
+    if (mIsTunnel) {
+      RefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
+      if (qiTrans) {
+        doBuffer = qiTrans->ConnectedReadyForInput();
+      }
+    }
+    // stash this data
+    if (doBuffer) {
+      rv = BufferInput(count, countWritten);
+      LOG3(("Http2Stream::WriteSegments %p Buffered %X %d\n", this, rv, *countWritten));
+    }
+  }
+  mSegmentWriter = nullptr;
   return rv;
 }
 
 nsresult
-Http2Stream::MakeOriginURL(const nsACString &origin, nsRefPtr<nsStandardURL> &url)
+Http2Stream::MakeOriginURL(const nsACString &origin, RefPtr<nsStandardURL> &url)
 {
   nsAutoCString scheme;
   nsresult rv = net_ExtractURLScheme(origin, nullptr, nullptr, &scheme);
@@ -269,7 +350,7 @@ Http2Stream::MakeOriginURL(const nsACString &origin, nsRefPtr<nsStandardURL> &ur
 
 nsresult
 Http2Stream::MakeOriginURL(const nsACString &scheme, const nsACString &origin,
-                           nsRefPtr<nsStandardURL> &url)
+                           RefPtr<nsStandardURL> &url)
 {
   url = new nsStandardURL();
   nsresult rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY,
@@ -292,7 +373,7 @@ Http2Stream::CreatePushHashKey(const nsCString &scheme,
   fullOrigin.AppendLiteral("://");
   fullOrigin.Append(hostHeader);
 
-  nsRefPtr<nsStandardURL> origin;
+  RefPtr<nsStandardURL> origin;
   nsresult rv = Http2Stream::MakeOriginURL(scheme, fullOrigin, origin);
 
   if (NS_SUCCEEDED(rv)) {
@@ -363,10 +444,10 @@ Http2Stream::ParseHttpRequestHeaders(const char *buf,
   // check the push cache for GET
   if (head->IsGet()) {
     // from :scheme, :authority, :path
-    nsILoadGroupConnectionInfo *loadGroupCI = mTransaction->LoadGroupConnectionInfo();
+    nsISchedulingContext *schedulingContext = mTransaction->SchedulingContext();
     SpdyPushCache *cache = nullptr;
-    if (loadGroupCI) {
-      loadGroupCI->GetSpdyPushCache(&cache);
+    if (schedulingContext) {
+      schedulingContext->GetSpdyPushCache(&cache);
     }
 
     Http2PushedStream *pushedStream = nullptr;
@@ -393,8 +474,8 @@ Http2Stream::ParseHttpRequestHeaders(const char *buf,
     }
 
     LOG3(("Pushed Stream Lookup "
-          "session=%p key=%s loadgroupci=%p cache=%p hit=%p\n",
-          mSession, hashkey.get(), loadGroupCI, cache, pushedStream));
+          "session=%p key=%s schedulingcontext=%p cache=%p hit=%p\n",
+          mSession, hashkey.get(), schedulingContext, cache, pushedStream));
 
     if (pushedStream) {
       LOG3(("Pushed Stream Match located id=0x%X key=%s\n",
@@ -464,9 +545,9 @@ Http2Stream::GenerateOpen()
       return NS_ERROR_UNEXPECTED;
     }
 
-    authorityHeader = ci->GetHost();
+    authorityHeader = ci->GetOrigin();
     authorityHeader.Append(':');
-    authorityHeader.AppendInt(ci->Port());
+    authorityHeader.AppendInt(ci->OriginPort());
   }
 
   mSession->Compressor()->EncodeHeaderBlock(mFlatHttpRequestHeaders,
@@ -530,7 +611,7 @@ Http2Stream::GenerateOpen()
   messageSize += Http2Session::kFrameHeaderBytes + 5; // frame header + priority overhead in HEADERS frame
   messageSize += (numFrames - 1) * Http2Session::kFrameHeaderBytes; // frame header overhead in CONTINUATION frames
 
-  EnsureBuffer(mTxInlineFrame, dataLength + messageSize,
+  EnsureBuffer(mTxInlineFrame, messageSize,
                mTxInlineFrameUsed, mTxInlineFrameSize);
 
   mTxInlineFrameUsed += messageSize;
@@ -623,8 +704,21 @@ Http2Stream::AdjustInitialWindow()
     return;
   }
 
-  MOZ_ASSERT(mClientReceiveWindow <= ASpdySession::kInitialRwin);
-  uint32_t bump = ASpdySession::kInitialRwin - mClientReceiveWindow;
+  // right now mClientReceiveWindow is the lower push limit
+  // bump it up to the pull limit set by the channel or session
+  // don't allow windows less than push
+  uint32_t bump = 0;
+  nsHttpTransaction *trans = mTransaction->QueryHttpTransaction();
+  if (trans && trans->InitialRwin()) {
+    bump = (trans->InitialRwin() > mClientReceiveWindow) ?
+      (trans->InitialRwin() - mClientReceiveWindow) : 0;
+  } else {
+    MOZ_ASSERT(mSession->InitialRwin() >= mClientReceiveWindow);
+    bump = mSession->InitialRwin() - mClientReceiveWindow;
+  }
+
+  LOG3(("AdjustInitialwindow increased flow control window %p 0x%X %u\n",
+        this, stream->mStreamID, bump));
   if (!bump) { // nothing to do
     return;
   }
@@ -641,8 +735,6 @@ Http2Stream::AdjustInitialWindow()
   mClientReceiveWindow += bump;
   bump = PR_htonl(bump);
   memcpy(packet + Http2Session::kFrameHeaderBytes, &bump, 4);
-  LOG3(("AdjustInitialwindow increased flow control window %p 0x%X\n",
-        this, stream->mStreamID));
 }
 
 void
@@ -766,8 +858,7 @@ Http2Stream::TransmitFrame(const char *buf,
       mTxStreamFrameSize < Http2Session::kDefaultBufferSize &&
       mTxInlineFrameUsed + mTxStreamFrameSize < mTxInlineFrameSize) {
     LOG3(("Coalesce Transmit"));
-    memcpy (mTxInlineFrame + mTxInlineFrameUsed,
-            buf, mTxStreamFrameSize);
+    memcpy (&mTxInlineFrame[mTxInlineFrameUsed], buf, mTxStreamFrameSize);
     if (countUsed)
       *countUsed += mTxStreamFrameSize;
     mTxInlineFrameUsed += mTxStreamFrameSize;
@@ -910,7 +1001,7 @@ Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
                                     aHeadersOut, false);
   if (NS_FAILED(rv)) {
     LOG3(("Http2Stream::ConvertResponseHeaders %p decode Error\n", this));
-    return NS_ERROR_ILLEGAL_VALUE;
+    return rv;
   }
 
   nsAutoCString statusString;
@@ -945,10 +1036,7 @@ Http2Stream::ConvertResponseHeaders(Http2Decompressor *decompressor,
   // The decoding went ok. Now we can customize and clean up.
 
   aHeadersIn.Truncate();
-  nsAutoCString negotiatedToken;
-  mSession->GetNegotiatedToken(negotiatedToken);
-  aHeadersOut.Append("X-Firefox-Spdy: ");
-  aHeadersOut.Append(negotiatedToken);
+  aHeadersOut.Append("X-Firefox-Spdy: h2");
   aHeadersOut.Append("\r\n\r\n");
   LOG (("decoded response headers are:\n%s", aHeadersOut.BeginReading()));
   if (mIsTunnel && !mPlainTextTunnel) {
@@ -974,7 +1062,7 @@ Http2Stream::ConvertPushHeaders(Http2Decompressor *decompressor,
                                     aHeadersOut, true);
   if (NS_FAILED(rv)) {
     LOG3(("Http2Stream::ConvertPushHeaders %p Error\n", this));
-    return NS_ERROR_ILLEGAL_VALUE;
+    return rv;
   }
 
   nsCString method;
@@ -1251,15 +1339,15 @@ Http2Stream::OnReadSegment(const char *buf,
       dataLength = static_cast<uint32_t>(mServerReceiveWindow);
 
     LOG3(("Http2Stream this=%p id 0x%X send calculation "
-          "avail=%d chunksize=%d stream window=%d session window=%d "
-          "max frame=%d USING=%d\n", this, mStreamID,
+          "avail=%d chunksize=%d stream window=%" PRId64 " session window=%" PRId64 " "
+          "max frame=%d USING=%u\n", this, mStreamID,
           count, mChunkSize, mServerReceiveWindow, mSession->ServerSessionWindow(),
           Http2Session::kMaxFrameData, dataLength));
 
     mSession->DecrementServerSessionWindow(dataLength);
     mServerReceiveWindow -= dataLength;
 
-    LOG3(("Http2Stream %p id %x request len remaining %u, "
+    LOG3(("Http2Stream %p id 0x%x request len remaining %" PRId64 ", "
           "count avail %u, chunk used %u",
           this, mStreamID, mRequestBodyLenRemaining, count, dataLength));
     if (!dataLength && mRequestBodyLenRemaining) {
@@ -1321,16 +1409,34 @@ Http2Stream::OnWriteSegment(char *buf,
   MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
   MOZ_ASSERT(mSegmentWriter);
 
-  if (!mPushSource)
-    return mSegmentWriter->OnWriteSegment(buf, count, countWritten);
+  if (mPushSource) {
+    nsresult rv;
+    rv = mPushSource->GetBufferedData(buf, count, countWritten);
+    if (NS_FAILED(rv))
+      return rv;
 
-  nsresult rv;
-  rv = mPushSource->GetBufferedData(buf, count, countWritten);
-  if (NS_FAILED(rv))
+    mSession->ConnectPushedStream(this);
+    return NS_OK;
+  }
+
+  // sometimes we have read data from the network and stored it in a pipe
+  // so that other streams can proceed when the gecko caller is not processing
+  // data events fast enough and flow control hasn't caught up yet. This
+  // gets the stored data out of that pipe
+  if (!mBypassInputBuffer && IsDataAvailable(mInputBufferIn)) {
+    nsresult rv = mInputBufferIn->Read(buf, count, countWritten);
+    LOG3(("Http2Stream::OnWriteSegment read from flow control buffer %p %x %d\n",
+          this, mStreamID, *countWritten));
+    if (!IsDataAvailable(mInputBufferIn)) {
+      // drop the pipe if we don't need it anymore
+      mInputBufferIn = nullptr;
+      mInputBufferOut = nullptr;
+    }
     return rv;
+  }
 
-  mSession->ConnectPushedStream(this);
-  return NS_OK;
+  // read from the network
+  return mSegmentWriter->OnWriteSegment(buf, count, countWritten);
 }
 
 /// connect tunnels
@@ -1349,7 +1455,7 @@ Http2Stream::ClearTransactionsBlockedOnTunnel()
 void
 Http2Stream::MapStreamToPlainText()
 {
-  nsRefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
+  RefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
   MOZ_ASSERT(qiTrans);
   mPlainTextTunnel = true;
   qiTrans->ForcePlainText();
@@ -1358,12 +1464,12 @@ Http2Stream::MapStreamToPlainText()
 void
 Http2Stream::MapStreamToHttpConnection()
 {
-  nsRefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
+  RefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
   MOZ_ASSERT(qiTrans);
   qiTrans->MapStreamToHttpConnection(mSocketTransport,
                                      mTransaction->ConnectionInfo());
 }
 
-} // namespace mozilla::net
+} // namespace net
 } // namespace mozilla
 
