@@ -14,8 +14,8 @@
 #include "nsIURL.h"
 #include "nsIUnicharInputStream.h"
 #include "nsISimpleUnicharStreamFactory.h"
+#include "nsIProtocolHandler.h"
 #include "nsNetUtil.h"
-#include "nsNullPrincipal.h"
 #include "prprf.h"
 #include "prmem.h"
 #include "nsTextFormatter.h"
@@ -28,12 +28,17 @@
 #include "nsError.h"
 #include "nsXPCOMCIDInternal.h"
 #include "nsUnicharInputStream.h"
+#include "nsContentUtils.h"
+#include "nsNullPrincipal.h"
+
+#include "mozilla/Logging.h"
+
+using mozilla::LogLevel;
 
 #define kExpatSeparatorChar 0xFFFF
 
 static const char16_t kUTF16[] = { 'U', 'T', 'F', '-', '1', '6', '\0' };
 
-#ifdef PR_LOGGING
 static PRLogModuleInfo *
 GetExpatDriverLog()
 {
@@ -42,7 +47,6 @@ GetExpatDriverLog()
     sLog = PR_NewLogModule("expatdriver");
   return sLog;
 }
-#endif
 
 /***************************** EXPAT CALL BACKS ******************************/
 // The callback handlers that get called from the expat parser.
@@ -753,74 +757,66 @@ nsExpatDriver::OpenInputStreamFromExternalDTD(const char16_t* aFPIStr,
                  baseURI);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // check if it is alright to load this uri
-  bool isChrome = false;
-  uri->SchemeIs("chrome", &isChrome);
-  if (!isChrome) {
-    // since the url is not a chrome url, check to see if we can map the DTD
-    // to a known local DTD, or if a DTD file of the same name exists in the
-    // special DTD directory
+  // make sure the URI is allowed to be loaded in sync
+  bool isUIResource = false;
+  rv = NS_URIChainHasFlags(uri, nsIProtocolHandler::URI_IS_UI_RESOURCE,
+                           &isUIResource);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIURI> localURI;
+  if (!isUIResource) {
+    // Check to see if we can map the DTD to a known local DTD, or if a DTD
+    // file of the same name exists in the special DTD directory
     if (aFPIStr) {
       // see if the Formal Public Identifier (FPI) maps to a catalog entry
       mCatalogData = LookupCatalogData(aFPIStr);
+      GetLocalDTDURI(mCatalogData, uri, getter_AddRefs(localURI));
     }
-
-    nsCOMPtr<nsIURI> localURI;
-    GetLocalDTDURI(mCatalogData, uri, getter_AddRefs(localURI));
     if (!localURI) {
       return NS_ERROR_NOT_IMPLEMENTED;
     }
-
-    localURI.swap(uri);
   }
 
-  nsCOMPtr<nsIDocument> doc;
-  NS_ASSERTION(mSink == nsCOMPtr<nsIExpatSink>(do_QueryInterface(mOriginalSink)),
-               "In nsExpatDriver::OpenInputStreamFromExternalDTD: "
-               "mOriginalSink not the same object as mSink?");
-  if (mOriginalSink)
-    doc = do_QueryInterface(mOriginalSink->GetTarget());
-  int16_t shouldLoad = nsIContentPolicy::ACCEPT;
-  rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_DTD,
-                                uri,
-                                (doc ? doc->NodePrincipal() : nullptr),
-                                doc,
-                                EmptyCString(), //mime guess
-                                nullptr,         //extra
-                                &shouldLoad);
-  if (NS_FAILED(rv)) return rv;
-  if (NS_CP_REJECTED(shouldLoad)) {
-    // Disallowed by content policy
-    return NS_ERROR_CONTENT_BLOCKED;
-  }
-
-  nsAutoCString absURL;
-  uri->GetSpec(absURL);
-
-  CopyUTF8toUTF16(absURL, aAbsURL);
-
   nsCOMPtr<nsIChannel> channel;
-  if (doc) {
+  if (localURI) {
+    localURI.swap(uri);
     rv = NS_NewChannel(getter_AddRefs(channel),
                        uri,
-                       doc,
-                       nsILoadInfo::SEC_NORMAL,
+                       nsContentUtils::GetSystemPrincipal(),
+                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
                        nsIContentPolicy::TYPE_DTD);
   }
   else {
-    nsCOMPtr<nsIPrincipal> nullPrincipal =
-      do_CreateInstance("@mozilla.org/nullprincipal;1", &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ASSERTION(mSink == nsCOMPtr<nsIExpatSink>(do_QueryInterface(mOriginalSink)),
+                 "In nsExpatDriver::OpenInputStreamFromExternalDTD: "
+                 "mOriginalSink not the same object as mSink?");
+    nsCOMPtr<nsIPrincipal> loadingPrincipal;
+    if (mOriginalSink) {
+      nsCOMPtr<nsIDocument> doc;
+      doc = do_QueryInterface(mOriginalSink->GetTarget());
+      if (doc) {
+        loadingPrincipal = doc->NodePrincipal();
+      }
+    }
+    if (!loadingPrincipal) {
+      loadingPrincipal = nsNullPrincipal::Create();
+      NS_ENSURE_TRUE(loadingPrincipal, NS_ERROR_FAILURE);
+    }
     rv = NS_NewChannel(getter_AddRefs(channel),
                        uri,
-                       nullPrincipal,
-                       nsILoadInfo::SEC_NORMAL,
+                       loadingPrincipal,
+                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS |
+                       nsILoadInfo::SEC_ALLOW_CHROME,
                        nsIContentPolicy::TYPE_DTD);
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsAutoCString absURL;
+  uri->GetSpec(absURL);
+  CopyUTF8toUTF16(absURL, aAbsURL);
+
   channel->SetContentType(NS_LITERAL_CSTRING("application/xml"));
-  return channel->Open(aStream);
+  return channel->Open2(aStream);
 }
 
 static nsresult
@@ -1061,7 +1057,7 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner, bool& aFlushTokens)
   nsScannerIterator end;
   aScanner.EndReading(end);
 
-  PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+  MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
          ("Remaining in expat's buffer: %i, remaining in scanner: %i.",
           mExpatBuffered, Distance(start, end)));
 
@@ -1081,9 +1077,8 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner, bool& aFlushTokens)
       buffer = nullptr;
       length = 0;
 
-#if defined(PR_LOGGING) || defined (DEBUG)
       if (blocked) {
-        PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+        MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
                ("Resuming Expat, will parse data remaining in Expat's "
                 "buffer.\nContent of Expat's buffer:\n-----\n%s\n-----\n",
                 NS_ConvertUTF16toUTF8(currentExpatPosition.get(),
@@ -1092,19 +1087,18 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner, bool& aFlushTokens)
       else {
         NS_ASSERTION(mExpatBuffered == Distance(currentExpatPosition, end),
                      "Didn't pass all the data to Expat?");
-        PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+        MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
                ("Last call to Expat, will parse data remaining in Expat's "
                 "buffer.\nContent of Expat's buffer:\n-----\n%s\n-----\n",
                 NS_ConvertUTF16toUTF8(currentExpatPosition.get(),
                                       mExpatBuffered).get()));
       }
-#endif
     }
     else {
       buffer = start.get();
       length = uint32_t(start.size_forward());
 
-      PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+      MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
              ("Calling Expat, will parse data remaining in Expat's buffer and "
               "new data.\nContent of Expat's buffer:\n-----\n%s\n-----\nNew "
               "data:\n-----\n%s\n-----\n",
@@ -1150,7 +1144,7 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner, bool& aFlushTokens)
     mExpatBuffered += length - consumed;
 
     if (BlockedOrInterrupted()) {
-      PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+      MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
              ("Blocked or interrupted parser (probably for loading linked "
               "stylesheets or scripts)."));
 
@@ -1211,7 +1205,7 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner, bool& aFlushTokens)
   aScanner.SetPosition(currentExpatPosition, true);
   aScanner.Mark();
 
-  PR_LOG(GetExpatDriverLog(), PR_LOG_DEBUG,
+  MOZ_LOG(GetExpatDriverLog(), LogLevel::Debug,
          ("Remaining in expat's buffer: %i, remaining in scanner: %i.",
           mExpatBuffered, Distance(currentExpatPosition, end)));
 
