@@ -1,6 +1,24 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
+import os
+import re
 from setuptools import setup, find_packages
 
-version = '0.4'
+
+THIS_DIR = os.path.dirname(os.path.realpath(__name__))
+
+
+def read(*parts):
+    with open(os.path.join(THIS_DIR, *parts)) as f:
+        return f.read()
+
+
+def get_version():
+    return re.findall("__version__ = '([\d\.]+)'",
+                      read('marionette_transport', '__init__.py'), re.M)[0]
+
 
 long_description = \
 """Marionette_ is a GNU project to enable remote automation in Gecko-based
@@ -14,11 +32,9 @@ points; rather it's designed to be used by a Marionette client implementation.
 .. _Marionette: https://developer.mozilla.org/en/Marionette
 .. _`Selenium Webdriver`: http://www.seleniumhq.org/projects/webdriver/"""
 
-# dependencies
-deps = []
 
 setup(name='marionette-transport',
-      version=version,
+      version=get_version(),
       description="Transport layer for Marionette client",
       long_description=long_description,
       classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
@@ -33,6 +49,5 @@ setup(name='marionette-transport',
       zip_safe=False,
       entry_points="""
       """,
-      install_requires=deps,
+      install_requires=[],
       )
-
