@@ -6,28 +6,136 @@ import os
 import re
 import time
 
+from abc import ABCMeta
+
+import version_codes
+
 from adb import ADBDevice, ADBError
-from distutils.version import StrictVersion
 
 
-class ADBAndroidMixin(object):
-    """Mixin to extend ADB with Android-specific functionality"""
+class ADBAndroid(ADBDevice):
+    """ADBAndroid implements :class:`ADBDevice` providing Android-specific
+    functionality.
 
-    # Informational methods
+    ::
 
-    def get_battery_percentage(self, timeout=None):
-        """Returns the battery charge as a percentage.
+       from mozdevice import ADBAndroid
+
+       adbdevice = ADBAndroid()
+       print adbdevice.list_files("/mnt/sdcard")
+       if adbdevice.process_exist("org.mozilla.icecatmobile"):
+           print "IceCatMobile is running"
+    """
+    __metaclass__ = ABCMeta
+
+    def __init__(self,
+                 device=None,
+                 adb='adb',
+                 adb_host=None,
+                 adb_port=None,
+                 test_root='',
+                 logger_name='adb',
+                 timeout=300,
+                 verbose=False,
+                 device_ready_retry_wait=20,
+                 device_ready_retry_attempts=3):
+        """Initializes the ADBAndroid object.
+
+        :param device: When a string is passed, it is interpreted as the
+            device serial number. This form is not compatible with
+            devices containing a ":" in the serial; in this case
+            ValueError will be raised.
+            When a dictionary is passed it must have one or both of
+            the keys "device_serial" and "usb". This is compatible
+            with the dictionaries in the list returned by
+            ADBHost.devices(). If the value of device_serial is a
+            valid serial not containing a ":" it will be used to
+            identify the device, otherwise the value of the usb key,
+            prefixed with "usb:" is used.
+            If None is passed and there is exactly one device attached
+            to the host, that device is used. If there is more than one
+            device attached, ValueError is raised. If no device is
+            attached the constructor will block until a device is
+            attached or the timeout is reached.
+        :type device: dict, str or None
+        :param adb_host: host of the adb server to connect to.
+        :type adb_host: str or None
+        :param adb_port: port of the adb server to connect to.
+        :type adb_port: integer or None
+        :param str logger_name: logging logger name. Defaults to 'adb'.
+        :param integer device_ready_retry_wait: number of seconds to wait
+            between attempts to check if the device is ready after a
+            reboot.
+        :param integer device_ready_retry_attempts: number of attempts when
+            checking if a device is ready.
+
+        :raises: * ADBError
+                 * ADBTimeoutError
+                 * ValueError
+        """
+        ADBDevice.__init__(self, device=device, adb=adb,
+                           adb_host=adb_host, adb_port=adb_port,
+                           test_root=test_root,
+                           logger_name=logger_name, timeout=timeout,
+                           verbose=verbose,
+                           device_ready_retry_wait=device_ready_retry_wait,
+                           device_ready_retry_attempts=device_ready_retry_attempts)
+        # https://source.android.com/devices/tech/security/selinux/index.html
+        # setenforce
+        # usage:  setenforce [ Enforcing | Permissive | 1 | 0 ]
+        # getenforce returns either Enforcing or Permissive
+
+        try:
+            self.selinux = True
+            if self.shell_output('getenforce', timeout=timeout) != 'Permissive':
+                self._logger.info('Setting SELinux Permissive Mode')
+                self.shell_output("setenforce Permissive", timeout=timeout, root=True)
+        except ADBError:
+            self.selinux = False
+
+    def reboot(self, timeout=None):
+        """Reboots the device.
 
         :param timeout: optional integer specifying the maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
-            set in the ADBDevice constructor is used.
-        :returns: battery charge as a percentage.
+            set in the ADB constructor is used.
         :raises: * ADBTimeoutError
                  * ADBError
 
+        reboot() reboots the device, issues an adb wait-for-device in order to
+        wait for the device to complete rebooting, then calls is_device_ready()
+        to determine if the device has completed booting.
+
+        If the device supports running adbd as root, adbd will be
+        restarted running as root. Then, if the device supports
+        SELinux, setenforce Permissive will be called to change
+        SELinux to permissive. This must be done after adbd is
+        restarted in order for the SELinux Permissive setting to
+        persist.
+
+        """
+        ready = ADBDevice.reboot(self, timeout=timeout)
+        self._check_adb_root(timeout=timeout)
+        return ready
+
+    # Informational methods
+
+    def get_battery_percentage(self, timeout=None):
+        """Returns the battery charge as a percentage.
+
+        :param timeout: The maximum time in
+            seconds for any spawned adb process to complete before
+            throwing an ADBTimeoutError.
+            This timeout is per adb call. The total time spent
+            may exceed this value. If it is not specified, the value
+            set in the ADBDevice constructor is used.
+        :type timeout: integer or None
+        :returns: battery charge as a percentage.
+        :raises: * ADBTimeoutError
+                 * ADBError
         """
         level = None
         scale = None
@@ -57,17 +165,23 @@ class ADBAndroidMixin(object):
         This method uses the android only package manager to check for
         readiness.
 
-        :param timeout: optional integer specifying the maximum time
+        :param timeout: The maximum time
             in seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
-        self.command_output(["wait-for-device"], timeout=timeout)
+        # command_output automatically inserts a 'wait-for-device'
+        # argument to adb. Issuing an empty command is the same as adb
+        # -s <device> wait-for-device. We don't send an explicit
+        # 'wait-for-device' since that would add duplicate
+        # 'wait-for-device' arguments which is an error in newer
+        # versions of adb.
+        self.command_output([], timeout=timeout)
         pm_error_string = "Error: Could not access the Package Manager"
         pm_list_commands = ["packages", "permission-groups", "permissions",
                             "instrumentation", "features", "libraries"]
@@ -81,10 +195,15 @@ class ADBAndroidMixin(object):
                     failure = "Device state: %s" % state
                     success = False
                 else:
-                    if self.is_dir(ready_path, timeout=timeout):
-                        self.rmdir(ready_path, timeout=timeout)
-                    self.mkdir(ready_path, timeout=timeout)
-                    self.rmdir(ready_path, timeout=timeout)
+                    if (self.selinux and
+                        self.shell_output('getenforce',
+                                          timeout=timeout) != 'Permissive'):
+                        self._logger.info('Setting SELinux Permissive Mode')
+                        self.shell_output("setenforce Permissive", timeout=timeout, root=True)
+                    if self.is_dir(ready_path, timeout=timeout, root=True):
+                        self.rmdir(ready_path, timeout=timeout, root=True)
+                    self.mkdir(ready_path, timeout=timeout, root=True)
+                    self.rmdir(ready_path, timeout=timeout, root=True)
                     # Invoke the pm list commands to see if it is up and
                     # running.
                     for pm_list_cmd in pm_list_commands:
@@ -109,18 +228,20 @@ class ADBAndroidMixin(object):
     def power_on(self, timeout=None):
         """Sets the device's power stayon value.
 
-        :param timeout: optional integer specifying the maximum time in
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         try:
-            self.shell_output('svc power stayon true', timeout=timeout)
+            self.shell_output('svc power stayon true',
+                              timeout=timeout,
+                              root=True)
         except ADBError, e:
             # Executing this via adb shell errors, but not interactively.
             # Any other exitcode is a real error.
@@ -128,46 +249,21 @@ class ADBAndroidMixin(object):
                 raise
             self._logger.warning('Unable to set power stayon true: %s' % e)
 
-    def reboot(self, timeout=None):
-        """Reboots the device.
-
-        This method uses the Android only package manager to determine
-        if the device is ready after the reboot.
-
-        :param timeout: optional integer specifying the maximum time in
-            seconds for any spawned adb process to complete before
-            throwing an ADBTimeoutError.
-            This timeout is per adb call. The total time spent
-            may exceed this value. If it is not specified, the value
-            set in the ADB constructor is used.
-        :raises: * ADBTimeoutError
-                 * ADBError
-
-        reboot() reboots the device, issues an adb wait-for-device in order to
-        wait for the device to complete rebooting, then calls is_device_ready()
-        to determine if the device has completed booting.
-
-        """
-        self.command_output(["reboot"], timeout=timeout)
-        self.command_output(["wait-for-device"], timeout=timeout)
-        return self.is_device_ready(timeout=timeout)
-
     # Application management methods
 
     def install_app(self, apk_path, timeout=None):
         """Installs an app on the device.
 
-        :param apk_path: string containing the apk file name to be
-            installed.
-        :param timeout: optional integer specifying the maximum time in
+        :param str apk_path: The apk file name to be installed.
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         data = self.command_output(["install", apk_path], timeout=timeout)
         if data.find('Success') == -1:
@@ -177,17 +273,16 @@ class ADBAndroidMixin(object):
     def is_app_installed(self, app_name, timeout=None):
         """Returns True if an app is installed on the device.
 
-        :param app_name: string containing the name of the app to be
-            checked.
-        :param timeout: optional integer specifying the maximum time in
+        :param str app_name: The name of the app to be checked.
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         pm_error_string = 'Error: Could not access the Package Manager'
         data = self.shell_output("pm list package %s" % app_name, timeout=timeout)
@@ -202,24 +297,26 @@ class ADBAndroidMixin(object):
                           timeout=None):
         """Launches an Android application
 
-        :param app_name: Name of application (e.g. `com.android.chrome`)
-        :param activity_name: Name of activity to launch (e.g. `.Main`)
-        :param intent: Intent to launch application with
+        :param str app_name: Name of application (e.g. `com.android.chrome`)
+        :param str activity_name: Name of activity to launch (e.g. `.Main`)
+        :param str intent: Intent to launch application with
         :param url: URL to open
-        :param extras: Dictionary of extra arguments for application.
-        :param wait: If True, wait for application to start before
+        :type url: str or None
+        :param extras: Extra arguments for application.
+        :type extras: dict or None
+        :param bool wait: If True, wait for application to start before
             returning.
-        :param fail_if_running: Raise an exception if instance of
+        :param bool fail_if_running: Raise an exception if instance of
             application is already running.
-        :param timeout: optional integer specifying the maximum time in
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         # If fail_if_running is True, we throw an exception here. Only one
         # instance of an application can be running at once on Android,
@@ -257,26 +354,29 @@ class ADBAndroidMixin(object):
         """Convenience method to launch IceCatMobile on Android with various
         debugging arguments
 
-        :param app_name: Name of icecatmobile application (e.g.
+        :param str app_name: Name of icecatmobile application (e.g.
             `org.mozilla.icecatmobile`)
-        :param intent: Intent to launch application.
+        :param str intent: Intent to launch application.
         :param moz_env: GNU specific environment to pass into
             application.
+        :type moz_env: str or None
         :param extra_args: Extra arguments to be parsed by icecatmobile.
+        :type extra_args: str or None
         :param url: URL to open
-        :param wait: If True, wait for application to start before
-            returning
-        :param fail_if_running: Raise an exception if instance of
-            application is already running
-        :param timeout: optional integer specifying the maximum time in
+        :type url: str or None
+        :param bool wait: If True, wait for application to start before
+            returning.
+        :param bool fail_if_running: Raise an exception if instance of
+            application is already running.
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         extras = {}
 
@@ -304,20 +404,22 @@ class ADBAndroidMixin(object):
         by the app repeatedly until none is around any more. This is
         less reliable and does require root.
 
-        :param app_name: Name of application (e.g. `com.android.chrome`)
-        :param timeout: optional integer specifying the maximum time in
+        :param str app_name: Name of application (e.g. `com.android.chrome`)
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
+        :param bool root: Flag specifying if the command should be
+            executed as root.
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
-        version = self.shell_output("getprop ro.build.version.release",
+        version = self.shell_output("getprop ro.build.version.sdk",
                                     timeout=timeout, root=root)
-        if StrictVersion(version) >= StrictVersion('3.0'):
+        if int(version) >= version_codes.HONEYCOMB:
             self.shell_output("am force-stop %s" % app_name,
                               timeout=timeout, root=root)
         else:
@@ -339,20 +441,20 @@ class ADBAndroidMixin(object):
     def uninstall_app(self, app_name, reboot=False, timeout=None):
         """Uninstalls an app on the device.
 
-        :param app_name: string containing the name of the app to be
+        :param str app_name: The name of the app to be
             uninstalled.
-        :param reboot: boolean flag indicating that the device should
+        :param bool reboot: Flag indicating that the device should
             be rebooted after the app is uninstalled. No reboot occurs
             if the app is not installed.
-        :param timeout: optional integer specifying the maximum time in
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         if self.is_app_installed(app_name, timeout=timeout):
             data = self.command_output(["uninstall", app_name], timeout=timeout)
@@ -365,37 +467,19 @@ class ADBAndroidMixin(object):
     def update_app(self, apk_path, timeout=None):
         """Updates an app on the device and reboots.
 
-        :param apk_path: string containing the apk file name to be
+        :param str apk_path: The apk file name to be
             updated.
-        :param timeout: optional integer specifying the maximum time in
+        :param timeout: The maximum time in
             seconds for any spawned adb process to complete before
             throwing an ADBTimeoutError.
             This timeout is per adb call. The total time spent
             may exceed this value. If it is not specified, the value
             set in the ADB constructor is used.
+        :type timeout: integer or None
         :raises: * ADBTimeoutError
                  * ADBError
-
         """
         output = self.command_output(["install", "-r", apk_path],
                                      timeout=timeout)
         self.reboot(timeout=timeout)
         return output
-
-
-class ADBAndroid(ADBDevice, ADBAndroidMixin):
-    """ADBAndroid provides all of the methods of :class:`mozdevice.ADB` with
-    Android specific extensions useful for that platform.
-
-    ::
-
-        from mozdevice import ADBAndroid as ADBDevice
-
-        adb = ADBDevice(...)
-
-        if adb.is_device_ready():
-            adb.install_app("/tmp/build.apk")
-            adb.launch_icecatmobile("org.mozilla.icecatmobile")
-
-    """
-    pass
