@@ -1,6 +1,6 @@
 #/usr/bin/env python
 import mozdevice
-import mozlog
+import logging
 import unittest
 from sut import MockAgent
 
@@ -12,7 +12,7 @@ class TestApp(unittest.TestCase):
                     "/data/data/org.gnu.icecat")]
 
         m = MockAgent(self, commands=command)
-        d = mozdevice.DroidSUT("127.0.0.1", port=m.port, logLevel=mozlog.DEBUG)
+        d = mozdevice.DroidSUT("127.0.0.1", port=m.port, logLevel=logging.DEBUG)
 
         self.assertEqual(command[0][1], d.getAppRoot('org.gnu.icecat'))
 
