@@ -10,6 +10,8 @@ config = {
         'sendchange',
     ],
     'stage_platform': 'macosx64-mulet',
+    'publish_nightly_en_US_routes': False,
+    'build_type': 'mulet-opt',
     'stage_product': 'b2g',
     'tooltool_manifest_src': "b2g/dev/config/tooltool-manifests/macosx64/releng.manifest",
     'objdir': 'obj-icecat',
