@@ -31,10 +31,8 @@ from mozharness.mozilla.testing.codecoverage import (
     code_coverage_config_options
 )
 from mozharness.mozilla.testing.testbase import TestingMixin, testing_config_options
-from mozharness.mozilla.buildbot import TBPL_WARNING
-
-SUITE_CATEGORIES = ['cppunittest', 'jittest', 'mochitest', 'reftest', 'xpcshell', 'mozbase', 'mozmill', 'webapprt']
 
+SUITE_CATEGORIES = ['gtest', 'cppunittest', 'jittest', 'mochitest', 'reftest', 'xpcshell', 'mozbase', 'mozmill', 'webapprt']
 
 # DesktopUnittest {{{1
 class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMixin, CodeCoverageMixin):
@@ -79,6 +77,14 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                     "Suites are defined in the config file\n."
                     "Examples: 'cppunittest'"}
          ],
+        [['--gtest-suite', ], {
+            "action": "extend",
+            "dest": "specified_gtest_suites",
+            "type": "string",
+            "help": "Specify which gtest suite to run. "
+                    "Suites are defined in the config file\n."
+                    "Examples: 'gtest'"}
+         ],
         [['--jittest-suite', ], {
             "action": "extend",
             "dest": "specified_jittest_suites",
@@ -207,6 +213,7 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
         dirs['abs_reftest_dir'] = os.path.join(dirs['abs_test_install_dir'], "reftest")
         dirs['abs_xpcshell_dir'] = os.path.join(dirs['abs_test_install_dir'], "xpcshell")
         dirs['abs_cppunittest_dir'] = os.path.join(dirs['abs_test_install_dir'], "cppunittest")
+        dirs['abs_gtest_dir'] = os.path.join(dirs['abs_test_install_dir'], "gtest")
         dirs['abs_blob_upload_dir'] = os.path.join(abs_dirs['abs_work_dir'], 'blobber_upload_dir')
         dirs['abs_jittest_dir'] = os.path.join(dirs['abs_test_install_dir'], "jit-test", "jit-test")
         dirs['abs_mozbase_dir'] = os.path.join(dirs['abs_test_install_dir'], "mozbase")
@@ -246,7 +253,7 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
 
         abs_app_dir = self.query_abs_app_dir()
         if self._is_darwin():
-            res_subdir = self.tree_config.get("mac_res_subdir", "Resources")
+            res_subdir = self.config.get("mac_res_subdir", "Resources")
             self.abs_res_dir = os.path.join(os.path.dirname(abs_app_dir), res_subdir)
         else:
             self.abs_res_dir = abs_app_dir
@@ -255,6 +262,9 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
     @PreScriptAction('create-virtualenv')
     def _pre_create_virtualenv(self, action):
         dirs = self.query_abs_dirs()
+
+        self.register_virtualenv_module(name='pip>=1.5')
+        self.register_virtualenv_module('psutil==3.1.1', method='pip')
         self.register_virtualenv_module(name='mock')
         self.register_virtualenv_module(name='simplejson')
 
@@ -326,13 +336,20 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
 
             raw_log_file = os.path.join(dirs['abs_blob_upload_dir'],
                                         '%s_raw.log' % suite)
+
+            error_summary_file = os.path.join(dirs['abs_blob_upload_dir'],
+                                              '%s_errorsummary.log' % suite)
             str_format_values = {
                 'binary_path': self.binary_path,
                 'symbols_path': self._query_symbols_url(),
                 'abs_app_dir': abs_app_dir,
                 'abs_res_dir': abs_res_dir,
                 'raw_log_file': raw_log_file,
+                'error_summary_file': error_summary_file,
+                'gtest_dir': os.path.join(dirs['abs_test_install_dir'],
+                                          'gtest'),
             }
+
             # TestingMixin._download_and_extract_symbols() will set
             # self.symbols_path when downloading/extracting.
             if self.symbols_path:
@@ -364,23 +381,10 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
             abs_res_plugins_dir = os.path.join(abs_res_dir, 'plugins')
             str_format_values['test_plugin_path'] = abs_res_plugins_dir
 
-            missing_key = True
-            if "suite_definitions" in self.tree_config: # new structure
-                if suite_category in self.tree_config["suite_definitions"]:
-                    missing_key = False
-                options = self.tree_config["suite_definitions"][suite_category]["options"]
-            else:
-                suite_options = '%s_options' % suite_category
-                if suite_options in self.tree_config:
-                    missing_key = False
-                options = self.tree_config[suite_options]
-
-            if missing_key:
-                self.fatal("'%s' not defined in the in-tree config! Please add it to '%s'. "
-                           "See bug 981030 for more details." %
-                           (suite_category,
-                            os.path.join('gecko', 'testing', self.config['in_tree_config'])))
+            if suite_category not in c["suite_definitions"]:
+                self.fatal("'%s' not defined in the config!")
 
+            options = c["suite_definitions"][suite_category]["options"]
             if options:
                 for option in options:
                     option = option % str_format_values
@@ -393,7 +397,14 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                              "please make sure they are specified in your "
                              "config under %s_options" %
                              (suite_category, suite_category))
-                return base_cmd
+
+
+            for option in options:
+                option = option % str_format_values
+                if not option.endswith('None'):
+                    base_cmd.append(option)
+
+            return base_cmd
         else:
             self.fatal("'binary_path' could not be determined.\n This should "
                        "be like '/path/build/application/icecat/icecat'"
@@ -429,6 +440,20 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
 
         return suites
 
+    def _query_try_flavor(self, category, suite):
+        flavors = {
+            "mochitest": [("plain.*", "mochitest"),
+                          ("browser-chrome.*", "browser-chrome"),
+                          ("mochitest-devtools-chrome.*", "devtools-chrome"),
+                          ("chrome", "chrome")],
+            "xpcshell": [("xpcshell", "xpcshell")],
+            "reftest": [("reftest", "reftest"),
+                        ("crashtest", "crashtest")]
+        }
+        for suite_pattern, flavor in flavors.get(category, []):
+            if re.compile(suite_pattern).match(suite):
+                return flavor
+
     # Actions {{{2
 
     # clobber defined in BaseScript, deletes mozharness/build if exists
@@ -474,13 +499,14 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                                   preflight_run_method=self.preflight_xpcshell)
         self._run_category_suites('cppunittest',
                                   preflight_run_method=self.preflight_cppunittest)
+        self._run_category_suites('gtest',
+                                  preflight_run_method=self.preflight_gtest)
         self._run_category_suites('jittest')
         self._run_category_suites('mozbase')
         self._run_category_suites('mozmill',
                                   preflight_run_method=self.preflight_mozmill)
 
-    def preflight_xpcshell(self, suites):
-        c = self.config
+    def preflight_copydirs(self, bin_name=None):
         dirs = self.query_abs_dirs()
         abs_app_dir = self.query_abs_app_dir()
 
@@ -491,24 +517,27 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
         abs_res_plugins_dir = os.path.join(abs_res_dir, 'plugins')
         abs_res_extensions_dir = os.path.join(abs_res_dir, 'extensions')
 
-        if suites:  # there are xpcshell suites to run
-            self.mkdir_p(abs_res_plugins_dir)
+        if bin_name:
             self.info('copying %s to %s' % (os.path.join(dirs['abs_test_bin_dir'],
-                      c['xpcshell_name']), os.path.join(abs_app_dir,
-                                                        c['xpcshell_name'])))
-            shutil.copy2(os.path.join(dirs['abs_test_bin_dir'], c['xpcshell_name']),
-                         os.path.join(abs_app_dir, c['xpcshell_name']))
-            self.copytree(dirs['abs_test_bin_components_dir'],
-                          abs_res_components_dir,
+                      bin_name), os.path.join(abs_app_dir, bin_name)))
+            shutil.copy2(os.path.join(dirs['abs_test_bin_dir'], bin_name),
+                         os.path.join(abs_app_dir, bin_name))
+
+        self.copytree(dirs['abs_test_bin_components_dir'],
+                      abs_res_components_dir,
+                      overwrite='overwrite_if_exists')
+        self.mkdir_p(abs_res_plugins_dir)
+        self.copytree(dirs['abs_test_bin_plugins_dir'],
+                      abs_res_plugins_dir,
+                      overwrite='overwrite_if_exists')
+        if os.path.isdir(dirs['abs_test_extensions_dir']):
+            self.mkdir_p(abs_res_extensions_dir)
+            self.copytree(dirs['abs_test_extensions_dir'],
+                          abs_res_extensions_dir,
                           overwrite='overwrite_if_exists')
-            self.copytree(dirs['abs_test_bin_plugins_dir'],
-                          abs_res_plugins_dir,
-                          overwrite='overwrite_if_exists')
-            if os.path.isdir(dirs['abs_test_extensions_dir']):
-                self.mkdir_p(abs_res_extensions_dir)
-                self.copytree(dirs['abs_test_extensions_dir'],
-                              abs_res_extensions_dir,
-                              overwrite='overwrite_if_exists')
+
+    def preflight_xpcshell(self, suites):
+        self.preflight_copydirs(self.config['xpcshell_name'])
 
     def preflight_cppunittest(self, suites):
         abs_res_dir = self.query_abs_res_dir()
@@ -521,26 +550,29 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
         for f in files:
             self.move(f, abs_res_dir)
 
-    def preflight_mozmill(self, suites):
-        c = self.config
-        dirs = self.query_abs_dirs()
+    def preflight_gtest(self, suites):
+        abs_res_dir = self.query_abs_res_dir()
         abs_app_dir = self.query_abs_app_dir()
-        abs_app_plugins_dir = os.path.join(abs_app_dir, 'plugins')
-        abs_app_extensions_dir = os.path.join(abs_app_dir, 'extensions')
-        if suites:  # there are mozmill suites to run
-            self.mkdir_p(abs_app_plugins_dir)
-            self.copytree(dirs['abs_test_bin_plugins_dir'],
-                          abs_app_plugins_dir,
-                          overwrite='overwrite_if_exists')
-            if os.path.isdir(dirs['abs_test_extensions_dir']):
-                self.copytree(dirs['abs_test_extensions_dir'],
-                              abs_app_extensions_dir,
-                              overwrite='overwrite_if_exists')
-            modules = ['jsbridge', 'mozmill']
-            for module in modules:
-                self.install_module(module=os.path.join(dirs['abs_mozmill_dir'],
-                                                        'resources',
-                                                        module))
+        dirs = self.query_abs_dirs()
+        abs_gtest_dir = dirs['abs_gtest_dir']
+        dirs['abs_test_bin_dir'] = os.path.join(dirs['abs_test_install_dir'], 'bin')
+
+        files = glob.glob(os.path.join(dirs['abs_test_bin_plugins_dir'], 'gmp-*'))
+        files.append(os.path.join(abs_gtest_dir, 'dependentlibs.list.gtest'))
+        for f in files:
+            self.move(f, abs_res_dir)
+
+        self.copytree(os.path.join(abs_gtest_dir, 'gtest_bin'),
+                      os.path.join(abs_app_dir))
+
+    def preflight_mozmill(self, suites):
+        self.preflight_copydirs()
+        dirs = self.query_abs_dirs()
+        modules = ['jsbridge', 'mozmill']
+        for module in modules:
+            self.install_module(module=os.path.join(dirs['abs_mozmill_dir'],
+                                                    'resources',
+                                                    module))
 
     def _run_category_suites(self, suite_category, preflight_run_method=None):
         """run suite(s) to a specific category"""
@@ -550,9 +582,9 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
         abs_app_dir = self.query_abs_app_dir()
         abs_res_dir = self.query_abs_res_dir()
 
-        if preflight_run_method:
-            preflight_run_method(suites)
         if suites:
+            if preflight_run_method:
+                preflight_run_method(suites)
             self.info('#### Running %s suites' % suite_category)
             for suite in suites:
                 abs_base_cmd = self._query_abs_base_cmd(suite_category, suite)
@@ -567,15 +599,22 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                 options_list = []
                 env = {}
                 if isinstance(suites[suite], dict):
-                    options_list = suites[suite]['options']
-                    env = copy.deepcopy(suites[suite]['env'])
+                    options_list = suites[suite].get('options', [])
+                    tests_list = suites[suite].get('tests', [])
+                    env = copy.deepcopy(suites[suite].get('env', {}))
                 else:
                     options_list = suites[suite]
+                    tests_list = []
 
-                for arg in options_list:
-                    cmd.append(arg % replace_dict)
+                flavor = self._query_try_flavor(suite_category, suite)
+                try_options, try_tests = self.try_args(flavor)
 
-                cmd = self.append_harness_extra_args(cmd)
+                cmd.extend(self.query_options(options_list,
+                                              try_options,
+                                              str_format_values=replace_dict))
+                cmd.extend(self.query_tests_args(tests_list,
+                                                 try_tests,
+                                                 str_format_values=replace_dict))
 
                 suite_name = suite_category + '-' + suite
                 tbpl_status, log_level = None, None
@@ -595,8 +634,9 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                 if not os.path.isdir(env['MOZ_UPLOAD_DIR']):
                     self.mkdir_p(env['MOZ_UPLOAD_DIR'])
                 env = self.query_env(partial_env=env, log_level=INFO)
+                cmd_timeout = 2500 if suite_category == 'cppunittest' else 1000
                 return_code = self.run_command(cmd, cwd=dirs['abs_work_dir'],
-                                               output_timeout=1000,
+                                               output_timeout=cmd_timeout,
                                                output_parser=parser,
                                                env=env)
 
@@ -611,7 +651,7 @@ class DesktopUnittest(TestingMixin, MercurialScript, BlobUploadMixin, MozbaseMix
                 # 3) checking to see if the return code is in success_codes
 
                 success_codes = None
-                if self._is_windows():
+                if self._is_windows() and suite_category != 'gtest':
                     # bug 1120644
                     success_codes = [0, 1]
 
