@@ -12,10 +12,10 @@ import runcppunittests as cppunittests
 import mozcrash
 import mozfile
 import mozinfo
+import mozlog
 import StringIO
 import posixpath
 from mozdevice import devicemanager, devicemanagerADB, devicemanagerSUT
-from mozlog import structured
 
 try:
     from mozbuild.base import MozbuildObject
@@ -52,43 +52,68 @@ class RemoteCPPUnitTests(cppunittests.CPPUnitTests):
         self.device.chmodDir(self.remote_bin_dir)
 
     def push_libs(self):
+        if self.options.local_bin is not None:
+            szip = os.path.join(self.options.local_bin, '..', 'host', 'bin', 'szip')
+            if not os.path.exists(szip):
+                # Tinderbox builds must run szip from the test package
+                szip = os.path.join(self.options.local_bin, 'host', 'szip')
+            if not os.path.exists(szip):
+                # If the test package doesn't contain szip, it means files
+                # are not szipped in the test package.
+                szip = None
+        else:
+            szip = None
         if self.options.local_apk:
             with mozfile.TemporaryDirectory() as tmpdir:
                 apk_contents = ZipFile(self.options.local_apk)
-                szip = os.path.join(self.options.local_bin, '..', 'host', 'bin', 'szip')
-                if not os.path.exists(szip):
-                    # Tinderbox builds must run szip from the test package
-                    szip = os.path.join(self.options.local_bin, 'host', 'szip')
-                if not os.path.exists(szip):
-                    # If the test package doesn't contain szip, it means files
-                    # are not szipped in the test package.
-                    szip = None
 
                 for info in apk_contents.infolist():
                     if info.filename.endswith(".so"):
                         print >> sys.stderr, "Pushing %s.." % info.filename
                         remote_file = posixpath.join(self.remote_bin_dir, os.path.basename(info.filename))
                         apk_contents.extract(info, tmpdir)
-                        file = os.path.join(tmpdir, info.filename)
+                        local_file = os.path.join(tmpdir, info.filename)
                         if szip:
-                            out = subprocess.check_output([szip, '-d', file], stderr=subprocess.STDOUT)
-                        self.device.pushFile(os.path.join(tmpdir, info.filename), remote_file)
-            return
+                            try:
+                                out = subprocess.check_output([szip, '-d', local_file], stderr=subprocess.STDOUT)
+                            except CalledProcessError:
+                                print >> sys.stderr, "Error calling %s on %s.." % (szip, local_file)
+                                if out:
+                                    print >> sys.stderr, out
+                        self.device.pushFile(local_file, remote_file)
 
         elif self.options.local_lib:
             for file in os.listdir(self.options.local_lib):
                 if file.endswith(".so"):
                     print >> sys.stderr, "Pushing %s.." % file
                     remote_file = posixpath.join(self.remote_bin_dir, file)
-                    self.device.pushFile(os.path.join(self.options.local_lib, file), remote_file)
+                    local_file = os.path.join(self.options.local_lib, file)
+                    if szip:
+                        try:
+                            out = subprocess.check_output([szip, '-d', local_file], stderr=subprocess.STDOUT)
+                        except CalledProcessError:
+                            print >> sys.stderr, "Error calling %s on %s.." % (szip, local_file)
+                            if out:
+                                print >> sys.stderr, out
+                    self.device.pushFile(local_file, remote_file)
             # Additional libraries may be found in a sub-directory such as "lib/armeabi-v7a"
-            local_arm_lib = os.path.join(self.options.local_lib, "lib")
-            if os.path.isdir(local_arm_lib):
-                for root, dirs, files in os.walk(local_arm_lib):
-                    for file in files:
-                        if (file.endswith(".so")):
-                            remote_file = posixpath.join(self.remote_bin_dir, file)
-                            self.device.pushFile(os.path.join(root, file), remote_file)
+            for subdir in ["assets", "lib"]:
+                local_arm_lib = os.path.join(self.options.local_lib, subdir)
+                if os.path.isdir(local_arm_lib):
+                    for root, dirs, files in os.walk(local_arm_lib):
+                        for file in files:
+                            if (file.endswith(".so")):
+                                print >> sys.stderr, "Pushing %s.." % file
+                                remote_file = posixpath.join(self.remote_bin_dir, file)
+                                local_file = os.path.join(root, file)
+                                if szip:
+                                    try:
+                                        out = subprocess.check_output([szip, '-d', local_file], stderr=subprocess.STDOUT)
+                                    except CalledProcessError:
+                                        print >> sys.stderr, "Error calling %s on %s.." % (szip, local_file)
+                                        if out:
+                                            print >> sys.stderr, out
+                                self.device.pushFile(local_file, remote_file)
 
     def push_progs(self, progs):
         for local_file in progs:
@@ -114,7 +139,8 @@ class RemoteCPPUnitTests(cppunittests.CPPUnitTests):
 
         return env
 
-    def run_one_test(self, prog, env, symbols_path=None, interactive=False):
+    def run_one_test(self, prog, env, symbols_path=None, interactive=False,
+                     timeout_factor=1):
         """
         Run a single C++ unit test program remotely.
 
@@ -123,6 +149,7 @@ class RemoteCPPUnitTests(cppunittests.CPPUnitTests):
         * env: The environment to use for running the program.
         * symbols_path: A path to a directory containing Breakpad-formatted
                         symbol files for producing stack traces on crash.
+        * timeout_factor: An optional test-specific timeout multiplier.
 
         Return True if the program exits with a zero status, False otherwise.
         """
@@ -130,8 +157,9 @@ class RemoteCPPUnitTests(cppunittests.CPPUnitTests):
         remote_bin = posixpath.join(self.remote_bin_dir, basename)
         self.log.test_start(basename)
         buf = StringIO.StringIO()
+        test_timeout = cppunittests.CPPUnitTests.TEST_PROC_TIMEOUT * timeout_factor
         returncode = self.device.shell([remote_bin], buf, env=env, cwd=self.remote_home_dir,
-                                       timeout=cppunittests.CPPUnitTests.TEST_PROC_TIMEOUT)
+                                       timeout=test_timeout)
         self.log.process_output(basename, "\n%s" % buf.getvalue(),
                                 command=[remote_bin])
         with mozfile.TemporaryDirectory() as tempdir:
@@ -192,13 +220,15 @@ class RemoteCPPUnittestOptions(cppunittests.CPPUnittestOptions):
         self.add_option("--remoteTestRoot", action = "store",
                     type = "string", dest = "remote_test_root",
                     help = "remote directory to use as test root (eg. /data/local/tests)")
-        self.add_option("--with-b2g-emulator", action = "store",
-                    type = "string", dest = "with_b2g_emulator",
-                    help = "Start B2G Emulator (specify path to b2g home)")
         # /data/local/tests is used because it is usually not possible to set +x permissions
         # on binaries on /mnt/sdcard
         defaults["remote_test_root"] = "/data/local/tests"
 
+        self.add_option("--with-b2g-emulator", action = "store",
+                    type = "string", dest = "with_b2g_emulator",
+                    help = "Start B2G Emulator (specify path to b2g home)")
+        self.add_option("--emulator", default="arm", choices=["x86", "arm"],
+                    help = "Architecture of emulator to use: x86 or arm")
         self.add_option("--addEnv", action = "append",
                     type = "string", dest = "add_env",
                     help = "additional remote environment variable definitions (eg. --addEnv \"somevar=something\")")
@@ -206,25 +236,10 @@ class RemoteCPPUnittestOptions(cppunittests.CPPUnittestOptions):
 
         self.set_defaults(**defaults)
 
-def main():
-    parser = RemoteCPPUnittestOptions()
-    structured.commandline.add_logging_group(parser)
-    options, args = parser.parse_args()
-    if not args:
-        print >>sys.stderr, """Usage: %s <test binary> [<test binary>...]""" % sys.argv[0]
-        sys.exit(1)
-    if options.local_lib is not None and not os.path.isdir(options.local_lib):
-        print >>sys.stderr, """Error: --localLib directory %s not found""" % options.local_lib
-        sys.exit(1)
-    if options.local_apk is not None and not os.path.isfile(options.local_apk):
-        print >>sys.stderr, """Error: --apk file %s not found""" % options.local_apk
-        sys.exit(1)
-    if not options.xre_path:
-        print >>sys.stderr, """Error: --xre-path is required"""
-        sys.exit(1)
+def run_test_harness(options, args):
     if options.with_b2g_emulator:
         from mozrunner import B2GEmulatorRunner
-        runner = B2GEmulatorRunner(b2g_home=options.with_b2g_emulator)
+        runner = B2GEmulatorRunner(arch=options.emulator, b2g_home=options.with_b2g_emulator)
         runner.start()
     if options.dm_trans == "adb":
         if options.with_b2g_emulator:
@@ -249,23 +264,44 @@ def main():
             print "Error: you must provide a device IP to connect to via the --deviceIP option"
             sys.exit(1)
 
-    log = structured.commandline.setup_logging("remotecppunittests",
-                                               options,
-                                               {"tbpl": sys.stdout})
-
-
     options.xre_path = os.path.abspath(options.xre_path)
     cppunittests.update_mozinfo()
-    progs = cppunittests.extract_unittests_from_args(args, mozinfo.info)
-    tester = RemoteCPPUnitTests(dm, options, progs)
+    progs = cppunittests.extract_unittests_from_args(args,
+                                                     mozinfo.info,
+                                                     options.manifest_path)
+    tester = RemoteCPPUnitTests(dm, options, [item[0] for item in progs])
     try:
         result = tester.run_tests(progs, options.xre_path, options.symbols_path)
+    finally:
+        if options.with_b2g_emulator:
+            runner.cleanup()
+            runner.wait()
+    return result
+
+def main():
+    parser = RemoteCPPUnittestOptions()
+    mozlog.commandline.add_logging_group(parser)
+    options, args = parser.parse_args()
+    if not args:
+        print >>sys.stderr, """Usage: %s <test binary> [<test binary>...]""" % sys.argv[0]
+        sys.exit(1)
+    if options.local_lib is not None and not os.path.isdir(options.local_lib):
+        print >>sys.stderr, """Error: --localLib directory %s not found""" % options.local_lib
+        sys.exit(1)
+    if options.local_apk is not None and not os.path.isfile(options.local_apk):
+        print >>sys.stderr, """Error: --apk file %s not found""" % options.local_apk
+        sys.exit(1)
+    if not options.xre_path:
+        print >>sys.stderr, """Error: --xre-path is required"""
+        sys.exit(1)
+
+    log = mozlog.commandline.setup_logging("remotecppunittests", options,
+                                           {"tbpl": sys.stdout})
+    try:
+        result = run_test_harness(options, args)
     except Exception, e:
         log.error(str(e))
         result = False
-    if options.with_b2g_emulator:
-        runner.cleanup()
-        runner.wait()
     sys.exit(0 if result else 1)
 
 if __name__ == '__main__':
