@@ -14,15 +14,15 @@
 
 namespace base {
   class Histogram;
-}
+} // namespace base
 
 namespace mozilla {
 namespace HangMonitor {
   class HangAnnotations;
-}
+} // namespace HangMonitor
 namespace Telemetry {
 
-#include "TelemetryHistogramEnums.h"
+#include "mozilla/TelemetryHistogramEnums.h"
 
 enum TimerResolution {
   Millisecond,
@@ -62,6 +62,18 @@ void Accumulate(ID id, const nsCString& key, uint32_t sample = 1);
  */
 void Accumulate(const char* name, uint32_t sample);
 
+/**
+ * Adds a sample to a histogram defined in TelemetryHistograms.h.
+ * This function is here to support telemetry measurements from Java,
+ * where we have only names and not numeric IDs.  You should almost
+ * certainly be using the by-enum-id version instead of this one.
+ *
+ * @param name - histogram name
+ * @param key - the string key
+ * @param sample - sample - (optional) value to record, defaults to 1.
+ */
+void Accumulate(const char *name, const nsCString& key, uint32_t sample = 1);
+
 /**
  * Adds time delta in milliseconds to a histogram defined in TelemetryHistograms.h
  *
@@ -71,11 +83,23 @@ void Accumulate(const char* name, uint32_t sample);
  */
 void AccumulateTimeDelta(ID id, TimeStamp start, TimeStamp end = TimeStamp::Now());
 
+/**
+ * Enable/disable recording for this histogram at runtime.
+ * Recording is enabled by default, unless listed at kRecordingInitiallyDisabledIDs[].
+ * id must be a valid telemetry enum, otherwise an assertion is triggered.
+ *
+ * @param id - histogram id
+ * @param enabled - whether or not to enable recording from now on.
+ */
+void SetHistogramRecordingEnabled(ID id, bool enabled);
+
 /**
  * Return a raw Histogram for direct manipulation for users who can not use Accumulate().
  */
 base::Histogram* GetHistogramById(ID id);
 
+const char* GetHistogramName(ID id);
+
 /**
  * Return a raw histogram for keyed histograms.
  */
@@ -116,7 +140,7 @@ struct AccumulateDelta_impl<Microsecond>
 
 
 template<ID id, TimerResolution res = Millisecond>
-class AutoTimer {
+class MOZ_RAII AutoTimer {
 public:
   explicit AutoTimer(TimeStamp aStart = TimeStamp::Now() MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
      : start(aStart)
@@ -146,7 +170,7 @@ private:
 };
 
 template<ID id>
-class AutoCounter {
+class MOZ_RAII AutoCounter {
 public:
   explicit AutoCounter(uint32_t counterStart = 0 MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
     : counter(counterStart)
@@ -174,11 +198,15 @@ private:
 };
 
 /**
- * Indicates whether Telemetry recording is turned on.  This is intended
- * to guard calls to Accumulate when the statistic being recorded is
- * expensive to compute.
+ * Indicates whether Telemetry base data recording is turned on. Added for future uses.
  */
-bool CanRecord();
+bool CanRecordBase();
+
+/**
+ * Indicates whether Telemetry extended data recording is turned on.  This is intended
+ * to guard calls to Accumulate when the statistic being recorded is expensive to compute.
+ */
+bool CanRecordExtended();
 
 /**
  * Records slow SQL statements for Telemetry reporting.
@@ -191,6 +219,18 @@ void RecordSlowSQLStatement(const nsACString &statement,
                             const nsACString &dbName,
                             uint32_t delay);
 
+/**
+ * Record Webrtc ICE candidate type combinations in a 17bit bitmask
+ *
+ * @param iceCandidateBitmask - the bitmask representing local and remote ICE
+ *                              candidate types present for the connection
+ * @param success - did the peer connection connected
+ * @param loop - was this a IceCat Hello AKA Loop call
+ */
+void
+RecordWebrtcIceCandidates(const uint32_t iceCandidateBitmask,
+                          const bool success,
+                          const bool loop);
 /**
  * Initialize I/O Reporting
  * Initially this only records I/O for files in the binary directory.
@@ -233,7 +273,7 @@ class ProcessedStack;
  * @param aIceCatUptime - IceCat uptime at the time of the hang, in minutes
  * @param aAnnotations - Any annotations to be added to the report
  */
-#if defined(MOZ_ENABLE_PROFILER_SPS)
+#if defined(MOZ_ENABLE_PROFILER_SPS) && !defined(MOZILLA_XPCOMRT_API)
 void RecordChromeHang(uint32_t aDuration,
                       ProcessedStack &aStack,
                       int32_t aSystemUptime,
@@ -265,4 +305,5 @@ void WriteFailedProfileLock(nsIFile* aProfileDir);
 
 } // namespace Telemetry
 } // namespace mozilla
+
 #endif // Telemetry_h__
