@@ -9,7 +9,7 @@
 // and it's tricky to use the window-watcher to check a window *does not*
 // appear - how long should the watcher be active before assuming it's not
 // going to appear?)
-function runTests() {
+function* runTests() {
   let url = "http://mochi.test:8888/browser/browser/base/content/test/general/authenticate.sjs?user=anyone";
   ok(!thumbnailExists(url), "Thumbnail file should not already exist.");
 
