@@ -6,27 +6,23 @@
 #include "ProtocolParser.h"
 #include "LookupCache.h"
 #include "nsNetCID.h"
-#include "prlog.h"
+#include "mozilla/Logging.h"
 #include "prnetdb.h"
 #include "prprf.h"
 
 #include "nsUrlClassifierUtils.h"
 
-// NSPR_LOG_MODULES=UrlClassifierDbService:5
-extern PRLogModuleInfo *gUrlClassifierDbServiceLog;
-#if defined(PR_LOGGING)
-#define LOG(args) PR_LOG(gUrlClassifierDbServiceLog, PR_LOG_DEBUG, args)
-#define LOG_ENABLED() PR_LOG_TEST(gUrlClassifierDbServiceLog, 4)
-#else
-#define LOG(args)
-#define LOG_ENABLED() (false)
-#endif
+// NSPR_LOG_MODULES=UrlClassifierProtocolParser:5
+PRLogModuleInfo *gUrlClassifierProtocolParserLog = nullptr;
+#define PARSER_LOG(args) MOZ_LOG(gUrlClassifierProtocolParserLog, mozilla::LogLevel::Debug, args)
 
 namespace mozilla {
 namespace safebrowsing {
 
 // Updates will fail if fed chunks larger than this
 const uint32_t MAX_CHUNK_SIZE = (1024 * 1024);
+// Updates will fail if the total number of tocuhed chunks is larger than this
+const uint32_t MAX_CHUNK_RANGE = 1000000;
 
 const uint32_t DOMAIN_SIZE = 4;
 
@@ -69,6 +65,7 @@ ProtocolParser::ProtocolParser()
   , mUpdateStatus(NS_OK)
   , mUpdateWait(0)
   , mResetRequested(false)
+  , mTableUpdate(nullptr)
 {
 }
 
@@ -80,6 +77,10 @@ ProtocolParser::~ProtocolParser()
 nsresult
 ProtocolParser::Init(nsICryptoHash* aHasher)
 {
+  if (!gUrlClassifierProtocolParserLog) {
+    gUrlClassifierProtocolParserLog =
+      PR_NewLogModule("UrlClassifierProtocolParser");
+  }
   mCryptoHash = aHasher;
   return NS_OK;
 }
@@ -125,14 +126,14 @@ ProtocolParser::ProcessControl(bool* aDone)
   nsAutoCString line;
   *aDone = true;
   while (NextLine(line)) {
-    //LOG(("Processing %s\n", line.get()));
+    PARSER_LOG(("Processing %s\n", line.get()));
 
     if (StringBeginsWith(line, NS_LITERAL_CSTRING("i:"))) {
       // Set the table name from the table header line.
       SetCurrentTable(Substring(line, 2));
     } else if (StringBeginsWith(line, NS_LITERAL_CSTRING("n:"))) {
       if (PR_sscanf(line.get(), "n:%d", &mUpdateWait) != 1) {
-        LOG(("Error parsing n: '%s' (%d)", line.get(), mUpdateWait));
+        PARSER_LOG(("Error parsing n: '%s' (%d)", line.get(), mUpdateWait));
         mUpdateWait = 0;
       }
     } else if (line.EqualsLiteral("r:pleasereset")) {
@@ -171,11 +172,20 @@ ProtocolParser::ProcessExpirations(const nsCString& aLine)
   while (begin != end) {
     uint32_t first, last;
     if (ParseChunkRange(begin, end, &first, &last)) {
+      if (last < first) return NS_ERROR_FAILURE;
+      if (last - first > MAX_CHUNK_RANGE) return NS_ERROR_FAILURE;
       for (uint32_t num = first; num <= last; num++) {
-        if (aLine[0] == 'a')
-          mTableUpdate->NewAddExpiration(num);
-        else
-          mTableUpdate->NewSubExpiration(num);
+        if (aLine[0] == 'a') {
+          nsresult rv = mTableUpdate->NewAddExpiration(num);
+          if (NS_FAILED(rv)) {
+            return rv;
+          }
+        } else {
+          nsresult rv = mTableUpdate->NewSubExpiration(num);
+          if (NS_FAILED(rv)) {
+            return rv;
+          }
+        }
       }
     } else {
       return NS_ERROR_FAILURE;
@@ -203,10 +213,12 @@ ProtocolParser::ProcessChunkControl(const nsCString& aLine)
                 &mChunkState.num, &mChunkState.hashSize, &mChunkState.length)
       != 4)
   {
+    NS_WARNING(("PR_sscanf failed"));
     return NS_ERROR_FAILURE;
   }
 
   if (mChunkState.length > MAX_CHUNK_SIZE) {
+    NS_WARNING("Invalid length specified in update.");
     return NS_ERROR_FAILURE;
   }
 
@@ -223,21 +235,33 @@ ProtocolParser::ProcessChunkControl(const nsCString& aLine)
     mChunkState.type = (command == 'a') ? CHUNK_ADD : CHUNK_SUB;
   } else if (StringEndsWith(mTableUpdate->TableName(),
     NS_LITERAL_CSTRING("-digest256"))) {
-    LOG(("Processing digest256 data"));
     mChunkState.type = (command == 'a') ? CHUNK_ADD_DIGEST : CHUNK_SUB_DIGEST;
   }
+  nsresult rv;
   switch (mChunkState.type) {
     case CHUNK_ADD:
-      mTableUpdate->NewAddChunk(mChunkState.num);
+      rv = mTableUpdate->NewAddChunk(mChunkState.num);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
       break;
     case CHUNK_SUB:
-      mTableUpdate->NewSubChunk(mChunkState.num);
+      rv = mTableUpdate->NewSubChunk(mChunkState.num);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
       break;
     case CHUNK_ADD_DIGEST:
-      mTableUpdate->NewAddChunk(mChunkState.num);
+      rv = mTableUpdate->NewAddChunk(mChunkState.num);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
       break;
     case CHUNK_SUB_DIGEST:
-      mTableUpdate->NewSubChunk(mChunkState.num);
+      rv = mTableUpdate->NewSubChunk(mChunkState.num);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
       break;
   }
 
@@ -289,7 +313,6 @@ ProtocolParser::ProcessChunk(bool* aDone)
   *aDone = false;
   mState = PROTOCOL_STATE_CONTROL;
 
-  //LOG(("Handling a %d-byte chunk", chunk.Length()));
   if (StringEndsWith(mTableUpdate->TableName(),
                      NS_LITERAL_CSTRING("-shavar"))) {
     return ProcessShaChunk(chunk);
@@ -312,6 +335,8 @@ ProtocolParser::ProcessPlaintextChunk(const nsACString& aChunk)
     return NS_ERROR_FAILURE;
   }
 
+  PARSER_LOG(("Handling a %d-byte simple chunk", aChunk.Length()));
+
   nsTArray<nsCString> lines;
   ParseString(PromiseFlatCString(aChunk), '\n', lines);
 
@@ -323,12 +348,18 @@ ProtocolParser::ProcessPlaintextChunk(const nsACString& aChunk)
       if (mChunkState.hashSize == COMPLETE_SIZE) {
         Completion hash;
         hash.FromPlaintext(line, mCryptoHash);
-        mTableUpdate->NewAddComplete(mChunkState.num, hash);
+        nsresult rv = mTableUpdate->NewAddComplete(mChunkState.num, hash);
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
       } else {
         NS_ASSERTION(mChunkState.hashSize == 4, "Only 32- or 4-byte hashes can be used for add chunks.");
         Prefix hash;
         hash.FromPlaintext(line, mCryptoHash);
-        mTableUpdate->NewAddPrefix(mChunkState.num, hash);
+        nsresult rv = mTableUpdate->NewAddPrefix(mChunkState.num, hash);
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
       }
     } else {
       nsCString::const_iterator begin, iter, end;
@@ -346,12 +377,18 @@ ProtocolParser::ProcessPlaintextChunk(const nsACString& aChunk)
       if (mChunkState.hashSize == COMPLETE_SIZE) {
         Completion hash;
         hash.FromPlaintext(Substring(iter, end), mCryptoHash);
-        mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+        nsresult rv = mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
       } else {
         NS_ASSERTION(mChunkState.hashSize == 4, "Only 32- or 4-byte hashes can be used for add chunks.");
         Prefix hash;
         hash.FromPlaintext(Substring(iter, end), mCryptoHash);
-        mTableUpdate->NewSubPrefix(addChunk, hash, mChunkState.num);
+        nsresult rv = mTableUpdate->NewSubPrefix(addChunk, hash, mChunkState.num);
+        if (NS_FAILED(rv)) {
+          return rv;
+        }
       }
     }
   }
@@ -373,6 +410,10 @@ ProtocolParser::ProcessShaChunk(const nsACString& aChunk)
     uint8_t numEntries = static_cast<uint8_t>(aChunk[start]);
     start++;
 
+    PARSER_LOG(("Handling a %d-byte shavar chunk containing %u entries"
+                " for domain %X", aChunk.Length(), numEntries,
+                domain.ToUint32()));
+
     nsresult rv;
     if (mChunkState.type == CHUNK_ADD && mChunkState.hashSize == PREFIX_SIZE) {
       rv = ProcessHostAdd(domain, numEntries, aChunk, &start);
@@ -384,7 +425,7 @@ ProtocolParser::ProcessShaChunk(const nsACString& aChunk)
       rv = ProcessHostSubComplete(numEntries, aChunk, &start);
     } else {
       NS_WARNING("Unexpected chunk type/hash size!");
-      LOG(("Got an unexpected chunk type/hash size: %s:%d",
+      PARSER_LOG(("Got an unexpected chunk type/hash size: %s:%d",
            mChunkState.type == CHUNK_ADD ? "add" : "sub",
            mChunkState.hashSize));
       return NS_ERROR_FAILURE;
@@ -398,6 +439,8 @@ ProtocolParser::ProcessShaChunk(const nsACString& aChunk)
 nsresult
 ProtocolParser::ProcessDigestChunk(const nsACString& aChunk)
 {
+  PARSER_LOG(("Handling a %d-byte digest256 chunk", aChunk.Length()));
+
   if (mChunkState.type == CHUNK_ADD_DIGEST) {
     return ProcessDigestAdd(aChunk);
   }
@@ -418,7 +461,10 @@ ProtocolParser::ProcessDigestAdd(const nsACString& aChunk)
     Completion hash;
     hash.Assign(Substring(aChunk, start, COMPLETE_SIZE));
     start += COMPLETE_SIZE;
-    mTableUpdate->NewAddComplete(mChunkState.num, hash);
+    nsresult rv = mTableUpdate->NewAddComplete(mChunkState.num, hash);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
   }
   return NS_OK;
 }
@@ -445,7 +491,10 @@ ProtocolParser::ProcessDigestSub(const nsACString& aChunk)
     hash.Assign(Substring(aChunk, start, COMPLETE_SIZE));
     start += COMPLETE_SIZE;
 
-    mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+    nsresult rv = mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
   }
   return NS_OK;
 }
@@ -458,7 +507,10 @@ ProtocolParser::ProcessHostAdd(const Prefix& aDomain, uint8_t aNumEntries,
                "ProcessHostAdd should only be called for prefix hashes.");
 
   if (aNumEntries == 0) {
-    mTableUpdate->NewAddPrefix(mChunkState.num, aDomain);
+    nsresult rv = mTableUpdate->NewAddPrefix(mChunkState.num, aDomain);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
     return NS_OK;
   }
 
@@ -470,7 +522,11 @@ ProtocolParser::ProcessHostAdd(const Prefix& aDomain, uint8_t aNumEntries,
   for (uint8_t i = 0; i < aNumEntries; i++) {
     Prefix hash;
     hash.Assign(Substring(aChunk, *aStart, PREFIX_SIZE));
-    mTableUpdate->NewAddPrefix(mChunkState.num, hash);
+    PARSER_LOG(("Add prefix %X", hash.ToUint32()));
+    nsresult rv = mTableUpdate->NewAddPrefix(mChunkState.num, hash);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
     *aStart += PREFIX_SIZE;
   }
 
@@ -497,7 +553,11 @@ ProtocolParser::ProcessHostSub(const Prefix& aDomain, uint8_t aNumEntries,
     memcpy(&addChunk, addChunkStr.BeginReading(), 4);
     addChunk = PR_ntohl(addChunk);
 
-    mTableUpdate->NewSubPrefix(addChunk, aDomain, mChunkState.num);
+    PARSER_LOG(("Sub prefix (addchunk=%u)", addChunk));
+    nsresult rv = mTableUpdate->NewSubPrefix(addChunk, aDomain, mChunkState.num);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
     return NS_OK;
   }
 
@@ -518,7 +578,11 @@ ProtocolParser::ProcessHostSub(const Prefix& aDomain, uint8_t aNumEntries,
     prefix.Assign(Substring(aChunk, *aStart, PREFIX_SIZE));
     *aStart += PREFIX_SIZE;
 
-    mTableUpdate->NewSubPrefix(addChunk, prefix, mChunkState.num);
+    PARSER_LOG(("Sub prefix %X (addchunk=%u)", prefix.ToUint32(), addChunk));
+    nsresult rv = mTableUpdate->NewSubPrefix(addChunk, prefix, mChunkState.num);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
   }
 
   return NS_OK;
@@ -546,7 +610,10 @@ ProtocolParser::ProcessHostAddComplete(uint8_t aNumEntries,
   for (uint8_t i = 0; i < aNumEntries; i++) {
     Completion hash;
     hash.Assign(Substring(aChunk, *aStart, COMPLETE_SIZE));
-    mTableUpdate->NewAddComplete(mChunkState.num, hash);
+    nsresult rv = mTableUpdate->NewAddComplete(mChunkState.num, hash);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
     *aStart += COMPLETE_SIZE;
   }
 
@@ -583,7 +650,10 @@ ProtocolParser::ProcessHostSubComplete(uint8_t aNumEntries,
     memcpy(&addChunk, addChunkStr.BeginReading(), 4);
     addChunk = PR_ntohl(addChunk);
 
-    mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+    nsresult rv = mTableUpdate->NewSubComplete(addChunk, hash, mChunkState.num);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
   }
 
   return NS_OK;
