@@ -1,4 +1,5 @@
 #filter substitution
+#include @OBJDIR@/api_keys
 
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -25,8 +26,8 @@ Cu.import("resource://gre/modules/Services.jsm");
 const PREF_APP_DISTRIBUTION           = "distribution.id";
 const PREF_APP_DISTRIBUTION_VERSION   = "distribution.version";
 
-XPCOMUtils.defineLazyModuleGetter(this, "UpdateChannel",
-                                  "resource://gre/modules/UpdateChannel.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "UpdateUtils",
+                                  "resource://gre/modules/UpdateUtils.jsm");
 
 function nsURLFormatterService() {
   XPCOMUtils.defineLazyGetter(this, "appInfo", function UFS_appInfo() {
@@ -87,30 +88,39 @@ nsURLFormatterService.prototype = {
   QueryInterface: XPCOMUtils.generateQI([Ci.nsIURLFormatter]),
 
   _defaults: {
-    LOCALE:           function() Cc["@mozilla.org/chrome/chrome-registry;1"].
-                                 getService(Ci.nsIXULChromeRegistry).
-                                 getSelectedLocale('global'),
-    VENDOR:           function() this.appInfo.vendor,
-    NAME:             function() this.appInfo.name,
-    ID:               function() this.appInfo.ID,
-    VERSION:          function() this.appInfo.version,
-    APPBUILDID:       function() this.appInfo.appBuildID,
-    PLATFORMVERSION:  function() this.appInfo.platformVersion,
-    PLATFORMBUILDID:  function() this.appInfo.platformBuildID,
-    APP:              function() this.appInfo.name.toLowerCase().replace(/ /, ""),
-    OS:               function() this.appInfo.OS,
-    XPCOMABI:         function() this.ABI,
-    BUILD_TARGET:     function() this.appInfo.OS + "_" + this.ABI,
-    OS_VERSION:       function() this.OSVersion,
-    CHANNEL:          function() UpdateChannel.get(),
-    MOZILLA_API_KEY:   function() "@MOZ_MOZILLA_API_KEY@",
-    GOOGLE_API_KEY:   function() "@MOZ_GOOGLE_API_KEY@",
-    GOOGLE_OAUTH_API_CLIENTID:function() "@MOZ_GOOGLE_OAUTH_API_CLIENTID@",
-    GOOGLE_OAUTH_API_KEY:     function() "@MOZ_GOOGLE_OAUTH_API_KEY@",
-    BING_API_CLIENTID:function() "@MOZ_BING_API_CLIENTID@",
-    BING_API_KEY:     function() "@MOZ_BING_API_KEY@",
-    DISTRIBUTION:     function() this.distribution.id,
-    DISTRIBUTION_VERSION: function() this.distribution.version
+    LOCALE:           () => Cc["@mozilla.org/chrome/chrome-registry;1"].
+                            getService(Ci.nsIXULChromeRegistry).
+                            getSelectedLocale('global'),
+    REGION:           function() {
+      try {
+        // When the geoip lookup failed to identify the region, we fallback to
+        // the 'ZZ' region code to mean 'unknown'.
+        return Services.prefs.getCharPref("browser.search.region") || "ZZ";
+      } catch(e) {
+        return "ZZ";
+      }
+    },
+    VENDOR:           function() { return this.appInfo.vendor; },
+    NAME:             function() { return this.appInfo.name; },
+    ID:               function() { return this.appInfo.ID; },
+    VERSION:          function() { return this.appInfo.version; },
+    APPBUILDID:       function() { return this.appInfo.appBuildID; },
+    PLATFORMVERSION:  function() { return this.appInfo.platformVersion; },
+    PLATFORMBUILDID:  function() { return this.appInfo.platformBuildID; },
+    APP:              function() { return this.appInfo.name.toLowerCase().replace(/ /, ""); },
+    OS:               function() { return this.appInfo.OS; },
+    XPCOMABI:         function() { return this.ABI; },
+    BUILD_TARGET:     function() { return this.appInfo.OS + "_" + this.ABI; },
+    OS_VERSION:       function() { return this.OSVersion; },
+    CHANNEL:          () => UpdateUtils.UpdateChannel,
+    MOZILLA_API_KEY:  () => "@MOZ_MOZILLA_API_KEY@",
+    GOOGLE_API_KEY:   () => "@MOZ_GOOGLE_API_KEY@",
+    GOOGLE_OAUTH_API_CLIENTID:() => "@MOZ_GOOGLE_OAUTH_API_CLIENTID@",
+    GOOGLE_OAUTH_API_KEY:     () => "@MOZ_GOOGLE_OAUTH_API_KEY@",
+    BING_API_CLIENTID:() => "@MOZ_BING_API_CLIENTID@",
+    BING_API_KEY:     () => "@MOZ_BING_API_KEY@",
+    DISTRIBUTION:     function() { return this.distribution.id; },
+    DISTRIBUTION_VERSION: function() { return this.distribution.version; }
   },
 
   formatURL: function uf_formatURL(aFormat) {
