@@ -48,7 +48,7 @@ function log2() {
   logbase.apply(window, arguments);
 }
 
-let reportError = log;
+var reportError = log;
 
 /*
  * wsBorder class
@@ -551,7 +551,8 @@ WidgetStack.prototype = {
     this.globalOffsetX += x;
     this.globalOffsetY += y;
 
-    for each (let state in this._widgetState) {
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       state.rect.x += x;
       state.rect.y += y;
 
@@ -600,7 +601,8 @@ WidgetStack.prototype = {
     //log2("setViewportBounds dltrb", dleft, dtop, dright, dbottom);
 
     // move all vp-relative widgets to be the right offset from the bounds again
-    for each (let state in this._widgetState) {
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       if (state.vpRelative) {
         //log2("vpRelative widget", state.id, state.rect.x, dleft, dright);
         if (state.vpOffsetXBefore) {
@@ -769,8 +771,10 @@ WidgetStack.prototype = {
 
     // XXX these methods aren't working correctly yet, but they aren't strictly
     // necessary in IceCatMobile's default config
-    //for each (let s in this._widgetState)
+    //for (let wid in this._widgetState) {
+    //  let s = this._widgetState[wid];
     //  this._updateWidgetRect(s);
+    //}
     //this._updateViewportOverflow();
 
     this._viewingRect.width = width;
@@ -1112,7 +1116,8 @@ WidgetStack.prototype = {
     // will be ignored in commitState.
     // The widget rects are in real stack space though, so we need to subtract
     // our (now negated) dx, dy from their coordinates.
-    for each (let state in this._widgetState) {
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       if (!state.ignoreX)
         state.rect.x -= dx;
       if (!state.ignoreY)
@@ -1175,7 +1180,7 @@ WidgetStack.prototype = {
 
     if (w.hasAttribute("constraint")) {
       let cs = w.getAttribute("constraint").split(",");
-      for each (let s in cs) {
+      for (let s of cs) {
         if (s == "ignore-x")
           state.ignoreX = true;
         else if (s == "ignore-y")
@@ -1239,7 +1244,8 @@ WidgetStack.prototype = {
 
     let ofRect = this._viewingRect.clone();
 
-    for each (let state in this._widgetState) {
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       if (vp && state.vpRelative) {
         // compute the vpOffset from 0,0 assuming that the viewport rect is 0,0
         if (state.rect.left >= vp.rect.right) {
@@ -1273,7 +1279,8 @@ WidgetStack.prototype = {
 
     let ofRect = new wsRect(0, 0, this._viewingRect.width, this._viewingRect.height);
 
-    for each (let state in this._widgetState) {
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       if (vp && state.vpRelative) {
         ofRect.left = Math.min(ofRect.left, state.rect.left);
         ofRect.top = Math.min(ofRect.top, state.rect.top);
@@ -1300,8 +1307,10 @@ WidgetStack.prototype = {
   _widgetBounds: function () {
     let r = new wsRect(0,0,0,0);
 
-    for each (let state in this._widgetState)
+    for (let wid in this._widgetState) {
+      let state = this._widgetState[wid];
       r = r.union(state.rect);
+    }
 
     return r;
   },
@@ -1390,7 +1399,7 @@ WidgetStack.prototype = {
 
     if (el.hasAttribute("constraint")) {
       let cs = el.getAttribute("constraint").split(",");
-      for each (let s in cs) {
+      for (let s of cs) {
         if (s == "ignore-x")
           barrier.ignoreX = true;
         else if (s == "ignore-y")
