@@ -1,14 +1,15 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-const {utils: Cu} = Components;
+var {utils: Cu} = Components;
 
 Cu.import("resource://gre/modules/NetUtil.jsm");
 Cu.import("resource://gre/modules/osfile.jsm");
 
+Cu.import("resource://gre/modules/Promise.jsm")
 Cu.import("resource://gre/modules/Log.jsm");
 
-let testFormatter = {
+var testFormatter = {
   format: function format(message) {
     return message.loggerName + "\t" +
       message.levelDesc + "\t" +
@@ -232,7 +233,7 @@ function fileContents(path) {
   });
 }
 
-add_task(function test_FileAppender() {
+add_task(function* test_FileAppender() {
   // This directory does not exist yet
   let dir = OS.Path.join(do_get_profile().path, "test_Log");
   do_check_false(yield OS.File.exists(dir));
@@ -286,7 +287,7 @@ add_task(function test_FileAppender() {
               "test.FileAppender\tINFO\t5\n");
 });
 
-add_task(function test_BoundedFileAppender() {
+add_task(function* test_BoundedFileAppender() {
   let dir = OS.Path.join(do_get_profile().path, "test_Log");
 
   if (!(yield OS.File.exists(dir))) {
@@ -337,7 +338,7 @@ add_task(function test_BoundedFileAppender() {
 /*
  * Test parameter formatting.
  */
-add_task(function log_message_with_params() {
+add_task(function* log_message_with_params() {
   let formatter = new Log.BasicFormatter();
 
   function formatMessage(text, params) {
@@ -428,16 +429,16 @@ add_task(function log_message_with_params() {
    */
   let err = Components.Exception("test exception", Components.results.NS_ERROR_FAILURE);
   let str = formatMessage("Exception is ${}", err);
-  do_check_true(str.contains('Exception is [Exception... "test exception"'));
-  do_check_true(str.contains("(NS_ERROR_FAILURE)"));
+  do_check_true(str.includes('Exception is [Exception... "test exception"'));
+  do_check_true(str.includes("(NS_ERROR_FAILURE)"));
   str = formatMessage("Exception is", err);
-  do_check_true(str.contains('Exception is: [Exception... "test exception"'));
+  do_check_true(str.includes('Exception is: [Exception... "test exception"'));
   str = formatMessage("Exception is ${error}", {error: err});
-  do_check_true(str.contains('Exception is [Exception... "test exception"'));
+  do_check_true(str.includes('Exception is [Exception... "test exception"'));
   str = formatMessage("Exception is", {_error: err});
   do_print(str);
   // Exceptions buried inside objects are formatted badly.
-  do_check_true(str.contains('Exception is: {"_error":{}'));
+  do_check_true(str.includes('Exception is: {"_error":{}'));
   // If the message text is null, the message contains only the formatted params object.
   str = formatMessage(null, err);
   do_check_true(str.startsWith('[Exception... "test exception"'));
@@ -486,7 +487,7 @@ add_task(function log_message_with_params() {
  * with the object argument as parameters. This makes the log useful when the
  * caller does "catch(err) {logger.error(err)}"
  */
-add_task(function test_log_err_only() {
+add_task(function* test_log_err_only() {
   let log = Log.repository.getLogger("error.only");
   let testFormatter = { format: msg => msg };
   let appender = new MockAppender(testFormatter);
@@ -512,7 +513,7 @@ add_task(function test_log_err_only() {
 /*
  * Test logStructured() messages through basic formatter.
  */
-add_task(function test_structured_basic() {
+add_task(function* test_structured_basic() {
   let log = Log.repository.getLogger("test.logger");
   let appender = new MockAppender(new Log.BasicFormatter());
 
@@ -524,20 +525,20 @@ add_task(function test_structured_basic() {
   // except the 'action' field is added to the object.
   log.logStructured("action", {data: "structure"});
   do_check_eq(appender.messages.length, 1);
-  do_check_true(appender.messages[0].contains('{"data":"structure","action":"action"}'));
+  do_check_true(appender.messages[0].includes('{"data":"structure","action":"action"}'));
 
   // A structured entry with _message and substitution is treated the same as
   // log./level/(null, params).
   log.logStructured("action", {_message: "Structured sub ${data}", data: "structure"});
   do_check_eq(appender.messages.length, 2);
   do_print(appender.messages[1]);
-  do_check_true(appender.messages[1].contains('Structured sub structure'));
+  do_check_true(appender.messages[1].includes('Structured sub structure'));
 });
 
 /*
  * Test that all the basic logger methods pass the message and params through to the appender.
  */
-add_task(function log_message_with_params() {
+add_task(function* log_message_with_params() {
   let log = Log.repository.getLogger("error.logger");
   let testFormatter = { format: msg => msg };
   let appender = new MockAppender(testFormatter);
@@ -561,25 +562,31 @@ add_task(function log_message_with_params() {
 /*
  * Check that we format JS Errors reasonably.
  */
-add_task(function format_errors() {
+add_task(function* format_errors() {
   let pFormat = new Log.ParameterFormatter();
 
   // Test that subclasses of Error are recognized as errors.
   err = new ReferenceError("Ref Error", "ERROR_FILE", 28);
   str = pFormat.format(err);
-  do_check_true(str.contains("ReferenceError"));
-  do_check_true(str.contains("ERROR_FILE:28"));
-  do_check_true(str.contains("Ref Error"));
+  do_check_true(str.includes("ReferenceError"));
+  do_check_true(str.includes("ERROR_FILE:28"));
+  do_check_true(str.includes("Ref Error"));
 
   // Test that JS-generated Errors are recognized and formatted.
   try {
+    yield Promise.resolve();  // Scrambles the stack
     eval("javascript syntax error");
   }
   catch (e) {
     str = pFormat.format(e);
-    do_check_true(str.contains("SyntaxError: missing ;"));
+    do_check_true(str.includes("SyntaxError: missing ;"));
     // Make sure we identified it as an Error and formatted the error location as
     // lineNumber:columnNumber.
-    do_check_true(str.contains(":1:11)"));
+    do_check_true(str.includes(":1:11)"));
+    // Make sure that we use human-readable stack traces
+    // Check that the error doesn't contain any reference to "Promise.jsm" or "Task.jsm"
+    do_check_false(str.includes("Promise.jsm"));
+    do_check_false(str.includes("Task.jsm"));
+    do_check_true(str.includes("format_errors"));
   }
 });
