@@ -3,9 +3,9 @@
  */
 
 // This verifies that add-ons can be installed from XPI files
-const Cc = Components.classes;
-const Ci = Components.interfaces;
-const Cu = Components.utils;
+var Cc = Components.classes;
+var Ci = Components.interfaces;
+var Cu = Components.utils;
 
 // Maximum error in file modification times. Some file systems don't store
 // modification times exactly. As long as we are closer than this then it
@@ -195,8 +195,9 @@ function check_test_1(installSyncGUID) {
           do_check_eq(a1.icon64URL, uri + "icon64.png");
 
           a1.uninstall();
+          let { id, version } = a1;
           restartManager();
-          do_check_not_in_crash_annotation(a1.id, a1.version);
+          do_check_not_in_crash_annotation(id, version);
 
           do_execute_soon(run_test_2);
         }));
@@ -647,6 +648,8 @@ function check_test_10(install) {
 // correct sourceURI.
 function run_test_11() {
   prepare_test({ }, [
+    "onNewInstall",
+    "onNewInstall",
     "onNewInstall",
     "onNewInstall",
     "onNewInstall",
@@ -657,11 +660,22 @@ function run_test_11() {
     ensure_test_completed();
     do_check_neq(install, null);
     do_check_neq(install.linkedInstalls, null);
-    do_check_eq(install.linkedInstalls.length, 3);
+    do_check_eq(install.linkedInstalls.length, 5);
 
     // Might be in any order so sort them based on ID
     let installs = [install].concat(install.linkedInstalls);
     installs.sort(function(a, b) {
+      if (a.state != b.state) {
+        if (a.state == AddonManager.STATE_DOWNLOAD_FAILED)
+          return 1;
+        else if (b.state == AddonManager.STATE_DOWNLOAD_FAILED)
+          return -1;
+      }
+
+      // Don't care what order the failed installs show up in
+      if (a.state == AddonManager.STATE_DOWNLOAD_FAILED)
+        return 0;
+
       if (a.addon.id < b.addon.id)
         return -1;
       if (a.addon.id > b.addon.id)
@@ -709,6 +723,12 @@ function run_test_11() {
     do_check_true(hasFlag(installs[3].addon.operationsRequiringRestart,
                           AddonManager.OP_NEEDS_RESTART_INSTALL));
 
+    do_check_eq(installs[4].state, AddonManager.STATE_DOWNLOAD_FAILED);
+    do_check_eq(installs[4].error, AddonManager.ERROR_CORRUPT_FILE);
+
+    do_check_eq(installs[5].state, AddonManager.STATE_DOWNLOAD_FAILED);
+    do_check_eq(installs[5].error, AddonManager.ERROR_CORRUPT_FILE);
+
     AddonManager.getAllInstalls(function(aInstalls) {
       do_check_eq(aInstalls.length, 4);
 
@@ -815,6 +835,8 @@ function run_test_12() {
         "onNewInstall",
         "onNewInstall",
         "onNewInstall",
+        "onNewInstall",
+        "onNewInstall",
         "onDownloadEnded"
       ],
       "addon4@tests.mozilla.org": [
@@ -839,11 +861,22 @@ function run_test_12() {
 }
 
 function check_test_12() {
-  do_check_eq(gInstall.linkedInstalls.length, 3);
+  do_check_eq(gInstall.linkedInstalls.length, 5);
 
   // Might be in any order so sort them based on ID
   let installs = [gInstall].concat(gInstall.linkedInstalls);
   installs.sort(function(a, b) {
+    if (a.state != b.state) {
+      if (a.state == AddonManager.STATE_DOWNLOAD_FAILED)
+        return 1;
+      else if (b.state == AddonManager.STATE_DOWNLOAD_FAILED)
+        return -1;
+    }
+
+    // Don't care what order the failed installs show up in
+    if (a.state == AddonManager.STATE_DOWNLOAD_FAILED)
+      return 0;
+
     if (a.addon.id < b.addon.id)
       return -1;
     if (a.addon.id > b.addon.id)
@@ -883,6 +916,12 @@ function check_test_12() {
   do_check_eq(installs[3].name, "Multi Test 4");
   do_check_eq(installs[3].state, AddonManager.STATE_INSTALLED);
 
+  do_check_eq(installs[4].state, AddonManager.STATE_DOWNLOAD_FAILED);
+  do_check_eq(installs[4].error, AddonManager.ERROR_CORRUPT_FILE);
+
+  do_check_eq(installs[5].state, AddonManager.STATE_DOWNLOAD_FAILED);
+  do_check_eq(installs[5].error, AddonManager.ERROR_CORRUPT_FILE);
+
   restartManager();
 
   AddonManager.getAddonsByIDs(["addon4@tests.mozilla.org",
@@ -1755,7 +1794,45 @@ function check_test_29(install) {
 
   prepare_test({}, [
     "onDownloadCancelled"
-  ], do_test_finished);
+  ], run_test_30);
   install.cancel();
   return false;
 }
+
+// Tests that a multi-package XPI with no add-ons inside shows up as a
+// corrupt file
+function run_test_30() {
+  prepare_test({ }, [
+    "onNewInstall"
+  ]);
+
+  AddonManager.getInstallForFile(do_get_addon("test_install7"), function(install) {
+    ensure_test_completed();
+
+    do_check_neq(install, null);
+    do_check_eq(install.state, AddonManager.STATE_DOWNLOAD_FAILED);
+    do_check_eq(install.error, AddonManager.ERROR_CORRUPT_FILE);
+    do_check_eq(install.linkedInstalls, null);
+
+    run_test_31();
+  });
+}
+
+// Tests that a multi-package XPI with no valid add-ons inside shows up as a
+// corrupt file
+function run_test_31() {
+  prepare_test({ }, [
+    "onNewInstall"
+  ]);
+
+  AddonManager.getInstallForFile(do_get_addon("test_install8"), function(install) {
+    ensure_test_completed();
+
+    do_check_neq(install, null);
+    do_check_eq(install.state, AddonManager.STATE_DOWNLOAD_FAILED);
+    do_check_eq(install.error, AddonManager.ERROR_CORRUPT_FILE);
+    do_check_eq(install.linkedInstalls, null);
+
+    end_test();
+  });
+}
