@@ -373,6 +373,11 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
         case eIntID_ScrollbarButtonAutoRepeatBehavior:
             aResult = 0;
             break;
+        
+        case eIntID_ContextMenuOffsetVertical:
+        case eIntID_ContextMenuOffsetHorizontal:
+            aResult = 2;
+            break;
 
         default:
             aResult = 0;
