@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 et cindent: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -310,7 +310,7 @@ nsWindowsRegKey::ReadStringValue(const nsAString& aName, nsAString& aResult)
 
   // This must be a string type in order to fetch the value as a string.
   // We're being a bit forgiving here by allowing types other than REG_SZ.
-  if (type != REG_SZ && type == REG_EXPAND_SZ && type == REG_MULTI_SZ) {
+  if (type != REG_SZ && type != REG_EXPAND_SZ && type != REG_MULTI_SZ) {
     return NS_ERROR_FAILURE;
   }
 
@@ -559,7 +559,7 @@ nsWindowsRegKey::IsWatching(bool* aResult)
 nsresult
 NS_NewWindowsRegKey(nsIWindowsRegKey** aResult)
 {
-  nsRefPtr<nsWindowsRegKey> key = new nsWindowsRegKey();
+  RefPtr<nsWindowsRegKey> key = new nsWindowsRegKey();
   key.forget(aResult);
   return NS_OK;
 }
