@@ -28,6 +28,7 @@
 
 #include "nsDebug.h" // for |NS_ASSERTION|
 #include "nsISupportsUtils.h" // for |nsresult|, |NS_ADDREF|, |NS_GET_TEMPLATE_IID| et al
+#include "mozilla/RefPtr.h"
 
 #include "nsCycleCollectionNoteChild.h"
 
@@ -109,9 +110,7 @@
 #endif
 
 namespace mozilla {
-
-struct unused_t;
-
+template<class T> class OwningNonNull;
 } // namespace mozilla
 
 template<class T>
@@ -169,7 +168,7 @@ private:
   nsISupports* MOZ_OWNING_REF mRawPtr;
 };
 
-class nsQueryInterfaceWithError
+class nsQueryInterfaceWithError final
 {
 public:
   nsQueryInterfaceWithError(nsISupports* aRawPtr, nsresult* aError)
@@ -218,7 +217,7 @@ do_QueryInterface(already_AddRefed<T>&, nsresult*)
 
 ////////////////////////////////////////////////////////////////////////////
 // Using servicemanager with COMPtrs
-class nsGetServiceByCID
+class nsGetServiceByCID final
 {
 public:
   explicit nsGetServiceByCID(const nsCID& aCID) : mCID(aCID) {}
@@ -229,7 +228,7 @@ private:
   const nsCID& mCID;
 };
 
-class nsGetServiceByCIDWithError
+class nsGetServiceByCIDWithError final
 {
 public:
   nsGetServiceByCIDWithError(const nsCID& aCID, nsresult* aErrorPtr)
@@ -245,7 +244,7 @@ private:
   nsresult* mErrorPtr;
 };
 
-class nsGetServiceByContractID
+class nsGetServiceByContractID final
 {
 public:
   explicit nsGetServiceByContractID(const char* aContractID)
@@ -259,7 +258,7 @@ private:
   const char* mContractID;
 };
 
-class nsGetServiceByContractIDWithError
+class nsGetServiceByContractIDWithError final
 {
 public:
   nsGetServiceByContractIDWithError(const char* aContractID, nsresult* aErrorPtr)
@@ -340,6 +339,12 @@ protected:
 
 // template<class T> class nsGetterAddRefs;
 
+// Helper for assert_validity method
+template<class T>
+char (&TestForIID(decltype(&NS_GET_TEMPLATE_IID(T))))[2];
+template<class T>
+char TestForIID(...);
+
 template<class T>
 class nsCOMPtr final
 #ifdef NSCAP_FEATURE_USE_BASE
@@ -380,6 +385,15 @@ private:
   T* MOZ_OWNING_REF mRawPtr;
 #endif
 
+  void assert_validity()
+  {
+    static_assert(1 < sizeof(TestForIID<T>(nullptr)), "nsCOMPtr only works "
+                  "for types with IIDs.  Either use RefPtr; add an IID to "
+                  "your type with NS_DECLARE_STATIC_IID_ACCESSOR/"
+                  "NS_DEFINE_STATIC_IID_ACCESSOR; or make the nsCOMPtr point "
+                  "to a base class with an IID.");
+  }
+
 public:
   typedef T element_type;
 
@@ -413,12 +427,14 @@ public:
   nsCOMPtr()
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
   }
 
   nsCOMPtr(const nsCOMPtr<T>& aSmartPtr)
     : NSCAP_CTOR_BASE(aSmartPtr.mRawPtr)
   {
+    assert_validity();
     if (mRawPtr) {
       NSCAP_ADDREF(this, mRawPtr);
     }
@@ -428,6 +444,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(T* aRawPtr)
     : NSCAP_CTOR_BASE(aRawPtr)
   {
+    assert_validity();
     if (mRawPtr) {
       NSCAP_ADDREF(this, mRawPtr);
     }
@@ -438,6 +455,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<T>& aSmartPtr)
     : NSCAP_CTOR_BASE(aSmartPtr.take())
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
     NSCAP_ASSERT_NO_QUERY_NEEDED();
   }
@@ -446,6 +464,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<T>&& aSmartPtr)
     : NSCAP_CTOR_BASE(aSmartPtr.take())
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, mRawPtr);
     NSCAP_ASSERT_NO_QUERY_NEEDED();
   }
@@ -455,6 +474,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<U>& aSmartPtr)
     : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take()))
   {
+    assert_validity();
     // But make sure that U actually inherits from T.
     static_assert(mozilla::IsBaseOf<T, U>::value,
                   "U is not a subclass of T");
@@ -467,6 +487,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(already_AddRefed<U>&& aSmartPtr)
     : NSCAP_CTOR_BASE(static_cast<T*>(aSmartPtr.take()))
   {
+    assert_validity();
     // But make sure that U actually inherits from T.
     static_assert(mozilla::IsBaseOf<T, U>::value,
                   "U is not a subclass of T");
@@ -478,6 +499,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsQueryInterface aQI)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_qi(aQI, NS_GET_TEMPLATE_IID(T));
   }
@@ -486,6 +508,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsQueryInterfaceWithError& aQI)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_qi_with_error(aQI, NS_GET_TEMPLATE_IID(T));
   }
@@ -494,6 +517,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCID aGS)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_gs_cid(aGS, NS_GET_TEMPLATE_IID(T));
   }
@@ -502,6 +526,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByCIDWithError& aGS)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_gs_cid_with_error(aGS, NS_GET_TEMPLATE_IID(T));
   }
@@ -510,6 +535,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractID aGS)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_gs_contractid(aGS, NS_GET_TEMPLATE_IID(T));
   }
@@ -518,6 +544,7 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsGetServiceByContractIDWithError& aGS)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_gs_contractid_with_error(aGS, NS_GET_TEMPLATE_IID(T));
   }
@@ -527,11 +554,16 @@ public:
   MOZ_IMPLICIT nsCOMPtr(const nsCOMPtr_helper& aHelper)
     : NSCAP_CTOR_BASE(0)
   {
+    assert_validity();
     NSCAP_LOG_ASSIGNMENT(this, 0);
     assign_from_helper(aHelper, NS_GET_TEMPLATE_IID(T));
     NSCAP_ASSERT_NO_QUERY_NEEDED();
   }
 
+  // Defined in OwningNonNull.h
+  template<class U>
+  MOZ_IMPLICIT nsCOMPtr(const mozilla::OwningNonNull<U>& aOther);
+
 
   // Assignment operators
 
@@ -623,6 +655,10 @@ public:
     return *this;
   }
 
+  // Defined in OwningNonNull.h
+  template<class U>
+  nsCOMPtr<T>& operator=(const mozilla::OwningNonNull<U>& aOther);
+
   // Exchange ownership with |aRhs|; can save a pair of refcount operations.
   void swap(nsCOMPtr<T>& aRhs)
   {
@@ -1314,40 +1350,34 @@ operator!=(U* aLhs, const nsCOMPtr<T>& aRhs)
 
 
 
-// Comparing an |nsCOMPtr| to |0|
+// Comparing an |nsCOMPtr| to |nullptr|
 
-class NSCAP_Zero;
-
-// Specifically to allow |smartPtr == 0|.
 template<class T>
 inline bool
-operator==(const nsCOMPtr<T>& aLhs, NSCAP_Zero* aRhs)
+operator==(const nsCOMPtr<T>& aLhs, decltype(nullptr))
 {
-  return static_cast<const void*>(aLhs.get()) == reinterpret_cast<const void*>(aRhs);
+  return aLhs.get() == nullptr;
 }
 
-// Specifically to allow |0 == smartPtr|.
 template<class T>
 inline bool
-operator==(NSCAP_Zero* aLhs, const nsCOMPtr<T>& aRhs)
+operator==(decltype(nullptr), const nsCOMPtr<T>& aRhs)
 {
-  return reinterpret_cast<const void*>(aLhs) == static_cast<const void*>(aRhs.get());
+  return nullptr == aRhs.get();
 }
 
-// Specifically to allow |smartPtr != 0|.
 template<class T>
 inline bool
-operator!=(const nsCOMPtr<T>& aLhs, NSCAP_Zero* aRhs)
+operator!=(const nsCOMPtr<T>& aLhs, decltype(nullptr))
 {
-  return static_cast<const void*>(aLhs.get()) != reinterpret_cast<const void*>(aRhs);
+  return aLhs.get() != nullptr;
 }
 
-// Specifically to allow |0 != smartPtr|.
 template<class T>
 inline bool
-operator!=(NSCAP_Zero* aLhs, const nsCOMPtr<T>& aRhs)
+operator!=(decltype(nullptr), const nsCOMPtr<T>& aRhs)
 {
-  return reinterpret_cast<const void*>(aLhs) != static_cast<const void*>(aRhs.get());
+  return nullptr != aRhs.get();
 }
 
 
@@ -1369,4 +1399,27 @@ CallQueryInterface(nsCOMPtr<SourceType>& aSourcePtr, DestinationType** aDestPtr)
   return CallQueryInterface(aSourcePtr.get(), aDestPtr);
 }
 
+template <class T>
+RefPtr<T>::RefPtr(const nsCOMPtr_helper& aHelper)
+{
+  void* newRawPtr;
+  if (NS_FAILED(aHelper(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+    newRawPtr = 0;
+  }
+  mRawPtr = static_cast<T*>(newRawPtr);
+}
+
+template <class T>
+RefPtr<T>&
+RefPtr<T>::operator=(const nsCOMPtr_helper& aHelper)
+{
+  void* newRawPtr;
+  if (NS_FAILED(aHelper(NS_GET_TEMPLATE_IID(T), &newRawPtr))) {
+    newRawPtr = 0;
+  }
+  assign_assuming_AddRef(static_cast<T*>(newRawPtr));
+  return *this;
+}
+
+
 #endif // !defined(nsCOMPtr_h___)
