@@ -26,7 +26,7 @@ CopyASCIItoUTF16(const nsACString& aSource, nsAString& aDest)
 }
 
 void
-LossyCopyUTF16toASCII(const char16_t* aSource, nsACString& aDest)
+LossyCopyUTF16toASCII(const char16ptr_t aSource, nsACString& aDest)
 {
   aDest.Truncate();
   if (aSource) {
@@ -72,7 +72,7 @@ CopyUTF8toUTF16(const nsACString& aSource, nsAString& aDest)
 }
 
 void
-CopyUTF16toUTF8(const char16_t* aSource, nsACString& aDest)
+CopyUTF16toUTF8(const char16ptr_t aSource, nsACString& aDest)
 {
   aDest.Truncate();
   AppendUTF16toUTF8(aSource, aDest);
@@ -139,7 +139,7 @@ AppendASCIItoUTF16(const nsACString& aSource, nsAString& aDest,
 }
 
 void
-LossyAppendUTF16toASCII(const char16_t* aSource, nsACString& aDest)
+LossyAppendUTF16toASCII(const char16ptr_t aSource, nsACString& aDest)
 {
   if (aSource) {
     LossyAppendUTF16toASCII(nsDependentString(aSource), aDest);
@@ -255,7 +255,7 @@ AppendUTF8toUTF16(const nsACString& aSource, nsAString& aDest,
 }
 
 void
-AppendUTF16toUTF8(const char16_t* aSource, nsACString& aDest)
+AppendUTF16toUTF8(const char16ptr_t aSource, nsACString& aDest)
 {
   if (aSource) {
     AppendUTF16toUTF8(nsDependentString(aSource), aDest);
@@ -275,7 +275,7 @@ AppendUTF8toUTF16(const char* aSource, nsAString& aDest)
  * A helper function that allocates a buffer of the desired character type big enough to hold a copy of the supplied string (plus a zero terminator).
  *
  * @param aSource an string you will eventually be making a copy of
- * @return a new buffer (of the type specified by the second parameter) which you must free with |nsMemory::Free|.
+ * @return a new buffer (of the type specified by the second parameter) which you must free with |free|.
  *
  */
 template <class FromStringT, class ToCharT>
@@ -283,7 +283,7 @@ inline
 ToCharT*
 AllocateStringCopy(const FromStringT& aSource, ToCharT*)
 {
-  return static_cast<ToCharT*>(nsMemory::Alloc(
+  return static_cast<ToCharT*>(moz_xmalloc(
     (aSource.Length() + 1) * sizeof(ToCharT)));
 }
 
@@ -316,7 +316,7 @@ ToNewUTF8String(const nsAString& aSource, uint32_t* aUTF8Count)
   }
 
   char* result = static_cast<char*>
-                 (nsMemory::Alloc(calculator.Size() + 1));
+                 (moz_xmalloc(calculator.Size() + 1));
   if (!result) {
     return nullptr;
   }
@@ -408,7 +408,7 @@ UTF8ToNewUnicode(const nsACString& aSource, uint32_t* aUTF16Count)
 {
   const uint32_t length = CalcUTF8ToUnicodeLength(aSource);
   const size_t buffer_size = (length + 1) * sizeof(char16_t);
-  char16_t* buffer = static_cast<char16_t*>(nsMemory::Alloc(buffer_size));
+  char16_t* buffer = static_cast<char16_t*>(moz_xmalloc(buffer_size));
   if (!buffer) {
     return nullptr;
   }
